/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  NB IoT Measurements
 *
 *  Original Release: December 2017
 *  By: Martin Krcmar
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/

#include <string.h>
#include <stdio.h>
#include <stdlib.h>
#include "rscmwinm.h"
#include "rscmwinm_utility.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
    #define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
    #include <windows.h>
#endif
#if defined (_CVI_)
    #include <utility.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/

//static ViString switchArr[] = {"OFF","ON", NULL};
static ViString analyzerSalScenRXCArr[] = { "RF1C","RF2C","RF3C","RF4C","RFAC","RFBC",
                                            "R11","R12","R13","R14","R15","R16","R17","R18",
                                            "R21","R22","R23","R24","R25","R26","R27","R28",
                                            "R31","R32","R33","R34","R35","R36","R37","R38",
                                            "R41","R42","R43","R44","R45","R46","R47","R48",
                                            "RA1","RA2","RA3","RA4","RA5","RA6","RA7","RA8",
                                            "RB1","RB2","RB3","RB4","RB5","RB6","RB7","RB8",
                                            "R11C","R12C","R13C","R14C",
                                            "R21C","R22C","R23C","R24C",
                                            "R31C","R32C","R33C","R34C",
                                            "R41C","R42C","R43C","R44C", NULL};
static ViString analyzerSalScenRFCArr[] = { "RX1", "RX2", "RX3", "RX4",
                                            "RX11","RX12","RX13","RX14",
                                            "RX21","RX22","RX23","RX24",
                                            "RX31","RX32","RX33","RX34",
                                            "RX41","RX42","RX43","RX44", NULL};
static ViString routingScenarioArr[] = {"SAL", "MAPR","CSP", NULL};
static ViString resultTypeArr[] = {"CURR","AVER","MAX","MIN","EXTR","SDEV", NULL};
static ViString modSchemeArr[] = {"BPSK","QPSK","Q16", NULL};
static ViString bandArr[] = {"","OB1","OB2","OB3","OB4","OB5","OB7","OB8","OB9","OB10",
                             "OB11","OB12","OB13","OB14","OB17","OB18","OB19","OB20",
                             "OB21","OB22","OB24","OB25","OB26","OB27","OB28","OB30",
                             "OB31","OB65","OB66","OB68","OB70","OB71","OB72","OB73","OB74","OB85", NULL};
static ViString NPUSChFormatArr[] = {"F1","F2", NULL};
static ViString resourceUnitsArr[] = {"NRU01","NRU02","NRU03","NRU04","NRU05","NRU06","NRU08","NRU10", NULL};
static ViString repetitionsArr[] = {"NR1","NR2","NR4","NR8","NR16","NR32","NR64","NR128","NR256","NR512","NR1K","NR2K", NULL};
static ViString retriggerFlagArr[] = {"OFF","ON","IFP", NULL};

/*****************************************************************************
 * Function: rscmwinm_init
 * Purpose:  VXIplug&play required function. Calls the
 *           rscmwinm_InitWithOptions function.
 *****************************************************************************/

ViStatus _VI_FUNC rscmwinm_init (ViRsrc resourceName, ViBoolean IDQuery,
                                   ViBoolean resetDevice, ViSession *newInstrSession)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rscmwinm_InitWithOptions (resourceName, IDQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}

/*****************************************************************************
 * Function: rscmwinm_InitWithOptions
 * Purpose:  This function creates a new RS session and calls the
 *           RsInit function.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_InitWithOptions(
    ViRsrc          resourceName,
    ViBoolean       IDQuery,
    ViBoolean       resetDevice,
    ViString        optionString,
    ViSession      *newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr *)rscmwinm_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rscmwinm", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSCMWINM_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 3.12.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 3, 12, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rscmwinm_RsCoreRepCapTable));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSCMWINM_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSCMWINM_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rscmwinm_reset(instrSession));
    }
    else
    {
        checkErr(rscmwinm_DefaultInstrSetup(instrSession));
    }

    if (IDQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSCMWINM_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rscmwinm_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}

/// HIFN  This function activates the standalone scenario and selects the input
/// HIFN  connector for the measured RF signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR rxConnector/Defines the RX Connector.
/// HIPAR rfConverter/Defines the RF Converter.
ViStatus _VI_FUNC rscmwinm_ConfigureAnalyzerStandAloneScenario(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   rxConnector,
                            ViInt32   rfConverter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxConnector, RSCMWINM_VAL_ANALYZER_CONNECTOR_RF1C, RSCMWINM_VAL_ANALYZER_CONNECTOR_R44C),
            3, "RX Connector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rfConverter, RSCMWINM_VAL_ANALYZER_SAL_CONV_RX1, RSCMWINM_VAL_ANALYZER_SAL_CONV_RX44),
            4, "RF Convertor");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ROUT:NIOT:MEAS%ld:SCEN:SAL %s,%s", instance, analyzerSalScenRXCArr[rxConnector], analyzerSalScenRFCArr[rfConverter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function activates the Measure ad ProtocolTest scenario. The signal
/// HIFN  routing and analyzer settings are ignored by the measurement
/// HIFN  application. The corresponding settings have to be configured within
/// HIFN  the protocol test application used in parallel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR controller/Permits selecting the protocol test application.
ViStatus _VI_FUNC rscmwinm_AnalyzerMeasureProtocolTestScenario(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViString  controller)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MEASURE_PROTOCOL_TEST, controller));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function queries the configured routing settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSCMWINM_init or
/// HIPAR instrSession/RSCMWINM_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR scenario/Returns used scenario.
/// HIPAR controller/Returns controlling application for scenario MAPRotocol
/// HIPAR rxConnector/Returns the RX Connector.
/// HIPAR rfConverter/Returns the RF Converter.
ViStatus _VI_FUNC rscmwinm_QueryAnalyzerRoutingSettings(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32*  scenario,
                                                        ViChar    controller[],
                                                        ViInt32*  rxConnector,
                                                        ViInt32*  rfConverter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   *ext_buf = NULL;
    ViChar   *p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, controller), 4, "Controller");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ROUT:NIOT:MEAS%ld?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    sscanf (ext_buf, "%[^\r\n]", ext_buf);

    p2buf = strtok (ext_buf, ",");
    checkErr(rscmwinm_GetIndexOfToken(p2buf, routingScenarioArr, (ViUInt32 *)scenario));
    checkErr(rscmwinm_CheckStatus(instrSession));

    p2buf = strtok (NULL, ",");
    strcpy (controller, p2buf);

    p2buf = strtok (NULL, ",");
    checkErr(rscmwinm_GetIndexOfToken(p2buf, analyzerSalScenRXCArr, (ViUInt32 *) rxConnector));

    p2buf = strtok (NULL, ",");
    checkErr(rscmwinm_GetIndexOfToken(p2buf, analyzerSalScenRFCArr, (ViUInt32 *) rfConverter));

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures external attenuation (or gain, if the value
/// HIFN  is negative), to be applied to the selected RF input connector.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR externalAttenuation/Defines an external attenuation (or gain, if the value is negative),
/// HIPAR externalAttenuation/to be applied to the selected RF input connector.
ViStatus _VI_FUNC rscmwinm_ConfigureAnalyzerExternalAttenuation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64  externalAttenuation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_INPUT_EXTERNAL_ATTENUTATION, externalAttenuation),
            3, "External Attenuation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RF input path.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR expectedNominalPower/Sets the expected nominal power of the measured RF signal.
/// HIPAR margin/Sets the margin that the R&S CMW adds to the expected nominal power
/// HIPAR margin/minus the external input attenuation in order to determine its
/// HIPAR margin/reference power. The reference power must be within the power range of
/// HIPAR margin/the selected input connector; refer to the "Specifications".
/// HIPAR centerFrequency/Selects the center frequency of the RF analyzer.
/// HIPAR frequencyOffset/Specifies a positive or negative frequency offset to be added to the
/// HIPAR frequencyOffset/carrier center frequency.
/// HIPAR mixerLevelOffset/Selects a positive or negative offset frequency to be added to the
/// HIPAR mixerLevelOffset/center frequency.
ViStatus _VI_FUNC rscmwinm_ConfigureAnalyzer(ViSession instrSession,
                                             ViInt32   instance,
                                             ViReal64  expectedNominalPower,
                                             ViReal64  margin,
                                             ViReal64  centerFrequency,
                                             ViReal64  frequencyOffset,
                                             ViReal64  mixerLevelOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_EXPECTED_NOMINAL_POWER, expectedNominalPower),
            3, "Expected Nominal Power");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_USER_MARGIN, margin),
            4, "Margin");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_CENTER_FREQUENCY, centerFrequency),
            5, "Center Frequency");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_FREQUENCY_OFFSET, frequencyOffset),
            6, "Frequency Offset");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_MIXER_LEVEL_OFFSET, mixerLevelOffset),
            7, "Mixer Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the operating band (OB). For supported bands see the operating
/// HIFN  manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR band/Selects the operating band (OB). For supported bands see the operating
/// HIPAR band/manual.
ViStatus _VI_FUNC rscmwinm_ConfigureAnalyzerBand(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   band)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_ANALYZER_OPERATING_BAND, band),
            3, "Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the combined signal path scenario and selects the master
/// HIFN  application and carrier. The master controls most signal routing
/// HIFN  settings, analyzer settings and some measurement control settings while
/// HIFN  the combined signal path scenario is active.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR master/Activates the combined signal path scenario and selects the master
/// HIPAR master/application and carrier. The master controls most signal routing
/// HIPAR master/settings, analyzer settings and some measurement control settings while
/// HIPAR master/the combined signal path scenario is active.
/// HIPAR carrier/For future use.
ViStatus _VI_FUNC rscmwinm_ConfigureAnalyzerScenario(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViString  master,
                                                     ViInt32   carrier)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             carrier;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ROUT:NIOT:MEAS%ld:SCEN:CSP '%s'", instance, master);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the scope of the multi-evaluation
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR repetition/Specifies the repetition mode of the measurement; see Statistical
/// HIPAR repetition/Settings. The repetition mode specifies whether the measurement is
/// HIPAR repetition/stopped after a single shot or repeated continuously.
/// HIPAR stopCondition/Qualifies whether the measurement is stopped after a failed limit
/// HIPAR stopCondition/check or continued.
/// HIPAR measureonException/Specifies whether measurement results that the R&S CMW identifies as
/// HIPAR measureonException/faulty or inaccurate are rejected.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasStatistics(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   repetition,
                                                        ViInt32   stopCondition,
                                                        ViBoolean measureonException)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_REPETITION_MODE, repetition),
            3, "Repetition");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_STOP_CONDITION, stopCondition),
            4, "Stop Condition");

    viCheckParm(rscmwinm_SetAttributeViBoolean(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_MEASURE_ON_EXCEPTION, measureonException),
            5, "Measure on Exception");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the statistics count. The statistic count is
/// HIFN  the integer number of measurement intervals per measurement cycle
/// HIFN  (statistics cycle, single-shot measurement). The length of a
/// HIFN  measurement interval is measurement-specific.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Specifies the measurement.
/// HIPAR statisticCount/Specifies the statistic count of the measurement; see Statistical
/// HIPAR statisticCount/Settings. The statistic count is equal to the number of measurement
/// HIPAR statisticCount/intervals per single shot.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasStatisticsCount(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   measurement,
                                                             ViInt32   statisticCount)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_MODULATION_STATISTIC_COUNT,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_SPECTRUM_EMISSION_MASK_STATISTIC_COUNT,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_SPECTRUM_ACLR_STATISTIC_COUNT,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_POWER_STATISTIC_COUNT};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWINM_VAL_MOD, RSCMWINM_VAL_POWER),
            3, "Measurement");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, attr[measurement], statisticCount),
            4, "Statistic Count");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures a timeout for the measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeout/Defines a timeout for the measurement.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasTimeout(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViReal64  timeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function select the evaluated results and the displayed views.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/Specifies the measurement.
/// HIPAR state/Sets the result view state.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasResults(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   measurement,
                                                     ViBoolean state)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   attr[] = {RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_EVM,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_MAGNITUDE_ERROR,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_PHASE_ERROR,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_INBAND_EMISSIONS,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_IQ_CONSTELLATION_DIAGRAM,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_TX_MEASUREMENT,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_SPECTRUM_EMISSION_MASK,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_ACLR,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_POWER_MONITOR,
             RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_RESULT_POWER_DYNAMICS};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWINM_VAL_RES_EVM, RSCMWINM_VAL_RES_PDYN),
            3, "Measurement");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, attr[measurement], state),
            4, "State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the evaluation of results and shows or hides the
/// HIFN  views in the multi evaluation measurement
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Defines size of array in argument 'States'.
/// HIPAR states/Enables or disables the evaluation of results and shows or hides the
/// HIPAR states/views in the multi evaluation measurement.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasResultsAll(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   arraySize,
                                                        ViBoolean states[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:RES:ALL ", instance);
    checkErr(RsCore_WriteAsciiViBooleanArray(instrSession, cmd, states, arraySize));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the scope of the multi-evaluation
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurementMode/
/// HIPAR numberOfMeasureSlots/Configures the length of the captured sequence of slots.
/// HIPAR subcarrierSpacing/Selects the subcarrier spacing, 3.75 kHz or 15 kHz. Set this parameter
/// HIPAR subcarrierSpacing/in accordance with your measured signal.
/// HIPAR npuschFormat/Specifies the format of the NPUSCH.
/// HIPAR physicalCellID/Specifies the physical layer cell ID.
/// HIPAR deltaSequenceShift/Specifies the delta sequence shift value (delta ss) used to calculate
/// HIPAR deltaSequenceShift/the sequence shift pattern for the NPUSCH.
/// HIPAR modulationScheme/Selects the modulation scheme used by the measured signal.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasParameters(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   measurementMode,
                                                        ViInt32   numberOfMeasureSlots,
                                                        ViInt32   subcarrierSpacing,
                                                        ViInt32   npuschFormat,
                                                        ViInt32   physicalCellID,
                                                        ViInt32   deltaSequenceShift,
                                                        ViInt32   modulationScheme)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_MEAS_MODE, measurementMode),
            3, "Measurement Mode");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NUMBER_OF_MEASURE_SLOTS, numberOfMeasureSlots),
            4, "Number Of Measure Slots");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_SUBCARRIER_SPACING, subcarrierSpacing),
            5, "Subcarrier Spacing");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NPUSCH_FORMAT, npuschFormat),
            6, "NPUSCH Format");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_PHYSICAL_CELL_ID, physicalCellID),
            7, "Physical Cell ID");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_DELTA_SEQUENCE_SHIFT, deltaSequenceShift),
            8, "Delta Sequence Shift");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_MODULATION_SCHEME, modulationScheme),
            9, "Modulation Scheme");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Defines an exclusion period at the beginning of each NPUSCH
/// HIFN  transmission. The excluded symbols are ignored for modulation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR leading/Defines an exclusion period at the beginning of each NPUSCH
/// HIPAR leading/transmission. The excluded symbols are ignored for modulation results.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasNPUSChLeading(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   leading)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LEADING, leading),
            3, "Leading");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines an exclusion period at the end of each NPUSCH transmission.
/// HIFN  The excluded symbols are ignored for modulation results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR lagging/Defines an exclusion period at the end of each NPUSCH  transmission.
/// HIPAR lagging/The excluded symbols are ignored for modulation results.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasNPUSChLagging(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   lagging)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LAGGING, lagging),
            3, "Lagging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Sets the mode of occupied bandwidth measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mode/Sets the mode of occupied bandwidth measurement.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasOBWMode(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_OBW_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the frequency synchronization range.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR freqSyncRange/Specifies the frequency synchronization range.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasFreqSyncRange(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   freqSyncRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_FSR, freqSyncRange),
            3, "Freq Sync Range");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configure the settings according to the resource allocation of the
/// HIFN  measured signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR numberOfSubcarriers/Number of subcarriers per resource unit.
/// HIPAR offset/Offset of the first allocated subcarrier from the edge of the transmission bandwidth.
/// HIPAR numberOfResourceUnits/Specifies the number of resource units allocated for the NPUSCH.
/// HIPAR numberOfRepetitions/Specifies the number of NPUSCH repetitions.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasResourceAllocation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   numberOfSubcarriers,
                            ViInt32   offset,
                            ViInt32   numberOfResourceUnits,
                            ViInt32   numberOfRepetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, numberOfSubcarriers, 1, 12),
            3, "Number Of Subcarriers");
    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:SUBC %ld,%ld", instance, numberOfSubcarriers, offset);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwinm_CheckStatus(instrSession));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NUMBER_OF_RESOURCE_UNITS, numberOfResourceUnits),
            5, "Number Of Resource Units");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NUMBER_OF_REPETITIONS, numberOfRepetitions),
            6, "Number Of Repetitions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the evaluation of the adjacent UTRA channels and
/// HIFN  the adjacent GSM channels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR utra/Specifies whether measurement results that the R&S CMW identifies as
/// HIPAR utra/faulty or inaccurate are rejected.
/// HIPAR gsm/Specifies whether measurement results that the R&S CMW identifies as
/// HIPAR gsm/faulty or inaccurate are rejected.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasSelectSpectrumACLR(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean utra,
                            ViBoolean gsm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:SPEC:ACLR:ENAB %s,%s", instance, utra?"ON":"OFF", gsm?"ON":"OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the evaluation of the adjacent UTRA channels and
/// HIFN  the adjacent GSM channels.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     no attribute
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:UL[:SET<setNr>]
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR set/For future use.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Sets the start subcarrier.
/// HIPAR mcsIndex/MCS Index Value.
/// HIPAR resourceUnits/Number of resource units allocated for the NPUSCH.
/// HIPAR repetitions/Number of NPUSCH repetitions.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasUserDefinedUplink(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   set,
                            ViInt32   subcarriers,
                            ViInt32   startSubcarrier,
                            ViInt32   mcsIndex,
                            ViInt32   resourceUnits,
                            ViInt32   repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
             set;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, 1, 12),
            4, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startSubcarrier, 0, 47),
            5, "Start Subcarrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resourceUnits, RSCMWINM_VAL_RESOURCE_UNIT_NRU01, RSCMWINM_VAL_RESOURCE_UNIT_NRU10),
            6, "Resource Units");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSCMWINM_VAL_REPETITIONS_1, RSCMWINM_VAL_REPETITIONS_128),
            7, "Repetitions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:CONN:UDEF:UL SC%ld,MI%ld,%ld,%s,%s",
            instance, subcarriers, startSubcarrier, mcsIndex, resourceUnitsArr[resourceUnits], repetitionsArr[repetitions]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the trigger parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR source/Selects the source of the trigger events. Some values are always
/// HIPAR source/available in this firmware application. They are listed below.
/// HIPAR source/Depending on the installed options additional values may be available.
/// HIPAR source/A complete list of all currently supported values can be displayed
/// HIPAR source/using TRIGger:...:CATalog:SOURCe.
/// HIPAR slope/Qualifies whether the trigger event is generated at the rising or at
/// HIPAR slope/the falling edge of the trigger pulse (valid for external and power
/// HIPAR slope/trigger sources).
/// HIPAR threshold/Defines the trigger threshold for the power trigger sources.
/// HIPAR delay/Defines a time delaying the start of the measurement relative to the
/// HIPAR delay/trigger event. This setting has no influence on free run measurements.
/// HIPAR timeoutStatus/Enables or disables timeout.
/// HIPAR timeout/Selects the maximum time that the R&S CMW waits for a trigger event
/// HIPAR timeout/before it stops the measurement in remote control mode or indicates a
/// HIPAR timeout/trigger timeout in manual operation mode. This setting has no influence
/// HIPAR timeout/on 'Free Run' measurements.
/// HIPAR minTriggerGap/Sets a minimum time during which the IF signal must be below the
/// HIPAR minTriggerGap/trigger threshold before the trigger is armed so that an IF power
/// HIPAR minTriggerGap/trigger event can be generated.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasTrigger(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViString  source,
                                                     ViInt32   slope,
                                                     ViReal64  threshold,
                                                     ViReal64  delay,
                                                     ViInt32   timeoutStatus,
                                                     ViReal64  timeout,
                                                     ViReal64  minTriggerGap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_SOURCE, source),
            3, "Source");

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_SLOPE, slope),
            4, "Slope");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_THRESHOLD, threshold),
            5, "Threshold");

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_DELAY, delay),
            6, "Delay");

    if (timeoutStatus == RSCMWINM_VAL_STATE_VAL)
    {
        viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_TIMEOUT, timeout),
                8, "Timeout");
    }
    else
    {
        viCheckParm(rscmwinm_SetAttributeViBoolean(instrSession, repCap,
                                                    RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_TIMEOUT_ENABLED,
                                                    (ViBoolean) timeoutStatus), 7, "Timeout Status");
    }

    viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_TRIGGER_GAP, minTriggerGap),
            9, "Min Trigger Gap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function lists all trigger source values that can be set using
/// HIFN  TRIGger:NIOT:MEAS:MEValuation:SOURce.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/This control defines size of array passed to Catalog.
/// HIPAR catalog/Comma-separated list of all supported values. Each value is
/// HIPAR catalog/represented as a string.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasTriggerSourceCatalog(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViChar    catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, catalog),
        4, "Catalog");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "TRIG:NIOT:MEAS%ld:MEV:CAT:SOUR?", instance);
    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, arraySize, catalog, NULL));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables the list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enabled/Enables or disables the list mode.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListEnabled(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViBoolean(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the trigger mode for list mode measurements.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mode/Specifies the trigger mode for list mode measurements.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListMode(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the offline mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enabled/Enables or disables the offline mode.
/// HIPAR index/Selects the segment to be displayed in offline mode.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListOfflineMode(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViBoolean enabled,
                                                             ViInt32   index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    if (enabled == VI_TRUE)
    {
        viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_OFFLINE_MODE_INDEX, index),
                4, "Index");
    }
    else
    {
        checkErr(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_OFFLINE_MODE_DISABLED, NULL));
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies how the input connector is selected for NB-IoT list mode
/// HIFN  measurements with the R&S CMWS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mode/Specifies how the input connector is selected for NB-IoT list mode
/// HIPAR mode/measurements with the R&S CMWS.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListConnectorMode(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   mode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_LIST_CONNECTOR_MODE, mode),
            3, "Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the RF input connector for segment <no> for NB-IoT list mode
/// HIFN  measurements with the R&S CMWS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR connector/Selects the RF input connector for segment <no> for NB-IoT list mode
/// HIPAR connector/measurements with the R&S CMWS.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListCMWSConnector(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   connector)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2000),
            3, "Segment");

    snprintf (repCap, RS_REPCAP_BUF_SIZE, "i%ld,s%ld", instance, segment);

    viCheckParm(rscmwinm_SetAttributeViInt32(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_CMWS_CONNECTOR, connector),
            4, "Connector");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects a range of measured segments.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR startIndex/First measured segment in the range of configured segments.
/// HIPAR segments/Number of measured segments.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListRangeOfMeasuredSegments(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   startIndex,
                            ViInt32   segments)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startIndex, 1, 2000),
            3, "Start Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segments, 1, 1000),
            4, "Segments");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIST:LRAN %ld,%ld", instance, startIndex, segments);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines settings for ACLR measurements in list mode for segment <no>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR length/Statistical length in slots.
/// HIPAR aclr/Enables or disables the measurement of ACLR results.
/// HIPAR utra/Enables or disables the evaluation of adjacent UTRA channels.
/// HIPAR gsm/Enables or disables the evaluation of adjacent GSM channels.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListACLR(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   segment,
                                                      ViInt32   length,
                                                      ViBoolean aclr,
                                                      ViBoolean utra,
                                                      ViBoolean gsm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, length, 1, 1000),
            4, "Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:ACLR %ld,%s,%s,%s", instance, segment, length, aclr ? "ON" : "OFF", utra? "ON" : "OFF", gsm ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines settings for modulation measurements in list mode for segment
/// HIFN  <no>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR length/Statistical length in slots.
/// HIPAR modulation/Enables or disables the measurement of modulation results.
/// HIPAR evm/Enables or disables the measurement of EVM.
/// HIPAR magnitudeError/Enables or disables the measurement of magnitude error.
/// HIPAR phaseError/Enables or disables the measurement of phase error.
/// HIPAR ibe/Enables or disables the measurement of inband emissions.
/// HIPAR modulationScheme/Modulation scheme used by the NB-IoT uplink signal.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListModulation(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   segment,
                                                            ViInt32   length,
                                                            ViBoolean modulation,
                                                            ViBoolean evm,
                                                            ViBoolean magnitudeError,
                                                            ViBoolean phaseError,
                                                            ViBoolean ibe,
                                                            ViInt32   modulationScheme)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, length, 1, 1000),
            4, "Length");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modulationScheme, RSCMWINM_VAL_MODULATION_SCHEME_BPSK, RSCMWINM_VAL_MODULATION_SCHEME_Q16),
            10, "Modulation Scheme");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:MOD %ld,%s,%s,%s,%s,%s,%s", instance, segment, length, modulation ? "ON" : "OFF", evm ? "ON" : "OFF",
              magnitudeError ? "ON" : "OFF", phaseError ? "ON" : "OFF", ibe ? "ON" : "OFF", modSchemeArr[modulationScheme]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines settings for spectrum emission measurements in list mode for
/// HIFN  segment <no>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR length/Statistical length in slots.
/// HIPAR sem/Enables or disables the measurement of spectrum emission trace and
/// HIPAR sem/margin results.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListSpectrumEmission(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   length,
                            ViBoolean sem)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, length, 1, 1000),
            4, "Length");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SEM %ld,%s", instance, segment, length, sem ? "ON" : "OFF");
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the length and analyzer settings of segment <no>. This command
/// HIFN  must be sent for all segments to be measured
/// HIFN  (CONFigure:NIOT:MEAS:MEValuation:LIST:LRANge).
/// HIFN     The supported frequency range depends on the instrument model and the
/// HIFN  available options. The supported range can be smaller than stated here.
/// HIFN  Refer to the preface of your model-specific base unit manual.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR length/Number of slots in the segment.
/// HIPAR level/Expected nominal power in the segment.
/// HIPAR band/Sets the operation band.
/// HIPAR frequency/Carrier center frequency used in the segment.
/// HIPAR npuschFormat/Format of the NPUSCH.
/// HIPAR subcarriers/Number of subcarriers per resource unit.
/// HIPAR startSubcarrier/Offset of the first allocated subcarrier from the edge of the
/// HIPAR startSubcarrier/transmission bandwidth.
/// HIPAR resourceUnits/Number of resource units allocated for the NPUSCH.
/// HIPAR repetitions/Number of NPUSCH repetitions.
/// HIPAR retriggerFlag/Specifies whether the measurement waits for a trigger event
/// HIPAR evaluationOffset/Number of slots at the beginning of the segment that are not
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalMeasListSetup(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   segment,
                                                       ViInt32   length,
                                                       ViReal64  level,
                                                       ViInt32   band,
                                                       ViReal64  frequency,
                                                       ViInt32   npuschFormat,
                                                       ViInt32   subcarriers,
                                                       ViInt32   startSubcarrier,
                                                       ViInt32   resourceUnits,
                                                       ViInt32   repetitions,
                                                       ViInt32   retriggerFlag,
                                                       ViInt32   evaluationOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 2000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, length, 1, 2000),
            4, "Length");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, RSCMWINM_VAL_BAND_OB1, RSCMWINM_VAL_BAND_OB85),
            5, "Band");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequency, 70.0e+6, 6.0e+9), 6, "Frequency");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, npuschFormat, RSCMWINM_VAL_NPUSCH_FORMAT_F1, RSCMWINM_VAL_NPUSCH_FORMAT_F2),
            7, "NPUSCh Format");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, 1, 12),
            8, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resourceUnits, RSCMWINM_VAL_RESOURCE_UNIT_NRU01, RSCMWINM_VAL_RESOURCE_UNIT_NRU10),
            10, "Resource Units");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetitions, RSCMWINM_VAL_REPETITIONS_1, RSCMWINM_VAL_REPETITIONS_2056),
            11, "Repetitions");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, retriggerFlag, RSCMWINM_VAL_RETRIGGER_FLAG_OFF, RSCMWINM_VAL_RETRIGGER_FLAG_IFP),
            12, "Retrigger Flag");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, evaluationOffset, 1, 1000),
            13, "Evaluation Offset");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SET %ld,%.12lG,%s,%.12lG,%s,%ld,%ld,%s,%s,%s,%ld", instance, segment,
              length, level, bandArr[band], frequency, NPUSChFormatArr[npuschFormat], subcarriers, startSubcarrier, resourceUnitsArr[resourceUnits],
              repetitionsArr[repetitions], retriggerFlagArr[retriggerFlag], evaluationOffset);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines upper limits for the RMS and peak values of the
/// HIFN  selected modulation characteristic.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR modulationCharacteristic/This control selects modulation characteristic.
/// HIPAR rmsLimitValue/Sets the limit value.
/// HIPAR rmsLimitStatus/Enables or disables limit.
/// HIPAR peakLimitValue/Sets the limit value.
/// HIPAR peakLimitStatus/Enables or disables limit.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsModulation(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   modulationCharacteristic,
                                                          ViReal64  rmsLimitValue,
                                                          ViInt32   rmsLimitStatus,
                                                          ViReal64  peakLimitValue,
                                                          ViInt32   peakLimitStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViString param[] = {"EVM", "MERR", "PERR", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, modulationCharacteristic, RSCMWINM_VAL_RES_EVM, RSCMWINM_VAL_RES_PERR),
            3, "Modulation Characteristic");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rmsLimitStatus, RSCMWINM_VAL_STATE_OFF, RSCMWINM_VAL_STATE_VAL),
            5, "RMS Limit Status");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, peakLimitStatus, RSCMWINM_VAL_STATE_OFF, RSCMWINM_VAL_STATE_VAL),
            7, "Peak Limit Status");

    if (rmsLimitStatus == RSCMWINM_VAL_STATE_VAL)
    {
        if (peakLimitStatus == RSCMWINM_VAL_STATE_VAL)
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, rmsLimitValue, 0.0, 100.0), 4, "RMS Limit Value");
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, peakLimitValue, 0.0, 100.0), 6, "Peak Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:%s %.12lG, %.12lG", instance, param[modulationCharacteristic], rmsLimitValue, peakLimitValue);
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, rmsLimitValue, 0.0, 100.0), 4, "RMS Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:%s %.12lG, %s", instance, param[modulationCharacteristic], rmsLimitValue, peakLimitStatus?"ON":"OFF");
            checkErr(RsCore_Write(instrSession, cmd));
        }
    }
    else
    {
        if (peakLimitStatus == RSCMWINM_VAL_STATE_VAL)
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, peakLimitValue, 0.0, 100.0), 6, "Peak Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:%s %s, %.12lG", instance, param[modulationCharacteristic], rmsLimitStatus?"ON":"OFF", peakLimitValue);
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:%s %s, %s", instance, param[modulationCharacteristic], rmsLimitStatus?"ON":"OFF", peakLimitStatus?"ON":"OFF");
            checkErr(RsCore_Write(instrSession, cmd));
        }
    }

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines upper limits for the carrier frequency error.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR state/Enables or disables the limit check.
/// HIPAR limitLower/Upper limit for frequencies up to 1 GHz.
/// HIPAR limitUpper/Upper limit for frequencies above 1 GHz.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsFrequencyError(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean state,
                            ViReal64  limitLower,
                            ViReal64  limitUpper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, limitLower, 0.0, 1.0), 4, "Limit Lower");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, limitUpper, 0.0, 1.0), 5, "Limit Upper");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:FERR %s,%.12lG,%.12lG", instance, state?"ON":"OFF", limitLower, limitUpper);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines upper limits for the I/Q origin offset. Three
/// HIFN  different limits can be set for three TX power ranges.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enable/Enables or disables the limit check.
/// HIPAR highTXPowerRange/I/Q origin offset limit for high TX power range.
/// HIPAR intermediateTXPowerRange/I/Q origin offset limit for intermediate TX power range
/// HIPAR lowTXPowerRange/I/Q origin offset limit for low TX power range
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsIQOffset(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViBoolean enable,
                                                        ViReal64  highTXPowerRange,
                                                        ViReal64  intermediateTXPowerRange,
                                                        ViReal64  lowTXPowerRange)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, highTXPowerRange, -256.0, 256.0), 4, "> 0 dBm Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, intermediateTXPowerRange, -256.0, 256.0), 5, "> -30 dBm Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, lowTXPowerRange, -256.0, 256.0), 6, "> -40 dBm Limit");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:IQOF %s,%.12lG,%.12lG,%.12lG", instance, enable?"ON":"OFF", highTXPowerRange, intermediateTXPowerRange, lowTXPowerRange);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines parameters used for calculation of an upper
/// HIFN  limit for the inband emissions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enable/Enables or disables the limit check.
/// HIPAR minimumLimit/Sets the minimum limit value.
/// HIPAR scPowerLimit/Sets the SC Power limit value.
/// HIPAR iqImageLimit/Sets the IQ Image limit value.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsInbandEmissions(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enable,
                            ViReal64  minimumLimit,
                            ViReal64  scPowerLimit,
                            ViReal64  iqImageLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, minimumLimit, -256.0, 256.0), 4, "Minimum Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, scPowerLimit, -256.0, 256.0), 5, "SC Power Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, iqImageLimit, -256.0, 256.0), 6, "IQ Image Limit");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:IBE %s,%.12lG,%.12lG,%.12lG", instance, enable?"ON":"OFF", minimumLimit, scPowerLimit, iqImageLimit);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines I/Q origin offset values used for calculation of
/// HIFN  an upper limit for the inband emissions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR _0dBmLimit/Offset for high TX power range
/// HIPAR _30dBmLimit/Offset for intermediate TX power range
/// HIPAR _40dBmLimit/Offset for low TX power range
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsInbandEmissionsIQOffset(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64  _0dBmLimit,
                            ViReal64  _30dBmLimit,
                            ViReal64  _40dBmLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, _0dBmLimit, -256.0, 256.0), 3, "> 0 dBm Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, _30dBmLimit, -256.0, 256.0), 4, "> -30 dBm Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, _40dBmLimit, -256.0, 256.0), 5, "> -40 dBm Limit");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:IBE:IQOF %.12lG,%.12lG,%.12lG", instance, _0dBmLimit, _30dBmLimit, _40dBmLimit);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the overall occupied bandwidth limit and the
/// HIFN  state of the limit check.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR limitValue/Defines an upper limit for the occupied bandwidth.
/// HIPAR limitStatus/Enables or disables limit.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsOBW(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViReal64  limitValue,
                                                   ViInt32   limitStatus)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    if (limitStatus == RSCMWINM_VAL_STATE_VAL)
    {
        viCheckParm(rscmwinm_SetAttributeViReal64(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_SPECTRUM_OBW_LIMIT, limitValue),
                3, "Limit Value");
    }
    else
    {
        viCheckParm(rscmwinm_SetAttributeViBoolean(instrSession, repCap,
                                                    RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_SPECTRUM_OBW_LIMIT_ENABLED,
                                                    (ViBoolean) limitStatus), 4, "Limit Status");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines the emission mask area <no>.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR area/Sets the number of the emission mask area.
/// HIPAR frequencyStart/Start frequency of the area, relative to the edges of the channel
/// HIPAR enable/Enables or disables the limit check.
/// HIPAR frequencyStart/bandwidth.
/// HIPAR frequencyStop/Stop frequency of the area, relative to the edges of the channel
/// HIPAR frequencyStop/bandwidth.
/// HIPAR powerStart/Upper limit at the <FrequencyStart>.
/// HIPAR powerStop/Upper limit at the <FrequencyStop>.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsSpectrumEmissionMask(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   area,
                            ViBoolean enable,
                            ViReal64  frequencyStart,
                            ViReal64  frequencyStop,
                            ViReal64  powerStart,
                            ViReal64  powerStop)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, area, 1, 12),
            3, "Area");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequencyStart, 0.0, 1.9e6), 5, "Frequency Start");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, frequencyStop, 0.0, 1.9e6), 6, "Frequency Stop");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, powerStart, -256.0, 256.0), 7, "Power Start");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, powerStop, -256.0, 256.0), 8, "Power Stop");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:SEM:LIM%ld %s,%.12lG,%.12lG,%.12lG,%.12lG", instance, area, enable?"ON":"OFF",
                          frequencyStart, frequencyStop, powerStart, powerStop);
    checkErr(RsCore_Write(instrSession, cmd));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines a relative and absolute limit for the ACLR
/// HIFN  measured in the adjacent GSM channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR measurement/This control selects measurement.
/// HIPAR relativeLimitValue/Sets the relative limit value.
/// HIPAR relativeLimitStatus/Enables or disables limit.
/// HIPAR absoluteLimitValue/Sets the absolute limit value.
/// HIPAR absoluteLimitStatus/Enables or disables limit.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsACLR(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   measurement,
                                                    ViReal64  relativeLimitValue,
                                                    ViInt32   relativeLimitStatus,
                                                    ViReal64  absoluteLimitValue,
                                                    ViInt32   absoluteLimitStatus)
{
    ViStatus error = VI_SUCCESS;
    ViString param[] = {"GSM", "UTRA", NULL};
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, measurement, RSCMWINM_VAL_GSM, RSCMWINM_VAL_UTRA),
            3, "Measurement");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, relativeLimitStatus, RSCMWINM_VAL_STATE_OFF, RSCMWINM_VAL_STATE_VAL),
            5, "Relative Limit Status");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, absoluteLimitStatus, RSCMWINM_VAL_STATE_OFF, RSCMWINM_VAL_STATE_VAL),
            7, "Absolute Limit Status");

    if (relativeLimitStatus == RSCMWINM_VAL_STATE_VAL)
    {
        if (absoluteLimitStatus == RSCMWINM_VAL_STATE_VAL)
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, relativeLimitValue, -256.0, 256.0), 4, "Relative Limit Value");
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, absoluteLimitValue, -256.0, 256.0), 6, "Absolute Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:ACLR:%s %.12lG, %.12lG", instance, param[measurement], relativeLimitValue, absoluteLimitValue);
            checkErr(RsCore_Write(instrSession, cmd));
        }
        else
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, relativeLimitValue, -256.0, 256.0), 4, "Relative Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:ACLR:%s %.12lG, %s", instance, param[measurement], relativeLimitValue, absoluteLimitStatus?"ON":"OFF");
            checkErr(RsCore_Write(instrSession, cmd));
        }
    }
    else
    {
        if (absoluteLimitStatus == RSCMWINM_VAL_STATE_VAL)
        {
            viCheckParm(RsCore_InvalidViReal64Range(instrSession, absoluteLimitValue, -256.0, 256.0), 6, "Absolute Limit Value");

            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:ACLR:%s %s, %.12lG", instance, param[measurement], relativeLimitStatus?"ON":"OFF", absoluteLimitValue);
        }
        else
        {
            snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:ACLR:%s %s, %s", instance, param[measurement], relativeLimitStatus?"ON":"OFF", absoluteLimitStatus?"ON":"OFF");
        }
    }

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function defines limits for the ON power and OFF power determined
/// HIFN  with the power dynamics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR enable/Enables or disables the limit check.
/// HIPAR onPowerUpperLimit/Upper limit for the 'ON power'.
/// HIPAR onPowerLowerLimit/Lower limit for the 'ON power'.
/// HIPAR offPowerUpperLimit/Upper limit for the 'OFF power'.
ViStatus _VI_FUNC rscmwinm_ConfigureMEvalLimitsPowerDynamics(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViBoolean enable,
                                                             ViReal64  onPowerUpperLimit,
                                                             ViReal64  onPowerLowerLimit,
                                                             ViReal64  offPowerUpperLimit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, onPowerUpperLimit, -256.0, 256.0), 4, "On Power Upper Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, onPowerLowerLimit, -256.0, 256.0), 5, "On Power Lower Limit");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, offPowerUpperLimit, -256.0, 256.0), 6, "Off Power Upper Limit");

    snprintf (cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:MEAS%ld:MEV:LIM:PDYN %s,%.12lG,%.12lG,%.12lG", instance, enable?"ON":"OFF",
                          onPowerUpperLimit, onPowerLowerLimit, offPowerUpperLimit);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts or restarts the measurement; the R&S CMW 500 enters the "RUN"
/// HIFN  state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwinm_MEvalMeasInit(ViSession instrSession,
                                         ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_INIT, NULL),
            3, "Multi Evaluation Measurement Init");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Causes a running measurement to stop immediately; the R&S CMW 500
/// HIFN  enters the "OFF" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwinm_MEvalMeasAbort(ViSession instrSession,
                                          ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_ABORT, NULL),
            3, "Multi Evaluation Measurement Abort");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Causes a running measurement to stop after the current evaluation
/// HIFN  period is terminated and valid results are available; the R&S CMW 500
/// HIFN  enters the "RDY" state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
ViStatus _VI_FUNC rscmwinm_MEvalMeasStop(ViSession instrSession,
                                         ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwinm_SetAttributeViString(instrSession, repCap, RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_STOP, NULL),
            3, "Multi Evaluation Measurement Stop");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the main measurement state and the measurement substates. Both
/// HIFN  measurement substates are relevant for running measurements only. Use
/// HIFN  FETCh...MEASurement<instance>:STATe? to query the main measurement
/// HIFN  state only. Use INITiate..., STOP..., ABORt... to change the
/// HIFN  measurement state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR mainState/Returns main state.
/// HIPAR syncState/Returns synchronization state.
/// HIPAR resourcesState/Returns resources state.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasurementStatus(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32*  mainState,
                                                       ViInt32*  syncState,
                                                       ViInt32*  resourcesState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:STAT:ALL?", instance);
    checkErr(rscmwinm_QueryMeasStatus(instrSession, cmd, mainState, syncState, resourcesState));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns values of the EVM vs
/// HIFN  modulation symbol trace. The results of the current, average and
/// HIFN  maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the EVM vs modulation symbol trace.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasEVMTrace(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   resultType,
                                                 ViInt32   arraySize,
                                                 ViInt32*  reliability,
                                                 ViReal64  results[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:EVMS:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns values of the EVM vs modulation symbol trace. The results of
/// HIFN  the current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the EVM vs modulation symbol trace.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasEVMTrace(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   resultType,
                                                  ViInt32   arraySize,
                                                  ViInt32*  reliability,
                                                  ViReal64  results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:EVMS:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns values of the EVM RMS
/// HIFN  or peak bar graphs for the SC-FDMA symbols in the measured slot. The
/// HIFN  results of the current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the EVM RMS or peak bar graphs for the SC-FDMA
/// HIPAR results/symbols in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasEVMSCFDMASymbols(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   resultType,
                                                         ViInt32   type,
                                                         ViInt32   arraySize,
                                                         ViInt32*  reliability,
                                                         ViReal64  results[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString typeArr[] = {"",":PEAK", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSCMWINM_VAL_TYPE_RMS, RSCMWINM_VAL_TYPE_PEAK),
            4, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 7, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:EVM%s:%s?", instance, typeArr[type], resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns values of the EVM RMS or peak bar graphs for the SC-FDMA
/// HIFN  symbols in the measured slot. The results of the current, average and
/// HIFN  maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR type/Denotes the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the EVM RMS or peak bar graphs for the SC-FDMA
/// HIPAR results/symbols in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasEVMSCFDMASymbols(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   resultType,
                                                          ViInt32   type,
                                                          ViInt32   arraySize,
                                                          ViInt32*  reliability,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString typeArr[] = {"",":PEAK", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, type, RSCMWINM_VAL_TYPE_RMS, RSCMWINM_VAL_TYPE_PEAK),
            4, "Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 7, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:EVM%s:%s?", instance, typeArr[type], resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns values of the
/// HIFN  magnitude error bar graphs for the SC-FDMA symbols in the measured
/// HIFN  slot. The results of the current, average and maximum traces can be
/// HIFN  retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the magnitude error bar graphs for the SC-FDMA
/// HIPAR results/symbols in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasMagnitudeErrorSCFDMASymbols(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:MERR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns values of the magnitude error bar graphs for the SC-FDMA
/// HIFN  symbols in the measured slot. The results of the current, average and
/// HIFN  maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the magnitude error bar graphs for the SC-FDMA
/// HIPAR results/symbols in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasMagnitudeErrorSCFDMASymbols(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:MERR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns values of the phase
/// HIFN  error bar graphs for the SC-FDMA symbols in the measured slot. The
/// HIFN  results of the current, average and maximum traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the phase error bar graphs for the SC-FDMA symbols
/// HIPAR results/in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasPhaseErrorSCFDMASymbols(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:PERR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns values of the phase error bar graphs for the SC-FDMA symbols
/// HIFN  in the measured slot. The results of the current, average and maximum
/// HIFN  traces can be retrieved.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns values of the phase error bar graphs for the SC-FDMA symbols
/// HIPAR results/in the measured slot.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasPhaseErrorSCFDMASymbols(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:PERR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the values of the
/// HIFN  inband emissions trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the power values, one value per subcarrier.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasInbandEmissionTrace(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   arraySize,
                                                            ViInt32*  reliability,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:IEM?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the values of the inband emissions trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the power values, one value per subcarrier.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasInbandEmissionTrace(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   arraySize,
                                                             ViInt32*  reliability,
                                                             ViReal64  results[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:IEM?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the limit line margin results for the inband emissions. The
/// HIFN  CURRent margin indicates the minimum (vertical) distance between the
/// HIFN  inband emissions limit line and the current trace. A negative result
/// HIFN  indicates that the limit is exceeded. The AVERage and EXTReme values
/// HIFN  are calculated from the current margins.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR outOfTolerance/Out of tolerance result, i.e. percentage of measurement intervals of
/// HIPAR outOfTolerance/the statistic count for modulation measurements exceeding the specified
/// HIPAR outOfTolerance/inband emission limits.
/// HIPAR margin/Return the limit line margin results for the inband emissions.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasInbandEmissionMargin(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32*  reliability,
                            ViReal64* outOfTolerance,
                            ViReal64* margin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 results[2] = {RS_VAL_NAN_VI_REAL64, RS_VAL_NAN_VI_REAL64};
    ViInt32  returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, outOfTolerance), 5, "Out Of Tolerance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, margin), 6, "Margin");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:IEM:MARG:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, 2, reliability, results, &returnedValues));

    *outOfTolerance = results[0];

    *margin = results[1];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return the standard deviation of the limit line margin results for the
/// HIFN  inband emissions.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR outOfTolerance/Out of tolerance result, i.e. percentage of measurement intervals of
/// HIPAR outOfTolerance/the statistic count for modulation measurements exceeding the specified
/// HIPAR outOfTolerance/inband emission limits.
/// HIPAR margin/Return the limit line margin results for the inband emissions.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasInbandEmissionMarginStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32*  reliability,
                            ViReal64* outOfTolerance,
                            ViReal64* margin)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 results[2] = {RS_VAL_NAN_VI_REAL64, RS_VAL_NAN_VI_REAL64};
    ViInt32  returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, outOfTolerance), 4, "Out Of Tolerance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, margin), 5, "Margin");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:IEM:MARG:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, 2, reliability, results, &returnedValues));

    *outOfTolerance = results[0];

    *margin = results[1];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return subcarrier indices for inband emission margins. At these SC
/// HIFN  indices, the CURRent and EXTReme margins have been detected
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR outOfTolerance/Out of tolerance result, i.e. percentage of measurement intervals of
/// HIPAR outOfTolerance/the statistic count for modulation measurements exceeding the specified
/// HIPAR outOfTolerance/inband emission limits.
/// HIPAR scIndex/Return the subcarrier index.
/// HIPAR resultType/Denotes the statistics type.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasInbandEmissionMarginSCIndex(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32*  reliability,
                            ViReal64* outOfTolerance,
                            ViInt32*  scIndex)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 results[2] = {RS_VAL_NAN_VI_REAL64, RS_VAL_NAN_VI_REAL64};
    ViInt32  returnedValues = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, outOfTolerance), 5, "Out Of Tolerance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, scIndex), 6, "SC Index");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:IEM:MARG:%s:SCIN?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, 2, reliability, results, &returnedValues));

    *outOfTolerance = results[0];

    *scIndex = (ViInt32) results[1];

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the values of the
/// HIFN  spectrum emission traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasSpectrumEmissionMaskTrace(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:SEM:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the values of the spectrum emission traces.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumEmissionMaskTrace(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:SEM:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the single value
/// HIFN  results of the spectrum emission measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasSpectrumEmissionMask(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   resultType,
                                                             ViInt32   arraySize,
                                                             ViInt32*  reliability,
                                                             ViReal64  results[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:SEM:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the single value results of the spectrum emission measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumEmissionMask(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:SEM:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the standard
/// HIFN  deviation of the single value results of the spectrum emission
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasSpectrumEmissionMaskStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:SEM:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the standard deviation of the single value results of the
/// HIFN  spectrum emission measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumEmissionMaskStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:SEM:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns limit check results of the spectrum emission measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns limit check results of the spectrum emission measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasSpectrumEmissionMaskLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:NIOT:MEAS%ld:MEV:SEM:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasErrorCodes(instrSession, cmd, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns spectrum emission mask margin results. A negative margin
/// HIFN  indicates that the trace is located above the limit line, i.e. the
/// HIFN  limit is exceeded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement. For inactive areas, NAN is
/// HIPAR results/returned.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumEmissionMaskAllMargins(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:SEM:MARG:ALL?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns spectrum emission mask margin results. A negative margin
/// HIFN  indicates that the trace is located above the limit line, i.e. the
/// HIFN  limit is exceeded.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR marginType/Denotes the margin type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR outOfTolerance/Out of tolerance result, i.e. percentage of measurement intervals of
/// HIPAR outOfTolerance/the statistic count for spectrum emission measurements exceeding the
/// HIPAR outOfTolerance/specified spectrum emission mask limits.
/// HIPAR marginX/Returns X-positions of margin for emission mask areas 1 to 12.
/// HIPAR marginY/Returns Y-value of margin for emission mask areas 1 to 12.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumEmissionMaskMargin(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   marginType,
                            ViInt32*  reliability,
                            ViReal64* outOfTolerance,
                            ViReal64  marginX[],
                            ViReal64  marginY[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  i = 0;
    ViReal64 results[25] = {RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64};
    ViString marginTypeArr[] = {"NEG", "POS", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MIN),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marginType, RSCMWINM_VAL_NEGATIVE, RSCMWINM_VAL_POSITIVE),
            4, "Margin Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, outOfTolerance), 6, "Out Of Tolerance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, marginX), 7, "Margin X");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, marginY), 8, "Margin Y");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:SEM:MARG:%s:%s?", instance, resultTypeArr[resultType], marginTypeArr[marginType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, 25, reliability, results, returnedValues));

    *outOfTolerance = results[0];

    for (i = 0; i < 12; i++)
    {
        marginX[i] = results[1 + i*2];
        marginX[i] = results[2 + i*2];
    }

    *returnedValues = (*returnedValues - 1) / 2; // remove outOfTolerance, devide into two arrays MarginX and MarginY

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the absolute powers
/// HIFN  as displayed in the ACLR diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasSpectrumACLRTrace(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   resultType,
                                                          ViInt32   arraySize,
                                                          ViInt32*  reliability,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:ACLR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the absolute powers as displayed in the ACLR diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumACLRTrace(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   resultType,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:ACLR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the relative ACLR
/// HIFN  values as displayed in the table below the ACLR diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasSpectrumACLR(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   resultType,
                                                     ViInt32   arraySize,
                                                     ViInt32*  reliability,
                                                     ViReal64  results[],
                                                     ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:ACLR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the relative ACLR values as displayed in the table below the
/// HIFN  ACLR diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasSpectrumACLR(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   resultType,
                                                      ViInt32   arraySize,
                                                      ViInt32*  reliability,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:ACLR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns limit check results of the relative ACLR values as displayed
/// HIFN  in the table below the ACLR diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the limit check results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasSpectrumACLRLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:NIOT:MEAS%ld:MEV:ACLR:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasErrorCodes(instrSession, cmd, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the results in the
/// HIFN  I/Q constellation diagram.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result arrays.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR i/Returns the I amplitudes of the measurement.
/// HIPAR q/Returns the Q amplitudes of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasIQTrace(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   arraySize,
                                                 ViInt32*  reliability,
                                                 ViReal64  i[],
                                                 ViReal64  q[],
                                                 ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViReal64 *results;
    ViInt32  j = 0;
    results  = malloc (2*arraySize*sizeof(ViReal64));

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, i), 5, "I");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, q), 6, "Q");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:IQ?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize*2, reliability, results, returnedValues));

    for (j=0; j<*returnedValues; j++)
    {
        if (j%2)
            q[(ViInt32)j/2] = results[j];
        else
            i[(ViInt32)j/2] = results[j];
    }

Error:
    if (results) free (results);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the power monitor
/// HIFN  results for all captured slots. The number of captured slots is
/// HIFN  configurable, see
/// HIFN  RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NUMBER_OF_MEASURE_SLOTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasPowerMonitorTrace(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   arraySize,
                                                          ViInt32*  reliability,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:PMON?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the power monitor results for all captured slots. The number
/// HIFN  of captured slots is configurable, see
/// HIFN  RSCMWINM_ATTR_MULTI_EVAL_MEASUREMENT_NUMBER_OF_MEASURE_SLOTS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasPowerMonitorTrace(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:PMON?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the values of the
/// HIFN  power dynamics trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR powerType/Denotes the power type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the power values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasPowerDynamicsTrace(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   resultType,
                                                           ViInt32   powerType,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString powerTypeArr[] = {"",":POST", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerType, RSCMWINM_VAL_START, RSCMWINM_VAL_END),
            4, "Power Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 7, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:TRAC:PDYN%s:%s?", instance, powerTypeArr[powerType], resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the values of the power dynamics trace.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR powerType/Denotes the power type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the power values.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasPowerDynamicsTrace(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   resultType,
                                                            ViInt32   powerType,
                                                            ViInt32   arraySize,
                                                            ViInt32*  reliability,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViString powerTypeArr[] = {"",":POST", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MAX),
            3, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, powerType, RSCMWINM_VAL_START, RSCMWINM_VAL_END),
            4, "Power Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 7, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:TRAC:PDYN%s:%s?", instance, powerTypeArr[powerType], resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the single value
/// HIFN  results of the power dynamics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasPowerDynamics(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32   resultType,
                                                      ViInt32   arraySize,
                                                      ViInt32*  reliability,
                                                      ViReal64  results[],
                                                      ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MIN),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:PDYN:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the single value results of the power dynamics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasPowerDynamics(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   resultType,
                                                       ViInt32   arraySize,
                                                       ViInt32*  reliability,
                                                       ViReal64  results[],
                                                       ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MIN),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:PDYN:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the standard
/// HIFN  deviation of the single value results of the power dynamics
/// HIFN  measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasPowerDynamicsStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:PDYN:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the standard deviation of the single value results of the
/// HIFN  power dynamics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasPowerDynamicsStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:PDYN:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns limit check results of the power dynamics measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the limit check results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasPowerDynamicsLimitCheckResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MIN),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:NIOT:MEAS%ld:MEV:PDYN:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasErrorCodes(instrSession, cmd, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the results of the
/// HIFN  multi evaluation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasModulationResults(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   resultType,
                                                          ViInt32   arraySize,
                                                          ViInt32*  reliability,
                                                          ViReal64  results[],
                                                          ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:MOD:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the results of the multi evaluation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasModulationResults(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   resultType,
                                                           ViInt32   arraySize,
                                                           ViInt32*  reliability,
                                                           ViReal64  results[],
                                                           ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:MOD:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts a new single shot measurement and returns the standard
/// HIFN  deviation of the multi evaluation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_ReadMEvalMeasModulationStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "READ:NIOT:MEAS%ld:MEV:MOD:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the standard deviation of the multi evaluation measurement.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns the modulation results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasModulationStandardDeviation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 5, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:MOD:SDEV?", instance);

    checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns limit check of error codes of single value results.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns limit check result of the modulation results of the
/// HIPAR results/measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_QueryMEvalMeasModulationResultsLimitsCheckResult(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32   results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            3, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CALC:NIOT:MEAS%ld:MEV:MOD:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasErrorCodes(instrSession, cmd, arraySize, reliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Return ACLR single value results for segment <no> in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Return ACLR single value results for segment <no> in list mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListACLR(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   segment,
                                                  ViInt32   resultType,
                                                  ViInt32   arraySize,
                                                  ViInt32*  reliability,
                                                  ViInt32*  segmentReliability,
                                                  ViReal64  _VI_FAR results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_AVERAGE),
            4, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:ACLR:%s?", instance, segment, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return the inband emission limit line margin results for segment <no>
/// HIFN  in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Return the inband emission limit line margin results for segment <no>
/// HIPAR results/in list mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListInbandEmission(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   segment,
                                                            ViInt32   resultType,
                                                            ViInt32   arraySize,
                                                            ViInt32*  reliability,
                                                            ViInt32*  segmentReliability,
                                                            ViReal64  results[],
                                                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_SDEV),
            4, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:IEM:MARG:%s?", instance, segment, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return subcarrier indices for inband emission margins for segment <no>
/// HIFN  in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Return subcarrier indices for inband emission margins for segment <no>
/// HIPAR results/in list mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListInbandEmissionSCIndex(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_EXTREME),
            4, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:IEM:MARG:%s:SCIN?", instance, segment, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns current, average and standard deviation modulation single
/// HIFN  value results for segment <no> in list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns current, average and standard deviation modulation single
/// HIPAR results/value results for segment <no> in list mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListModulation(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViInt32   segment,
                                                        ViInt32   resultType,
                                                        ViInt32   arraySize,
                                                        ViInt32*  reliability,
                                                        ViInt32*  segmentReliability,
                                                        ViReal64  results[],
                                                        ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_SDEV),
            4, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:MOD:%s?", instance, segment, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns extreme modulation single value results for segment <no> in
/// HIFN  list mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns extreme modulation single value results for segment <no> in
/// HIPAR results/list mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListModulationExtreme(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:MOD:EXTR?", instance, segment);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return spectrum emission single value results for segment <no> in list
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Return spectrum emission single value results for segment <no> in list
/// HIPAR results/mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListSpectrumEmissionMask(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   resultType,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_SDEV),
            4, "Result Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SEM:%s?", instance, segment, resultTypeArr[resultType]);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns spectrum emission extreme results for segment <no> in list
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns spectrum emission extreme results for segment <no> in list
/// HIPAR results/mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListSpectrumEmissionMaskExtreme(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SEM:EXTR?", instance, segment);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns limit line margin values, i.e. vertical distances between the
/// HIFN  spectrum emission mask limit line and a trace, for segment <no> in list
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR results/Returns limit line margin values, i.e. vertical distances between the
/// HIPAR results/spectrum emission mask limit line and a trace, for segment <no> in list
/// HIPAR results/mode.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListSpectrumEmissionMaskMarginAll(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViReal64  results[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, results), 6, "Results");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SEM:MARG:ALL?", instance, segment);

    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, arraySize, reliability, segmentReliability, results, returnedValues));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Return spectrum emission mask margin results for segment <no> in list
/// HIFN  mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR segment/Segment number.
/// HIPAR resultType/Denotes the statistics type.
/// HIPAR marginType/Denotes the margin type.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR outOfTolerance/Out of tolerance result, i.e. percentage of measurement intervals of
/// HIPAR outOfTolerance/the statistic count for spectrum emission measurements exceeding the
/// HIPAR outOfTolerance/specified spectrum emission mask limits.
/// HIPAR marginX/Returns X-positions of margin for emission mask areas 1 to 12.
/// HIPAR marginY/Returns Y-value of margin for emission mask areas 1 to 12.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListSpectrumEmissionMaskMargin(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   segment,
                            ViInt32   resultType,
                            ViInt32   marginType,
                            ViInt32*  reliability,
                            ViInt32*  segmentReliability,
                            ViInt32*  statisticExpired,
                            ViReal64* outOfTolerance,
                            ViReal64  marginX[],
                            ViReal64  marginY[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViInt32  i = 0;
    ViReal64 results[26] = {RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64,
             RS_VAL_NAN_VI_REAL64,RS_VAL_NAN_VI_REAL64};
    ViString marginTypeArr[] = {"NEG", "POS", NULL};

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, segment, 1, 1000),
            3, "Segment");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINM_VAL_RESULT_CURRENT, RSCMWINM_VAL_RESULT_MIN),
            4, "Result Type");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, marginType, RSCMWINM_VAL_NEGATIVE, RSCMWINM_VAL_POSITIVE),
            5, "Margin Type");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, outOfTolerance), 6, "Out Of Tolerance");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, marginX), 7, "Margin X");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, marginY), 8, "Margin Y");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SEGM%ld:SEM:MARG:%s:%s?", instance, segment, resultTypeArr[resultType], marginTypeArr[marginType]);

    //checkErr(rscmwinm_GetMeas(instrSession, cmd, 15000, 25, reliability, results, returnedValues));
    checkErr(rscmwinm_GetMeasSeg(instrSession, cmd, 15000, 26, reliability, segmentReliability, results, returnedValues));

    *statisticExpired = (ViInt32) results[0];
    *outOfTolerance = results[1];

    for (i = 0; i < 12; i++)
    {
        marginX[i] = results[2 + i*2];
        marginY[i] = results[3 + i*2];
    }

    *returnedValues = (*returnedValues - 2) / 2; // remove outOfTolerance, statisticExpired devide into two arrays MarginX and MarginY

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the segment reliability for all measured list mode segments.
/// HIFN     A common reliability indicator of zero indicates that the results in
/// HIFN  all measured segments are valid. A non-zero value indicates that an
/// HIFN  error occurred in at least one of the
/// HIFN     measured segments. If you get a non-zero common reliability indicator,
/// HIFN  you can use this command to retrieve the individual reliability values
/// HIFN  of all measured segments for further analysis.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwinm_ReliabilityIndicator.
/// HIPAR segmentReliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR segmentReliability/result. To translate this code into human-readable string, use the
/// HIPAR segmentReliability/rscmwinm_ReliabilityIndicator.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwinm_FetchMEvalMeasListSegmentReliability(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32*  reliability,
                            ViInt32   segmentReliability[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  ext_buf=NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 32),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:MEAS%ld:MEV:LIST:SREL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &ext_buf));

    checkErr(rscmwinm_CheckStatus(instrSession));

    p2buf = strtok (ext_buf, ",");
	if (p2buf == NULL)
		return RSCMWINM_ERROR_INCORRECT_RESPONSE;

	if (reliability != NULL)
		*reliability = RsCore_Convert_String_To_ViInt32(p2buf);

    p2buf = strtok (NULL, ",\n");
    while (p2buf != NULL)
    {
        if (i<arraySize)
        {
            segmentReliability[i++] = RsCore_Convert_String_To_ViInt32(p2buf);
        }
        p2buf = strtok (NULL, ",\n");
    }
    if (returnedValues) *returnedValues=i;

Error:
    if (ext_buf) free (ext_buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument.  The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rscmwinm_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rscmwinm_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rscmwinm_DefaultInstrSetup(instrSession));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}

/// HIFN  This function runs the instrument's self test routine and returns the
/// HIFN  test result(s).
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR selfTestResult/This control contains the value returned from the instrument self
/// HIPAR selfTestResult/test.  Zero means success.  For any other code, see the device's
/// HIPAR selfTestResult/operator's manual.
/// HIPAR selfTestMessage/Returns the self-test response string from the instrument. See the
/// HIPAR selfTestMessage/device's operation manual for an explanation of the string's contents.
ViStatus _VI_FUNC rscmwinm_self_test(ViSession instrSession,
                                     ViInt16*  selfTestResult,
                                     ViChar    selfTestMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, selfTestResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, selfTestMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, selfTestResult, 48, selfTestMessage));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the revision numbers of the instrument driver
/// HIFN  and instrument firmware.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR instrumentDriverRevision/Returns the instrument driver software revision numbers in the form of
/// HIPAR instrumentDriverRevision/a string.
/// HIPAR firmwareRevision/Returns the instrument firmware revision numbers in the form of a
/// HIPAR firmwareRevision/string.
ViStatus _VI_FUNC rscmwinm_revision_query(ViSession instrSession,
                                          ViChar    driverRev[],
                                          ViChar    instrRev[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the number of the SCPI version, which is
/// HIFN      relevant for the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIFN      rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIFN      instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the
/// HIFN      instrument.
ViStatus _VI_FUNC rscmwinm_SystemVersion(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwinm_GetAttributeViString(instrSession, "", RSCMWINM_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            3, "System Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads an error code and a message from the instrument's
/// HIFN  error queue.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error
/// HIPAR errorMessage/message queue.
ViStatus _VI_FUNC rscmwinm_error_query(ViSession instrSession,
                                       ViInt32*  errorCode,
                                       ViChar    _VI_FAR errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function reads all system messages and returns a list of comma
/// HIFN      separated strings. Each string corresponds to an entry in the table
/// HIFN      SYSTEM MESSAGES.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIFN      rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIFN      instrument session.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error
/// HIFN      message queue.
ViStatus _VI_FUNC rscmwinm_ErrorList(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    errors[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0), 2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors), 3, "Null address for Errors");

    checkErr(rscmwinm_GetAttributeViString( instrSession, "", RSCMWINM_ATTR_SYSTEM_ALL_ERRORS,
        bufferSize, errors));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rscmwinm_error_message(ViSession instrSession,
                                         ViStatus  errorCode,
                                         ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;
    static   RsCoreStringValueTable errorTable =
             {
             RSCMWINM_ERROR_CODES_AND_MSGS,
             {0, NULL}
                 };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Reliability Indicator
 * Purpose:  This function converts a reliability code returned by an
 *           instrument driver function into a user-readable string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_ReliabilityIndicator(ViSession instrSession,
                                                ViInt32   reliabilityCode,
                                                ViChar    reliabilityMessage[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt16   cnt = 0;
    ViBoolean found = VI_FALSE;

    static      RsCoreStringValueTable reliabilityTable =
    {
        {0,       "OK"},
        {1,       "Measurement Timeout"},
        {2,       "Capture Buffer Overflow"},
        {3,       "Overdriven"},
        {4,       "Underdriven"},
        {6,       "Trigger Timeout"},
        {7,       "Acquisition Error"},
        {8,       "Sync Error"},
        {9,       "Uncal"},
        {15,      "Reference Frequency Error"},
        {16,      "RF Not Available"},
        {17,      "RF Level not Settled"},
        {19,      "Call not Established"},
        {20,      "Call Type not Usable"},
        {21,      "Call Lost"},
        {23,      "Missing Option"},
        {24,      "Invalid RF Setting"},
        {25,      "Level Overrange"},
        {26,      "Resource Conflict"},
        {27,      "No Sensor Connected"},
        {28,      "Unexpected Parameter Change"},
        {30,      "File not Found"},
        {31,      "No DTM reply"},
        {32,      "ACL Disconnected"},
        {40,      "ARB File CRC Error"},
        {42,      "ARB Header Tag Invalid"},
        {43,      "ARB Segment Overflow"},
        {44,      "ARB File not Found"},
        {45,      "ARB Memory Overflow"},
        {46,      "ARB Sample Rate out of Range"},
        {47,      "ARB Cycles out of Range"},
        {70,      "Wrong Standard"},
        {71,      "Wrong Bandwidth"},
        {72,      "Wrong Burst Type"},
        {73,      "MIMO Signal Detected"},
        {74,      "More Streams Than Antennas"},
        {75,      "Matrix Inversion Failed"},
        {76,      "SIG CRC Failed"},
        {77,      "Parity Check Failed"},
        {78,      "Bursts Not Identical"},
        {79,      "Wrong Modulation"},
        {93,      "OCXO Oven Temperature too low"},
        {101,     "Firmware Error"},
        {102,     "Unidentified Error"},
        {103,     "Parameter Error"},
        {104,     "Not Functional"},
        {0, NULL}
    };

    (void)RsCore_LockSession(instrSession);

    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
        if (reliabilityTable[cnt].value == reliabilityCode)
        {
            sprintf(reliabilityMessage, "%s", reliabilityTable[cnt].string);
            if (reliabilityCode)
                error = RSCMWINM_ERROR_RELIABILITY_INDICATOR;
            found = VI_TRUE;
        }
    }

    if (!found)
    {
        sprintf(reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");
        error = RS_WARN_UNKNOWN_INSTRUMENT_STATUS;
    }

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function initiates an instrument reset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIFN      rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIFN      instrument session.
ViStatus _VI_FUNC rscmwinm_Preset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_SetAttributeViString(instrSession, "", RSCMWINM_ATTR_SYSTEM_PRESET, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function switches ON/OFF various error checkings performed inside of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches option checking On or Off.
/// HIPAR rangeChecking/This control enables or disables range checking.
/// HIPAR statusChecking/This control enables or disables instrument state checking
ViStatus _VI_FUNC rscmwinm_ConfigureErrorChecking(ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function clears status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwinm_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the ID Query response string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Defines the size of buffer in argument 'ID Query Response'.
/// HIPAR IDQueryResponse/Returns the ID Query response string.
ViStatus _VI_FUNC rscmwinm_IDQueryResponse(ViSession instrSession,
                                           ViUInt32  bufferSize,
                                           ViChar    IDQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_QueryViString(instrSession, "*IDN?", bufferSize, IDQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Stops further command processing until all commands sent before
/// HIFN  *WAI have been executed.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwinm_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the OPC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR OPC/Queries the OPC.
ViStatus _VI_FUNC rscmwinm_QueryOPC(ViSession instrSession,
                                    ViInt32   *opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwinm_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
/// HIFN Use the function rscmwdau_GetError afterwards to get the detailed information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR? automatically in CheckStatus() when Instrument Status Error is detected.
ViStatus _VI_FUNC rscmwinm_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                       ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}

/// HIFN Session-related mutex switch.
/// HIFN If TRUE (default after init is FALSE), the driver sets a mutual-exclusion mechanism for every driver's hi-level function and attribute access.
/// HIFN Set this value to TRUE if you plan to use one session in more than one thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwls_init or
/// HIPAR instrSession/rscmwls_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch
ViStatus _VI_FUNC rscmwinm_ConfigureMultiThreadLocking(ViSession instrSession,
                                                       ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opcTimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwinm_SetOPCTimeout(ViSession instrSession,
                                         ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32) timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rscmwinm_GetOPCTimeout(ViSession instrSession,
                                         ViUInt32  *timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeViInt32(instrSession, NULL, RS_ATTR_OPC_TIMEOUT, 0, (ViInt32*) timeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function specifies the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwinm_SetVISATimeout(ViSession instrSession,
                                          ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function returns the minimum timeout value to use (in
/// HIFN milliseconds) when accessing the device associated with the
/// HIFN given session.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsspecan_init or
/// HIPAR instrSession/rsspecan_InitWithOptions function.
/// HIPAR VISATimeout/This control gets the timeout value.
ViStatus _VI_FUNC rscmwinm_GetVISATimeout(ViSession instrSession,
                                          ViUInt32  *VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function retrieves and then clears the IVI error information for
/// HIFN  the session or the current execution thread. One exception exists: If
/// HIFN  the BufferSize parameter is 0, the function does not clear the error
/// HIFN  information. By passing 0 for the buffer size, the caller can ascertain
/// HIFN  the buffer size required to get the entire error description string and
/// HIFN  then call the function again with a sufficiently large buffer.
/// HIFN
/// HIFN     If the user specifies a valid IVI session for the instrSession
/// HIFN  parameter, Get Error retrieves and then clears the error information
/// HIFN  for the session.  If the user passes NULL for the instrSession
/// HIFN  parameter, this function retrieves and then clears the error
/// HIFN  information for the current execution thread.  If the instrSession
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error. Normally, the error information describes the first error
/// HIFN  that occurred since the user last called rscmwinm_GetError or
/// HIFN  rscmwinm_ClearError.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
/// HIPAR errorCode/Returns the error code for the session or execution thread.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Description parameter.
/// HIPAR description/Returns the error description for the IVI session or execution thread.
/// HIPAR description/ If there is no description, the function returns an empty string.
ViStatus _VI_FUNC rscmwinm_GetError(ViSession instrSession,
                                    ViStatus* errorCode,
                                    ViInt32   bufferSize,
                                    ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rscmwinm_error_message, errorCode, bufferSize, description)
    );

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rscmwinm_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}


/// HIFN This function is used to read data from the instrument and write
/// HIFN it to a user specified file on the host computer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument
/// HIPAR source/to control computer.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from instrument to control computer applies.
ViStatus _VI_FUNC rscmwinm_ReadToFileFromInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function is used to read data from the host computer and
/// HIFN write it to a user specified file in the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control
/// HIPAR source/computer to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer
/// HIPAR destination/from control computer to instrument applies.
ViStatus _VI_FUNC rscmwinm_WriteFromFileToInstrument(ViSession instrSession,
                                                     ViString  source,
                                                     ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: WriteInstrData and ReadInstrData Functions
 * Purpose:  These functions enable the instrument driver user to
 *           write and read commands directly to and from the instrument.
 *
 *           Note:  These functions bypass the RS attribute state caching.
 *                  WriteInstrData invalidates the cached values for all
 *                  attributes.
 *****************************************************************************/


ViStatus _VI_FUNC rscmwinm_WriteInstrData(ViSession instrSession,
                                          ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

ViStatus _VI_FUNC rscmwinm_ReadInstrData(ViSession instrSession,
                                         ViInt32   numBytes,
                                         ViChar    rdBuf[],
                                         ViUInt32  *bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(rdBuf, numBytes, responseString));
    *bytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN This function queries the ViBoolean value.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value.
/*****************************************************************************
 * Function: Query ViBoolean
 * Purpose:  This function queries the ViBoolean value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_QueryViBoolean(ViSession instrSession,
                                          ViString  cmd,
                                          ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViInt32
 * Purpose:  This function queries the ViInt32 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_QueryViInt32(ViSession instrSession,
                                        ViString  cmd,
                                        ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Query ViReal64
 * Purpose:  This function queries the ViReal64 value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_QueryViReal64(ViSession instrSession,
                                         ViString  cmd,
                                         ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: Query ViString
 * Purpose:  This function queries the ViString value.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_QueryViString(ViSession instrSession,
                                         ViString  cmd,
                                         ViInt32   bufferSize,
                                         ViChar    _VI_FAR value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1), 2,
        "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, value, NULL));

    checkErr(rscmwinm_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/*****************************************************************************
 * Function: rscmwinm_SetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to set
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_SetAttributeViInt32(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to set the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwinm_SetAttributeViInt64(ViSession instrSession,
                                               ViString  channelName,
                                               ViAttr    attributeId,
                                               ViInt64   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwinm_SetAttributeViReal64(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwinm_SetAttributeViString(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a RawString attribute. It is also used
/// HIFN  to set an event-type attribute (attribute without a parameter). In this
/// HIFN  case the string value is ignored.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes. If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible. They handle order dependencies and
/// HIFN  multithread locking for you. In addition, they perform status checking
/// HIFN  only after setting all of the attributes. In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change. Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwinm_SetAttributeRawString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViRawString(instrSession, repeatedCapabilityName,
                                           attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwinm_SetAttributeViBoolean(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwinm_SetAttributeViSession(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViSession value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwinm_GetAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to get
 *           attribute values directly.  There are typesafe versions for
 *           ViInt32, ViReal64, ViString, ViBoolean, and ViSession attributes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwinm_GetAttributeViInt32(ViSession instrSession,
                                               ViString  channelName,
                                               ViUInt32  attributeId,
                                               ViInt32   *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViInt64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rsscope_init or
/// HIPAR instrSession/rsscope_InitWithOptions function.
/// HIPAR channelName/If the attribute is channel-based, this parameter specifies the name
/// HIPAR channelName/of the channel on which to obtain the value of the attribute. If the
/// HIPAR channelName/attribute is not channel-based, then pass NULL or an empty string.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR value/Returns the current value of the attribute.  Pass the address of a
/// HIPAR value/ViInt64 variable.
ViStatus _VI_FUNC rscmwinm_GetAttributeViInt64(ViSession instrSession,
                                               ViString  channelName,
                                               ViAttr    attributeId,
                                               ViInt64   *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViInt64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwinm_GetAttributeViReal64(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViReal64  *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwinm_GetAttributeViString(ViSession instrSession,
                                                ViString  channelName,
                                                ViUInt32  attributeId,
                                                ViInt32   bufSize,
                                                ViChar    value[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, bufSize,
                                        value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a RawString attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIFN
/// HIFN     You must provide a ViChar array to serve as a buffer for the value.
/// HIFN  You pass the number of bytes in the buffer as the Buffer Size
/// HIFN  parameter. If the current value of the attribute, including the
/// HIFN  terminating NUL byte, is larger than the size you indicate in the
/// HIFN  Buffer Size parameter, the function copies Buffer Size - 1 bytes into
/// HIFN  the buffer, places an ASCII NUL byte at the end of the buffer, and
/// HIFN  returns the buffer size you must pass to get the entire value.  For
/// HIFN  example, if the value is "123456" and the Buffer Size is 4, the
/// HIFN  function places "123" into the buffer and returns 7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute. The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rscmwinm_GetAttributeRawString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawString(instrSession, repeatedCapabilityName,
                                           attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwinm_GetAttributeViBoolean(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViBoolean *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwinm_GetAttributeViSession(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViSession *value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwdau_init or
/// HIPAR instrSession/rscmwdau_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rscmwinm_GetAttributeRepCapName(ViSession instrSession,
                                                  ViUInt32  attributeId,
                                                  ViString  repCapNameId,
                                                  ViInt32   index,
                                                  ViInt32   bufferSize,
                                                  ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Rs_GetAttributeRepCapNameId
 * Purpose:  This function returns the repeated capability id(s) that
 *           belongs to the attribute you specify. More then one id is listed
 *           in comma separated string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                             ViUInt32  attributeID,
                                                             ViInt32   bufferSize,
                                                             ViChar    _VI_FAR repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: Get Attribute Repeated Capability Id Name(s)
 * Purpose:  This function returns the repeated capability name(s) that
 *           belongs to the attribute and single repeated capability id you
 *           may specify. More then one name is listed in comma separated
 *           string.
 *****************************************************************************/
ViStatus _VI_FUNC rscmwinm_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    _VI_FAR repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/*****************************************************************************
 * Function: rscmwinm_CheckAttribute<type> Functions
 * Purpose:  These functions enable the instrument driver user to check
 *           attribute values directly.  These functions check the value you
 *           specify even if you set the rscmwinm_ATTR_RANGE_CHECK
 *           attribute to VI_FALSE.  There are typesafe versions for ViInt32,
 *           ViReal64, ViString, ViBoolean, and ViSession datatypes.
 *****************************************************************************/

/* ViInt32 */
ViStatus _VI_FUNC rscmwinm_CheckAttributeViInt32(ViSession instrSession,
                                                 ViString  channelName,
                                                 ViUInt32  attributeId,
                                                 ViInt32   value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViInt32(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViReal64 */
ViStatus _VI_FUNC rscmwinm_CheckAttributeViReal64(ViSession instrSession,
                                                  ViString  channelName,
                                                  ViUInt32  attributeId,
                                                  ViReal64  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViReal64(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViString */
ViStatus _VI_FUNC rscmwinm_CheckAttributeViString(ViSession instrSession,
                                                  ViString  channelName,
                                                  ViUInt32  attributeId,
                                                  ViString  value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViString(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViBoolean */
ViStatus _VI_FUNC rscmwinm_CheckAttributeViBoolean(ViSession instrSession,
                                                   ViString  channelName,
                                                   ViUInt32  attributeId,
                                                   ViBoolean value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViBoolean(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/* ViSession */
ViStatus _VI_FUNC rscmwinm_CheckAttributeViSession(ViSession instrSession,
                                                   ViString  channelName,
                                                   ViUInt32  attributeId,
                                                   ViSession value)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViSession(instrSession, channelName, attributeId, RS_VAL_DIRECT_USER_CALL, value);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function performs the following operations:
/// HIFN
/// HIFN     - Closes the instrument I/O session.
/// HIFN
/// HIFN     - Destroys the instrument driver session and all of its attributes.
/// HIFN
/// HIFN     - Deallocates any memory resources the driver uses.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) You must unlock the session before calling rscmwinm_close.
/// HIFN
/// HIFN     (2) After calling rscmwinm_close, you cannot use the instrument
/// HIFN  driver again until you call rscmwinm_init or
/// HIFN  rscmwinm_InitWithOptions.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwinm_init or
/// HIPAR instrSession/rscmwinm_InitWithOptions function.  The handle identifies a
/// HIPAR instrSession/particular instrument session.
ViStatus _VI_FUNC rscmwinm_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
