/*****************************************************************************
 *  Rohde&Schwarz CMW500 Wideband Radio Communication Tester instrument driver
 *  NB IoT Signaling
 *
 *  Original Release: June 2019
 *  By: Petr Kominek
 *
 *  Should you have any technical questions please contact the hotline of
 *  Rohde & Schwarz Vertriebs-GmbH Rohde & Schwarz Support Center
 *
 *  e-mail: CustomerSupport@rohde-schwarz.com
 *
 *  Modification History:
 *     see ChangeLog
 *
 *****************************************************************************/
#include "rscmwins.h"

#if (defined(WIN32) || defined(_WIN32) || defined(__WIN32__) || defined(__NT__))
#define ANSI
#endif
#ifndef ANSI                /* UNIX compatible */
    #include <sys/timeb.h>
    #include <sys/types.h>
    #include <unistd.h>
    #define Sleep sleep
#else                       /* ANSI compatible */
#include <windows.h>
#endif

/*****************************************************************************
 *- Value Definition and Range Tables ---------------------------------------*
 *****************************************************************************/
static ViString logMessageCategoryArr[] = {"INFO", "WARN", "ERR", "CONT", NULL};
static ViString rfInputConnectorArr[] = {
    "RF1C", "RF2C", "RF3C", "RF4C", "RFAC", "RFBC", "RF1O", "RF3O", "RFAO",
    "R11C", "R12C", "R13C", "R14C", "R21C", "R22C", "R23C", "R24C",
    "R31C", "R32C", "R33C", "R34C", "R41C", "R42C", "R43C", "R44C",
    "R11O", "R13O", "R21O", "R23O", "R31O", "R33O", "R41O", "R43O", NULL
};
static ViString rfInputConverterArr[] = {"RX1", "RX2", "RX3", "RX4", NULL};
static ViString rfOutputConnectorArr[] = {
    "RF1C", "RF1O", "RF2C", "RF3C", "RF3O", "RF4C", "RFAO", "RFBC", "IQ2O",
    "IQ4O", "IQ6O", "IQ8O", "R118", "R218", "R214", "R258", "R318", "R418", "R11C", "R11O", "R12C", "R13C", "R13O",
    "R14C", "I12O", "I14O", "I16O", "I18O", "R21C", "R21O", "R22C", "R23C", "R23O", "R24C", "I22O", "I24O", "I26O",
    "I28O", "R31C", "R31O", "R32C", "R33C", "R33O", "R34C", "I32O", "I34O", "I36O", "I38O", "R41C", "R41O", "R42C",
    "R43C", "R43O", "R44C", "I42O", "I44O", "I46O", "I48O", "R1183", "R1184", "R11O3", "R11O4", "RFAC", "RA18",
    "RB14", "RB18",NULL
};
static ViString rfOutputConverterArr[] = {"TX1", "TX2", "TX3", "TX4", NULL};
static ViString signalingUnitArr[] = {"SUA1", "SUA2", "SUA3", "SUA4", "SUA5", "SUA6", NULL};
static ViString stateArr[] = {"OFF", "ON", NULL};
static ViString syncStateArr[] = {"PEND", "ADJ", NULL};
static ViString categoryArr[] = {"INFO", "WARN", "ERR", "CONT", NULL};
static ViString scenarioArr[] = {"NAV", "SCEL", NULL};

static ViString bandsArr[] = {
    "UDEF", "OB1", "OB2", "OB3", "OB4", "OB5", "OB7", "OB8", "OB9", "OB10",
    "OB11", "OB12", "OB13", "OB14", "OB17", "OB18", "OB19", "OB20",
    "OB21", "OB22", "OB24", "OB25", "OB26", "OB27", "OB28", "OB30",
    "OB31", "OB65", "OB66", "OB68", "OB70", "OB71", "OB72", "OB73",
    "OB74", "OB85",NULL
};
static ViString yesNoArr[] = {"NO", "YES", NULL};
static ViString resultTypeArr[] = {"ABS", "REL", NULL};
static ViString directionArr[] = {"DL", "UL", NULL};

static ViString subcarriersArr[] = {"SC1", "SC3", "SC6", "SC12", NULL};

static ViString resourceUnitsArr[] = {"NRU01", "NRU02", "NRU03", "NRU04", "NRU05", "NRU06", "NRU08", "NRU10", NULL};
static ViString MCSIndexArr[] = {"MI0", "MI1", "MI2", "MI3", "MI4", "MI5", "MI6", "MI7", "MI8", "MI9", "MI10", "MI11", "MI12", NULL};
static ViString modulationArr[] = {"QPSK", "BPSK", NULL};
static ViString repetitionsArr[] = {"NR1", "NR2", "NR4", "NR8", "NR16", "NR32", "NR64", "NR128", NULL};

/*****************************************************************************
 *- User-Callable Functions (Exportable Functions) --------------------------*
 *****************************************************************************/


/// HIFN  This function performs the following initialization actions:
/// HIFN
/// HIFN     - Creates a new instrument driver session.
/// HIFN
/// HIFN     - Opens a session to the specified device using the interface and
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries
/// HIFN  the instrument ID and checks that it is valid for this instrument
/// HIFN  driver.
/// HIFN
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the
/// HIFN  instrument to a known state.
/// HIFN
/// HIFN     - Sends initialization commands to set the instrument to the state
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN
/// HIFN     - Returns a ViSession handle that you use to identify the instrument
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     This function creates a new session each time you invoke it. Although
/// HIFN  you can open more than one session for the same resource, it is best
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR idQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the
/// HIPAR resetDevice/initialization procedure.
/// HIPAR instrSession/Returns a ViSession handle that you use to identify the instrument in
/// HIPAR instrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rscmwins_init(
    ViRsrc resourceName,
    ViBoolean idQuery,
    ViBoolean resetDevice,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER,
                                  VI_ERROR_PARAMETER4, "Null address for Instrument Handle");

        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    checkErr(rscmwins_InitWithOptions (resourceName, idQuery, resetDevice, "", newInstrSession));

Error:
    return error;
}


/// HIFN  This function performs the following initialization actions:
/// HIFN
/// HIFN     - Creates a new instrument driver session and optionally sets the
/// HIFN  initial state of the following session attributes:
/// HIFN
/// HIFN         RSCMWINS_ATTR_RANGE_CHECK
/// HIFN         RSCMWINS_ATTR_QUERY_INSTRUMENT_STATUS
/// HIFN
/// HIFN     - Opens a session to the specified device using the interface and
/// HIFN  address you specify for the Resource Name parameter.
/// HIFN
/// HIFN     - If the ID Query parameter is set to VI_TRUE, this function queries
/// HIFN  the instrument ID and checks that it is valid for this instrument
/// HIFN  driver.
/// HIFN
/// HIFN     - If the Reset parameter is set to VI_TRUE, this function resets the
/// HIFN  instrument to a known state.
/// HIFN
/// HIFN     - Sends initialization commands to set the instrument to the state
/// HIFN  necessary for the operation of the instrument driver.
/// HIFN
/// HIFN     - Returns a ViSession handle that you use to identify the instrument
/// HIFN  in all subsequent instrument driver function calls.
/// HIFN
/// HIFN     Note:
/// HIFN
/// HIFN     This function creates a new session each time you invoke it. Although
/// HIFN  you can open more than one session for the same resource, it is best
/// HIFN  not to do so. You can use the same session in multiple program threads.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR resourceName/Pass the resource name of the device to initialize.
/// HIPAR idQuery/Specify whether you want the instrument driver to perform an ID Query.
/// HIPAR resetDevice/Specify whether you want the to reset the instrument during the
/// HIPAR resetDevice/initialization procedure.
/// HIPAR optionString/You can use this control to set the initial value of certain
/// HIPAR optionString/attributes for the session. The following table lists the attributes
/// HIPAR optionString/and the name you use in this parameter to identify the attribute.
/// HIPAR instrSession/Returns a ViSession handle that you use to identify the instrument in
/// HIPAR instrSession/all subsequent instrument driver function calls.
ViStatus _VI_FUNC rscmwins_InitWithOptions(
    ViRsrc resourceName,
    ViBoolean idQuery,
    ViBoolean resetDevice,
    ViString optionString,
    ViSession* newInstrSession
)
{
    ViStatus error = VI_SUCCESS;
    RsCoreAttributePtr* attrList = NULL;
    ViSession instrSession = 0;
    RsCoreSessionPtr sessionProperties = NULL;

    if (newInstrSession == NULL)
    {
        (void)RsCore_SetErrorInfo(0, VI_FALSE, RS_ERROR_INVALID_PARAMETER, VI_ERROR_PARAMETER5,
                                  "Null address for Instrument Handle");
        checkErr(RS_ERROR_INVALID_PARAMETER);
    }

    attrList = (RsCoreAttributePtr *)rscmwins_RsCoreAttributeList;

    *newInstrSession = 0;

    checkErr(RsCore_NewSpecificDriver(
        resourceName, // VISA Resource Name
        "rscmwins", // driver prefix
        optionString, // Init Options string, is applied to the session settings
        attrList, // List of all attributes from the rsxxx_AttrPropertiesList
        VI_FALSE, //idnModelFullName Model has the full *IDN? query name e.g. true: "CMW100A", VI_FALSE: "CMW" Used in CheckInstrument()
        0, // WriteDelay
        0, // ReadDelay
        100000, // IO Segment Size
        RS_VAL_OPCWAIT_STBPOLLING, // OPC Wait Mode
        RSCMWINS_OPC_TIMEOUT, // OPC timeout
        15000, // VISA Timeout
        600000, // Self-test timeout
        RS_VAL_BIN_FLOAT_FORMAT_SINGLE_4BYTES, // BinaryFloatNumbersFormat
        RS_VAL_BIN_INTEGER_FORMAT_INT32_4BYTES, // binaryIntegerNumbersFormat
        &instrSession));

    // Minimal allowed rscore version is 3.12.0
    checkErr(RsCore_CheckMinimalEngineVersion(instrSession, 3, 12, 0));

    checkErr(RsCore_GetRsSession(instrSession, &sessionProperties));

    // No SCPI command has been sent yet.
    // Call viClear before sending any SCPI command
    checkErr(RsCore_ViClear(instrSession));

    /* --- Here perform settings that are default for this driver,
    but can be overwritten by the optionsString settings */

    sessionProperties->autoSystErrQuery = VI_FALSE;

    // Parse option string and optionally sets the initial state of the following session attributes
    checkErr(RsCore_ApplyOptionString(instrSession, optionString));
    checkErr(RsCore_BuildRepCapTable(instrSession, rscmwins_RsCoreRepCapTable));

    // Query *IDN? string, parse it and set the following attributes:
    // -RS_ATTR_INSTRUMENT_MANUFACTURER
    // - RS_ATTR_INSTRUMENT_MODEL - based on the idnModelFullName true: "CMW100A", false : "CMW"
    // - RS_ATTR_INSTRUMENT_FIRMWARE_REVISION
    checkErr(RsCore_QueryAndParseIDNstring(instrSession, RSCMWINS_SIMULATION_ID_QUERY, NULL));

    // Query *OPT? string, parse the options, remove the duplicates, sort them and store the result string to RS_ATTR_OPTIONS_LIST
    checkErr(RsCore_QueryAndParseOPTstring(instrSession, RSCMWINS_SIMULATION_OPT_QUERY, RS_VAL_OPTIONS_PARSING_AUTO));

    // Default Instrument Setup + optional *RST
    if (resetDevice == VI_TRUE)
    {
        checkErr(rscmwins_reset(instrSession));
    }
    else
    {
        checkErr(rscmwins_DefaultInstrSetup(instrSession));
    }

    if (idQuery == VI_TRUE)
    {
        checkErr(RsCore_FitsIDNpattern(instrSession, RSCMWINS_VALID_ID_RESPONSE_STRING, NULL));
    }

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    if (error < VI_SUCCESS)
    {
        if (error == RS_ERROR_INSTRUMENT_STATUS && sessionProperties->autoSystErrQuery == VI_FALSE)
        {
            sessionProperties->autoSystErrQuery = VI_TRUE;
            (void)rscmwins_CheckStatus(instrSession);
            sessionProperties->autoSystErrQuery = VI_FALSE;
        }

        if (instrSession != 0)
            (void)RsCore_ViClose(instrSession);
    }
    else
    {
        *newInstrSession = instrSession;
    }

    return error;
}


/// HIFN  Turns the generator (the cell) on or off.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Turns the generator (the cell) on or off.
ViStatus _VI_FUNC rscmwins_ConfigureGeneratorState(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_GENERATOR_STATE, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns detailed information about the 'NB-IoT Signaling' generator
/// HIFN  state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mainState/Valid Values:
/// HIPAR synchronizationState/Valid Values:
ViStatus _VI_FUNC rscmwins_QueryCellStateAll(ViSession instrSession,
                                             ViInt32   instance,
                                             ViInt32*  mainState,
                                             ViInt32*  syncState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SOUR:NIOT:SIGN%ld:CELL:STAT:ALL?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *mainState = RsCore_FindStringIndex(stateArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *syncState = RsCore_FindStringIndex(syncStateArr, p2buf);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries whether an RRC connection is established (connected) or not
/// HIFN  (idle).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR rrcState/Queries whether an RRC connection is established (connected) or not
/// HIPAR rrcState/(idle).
ViStatus _VI_FUNC rscmwins_QueryRRCState(ViSession instrSession,
                                         ViInt32   instance,
                                         ViInt32*  rrcState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_RRC_STATE, rrcState),
            3, "RRC State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the behavior for the state transitions of an RRC connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR releaseCause/Selects the behavior for the state transitions of an RRC connection.
ViStatus _VI_FUNC rscmwins_ConfigureRRCReleaseCause(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   releaseCause)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_RRC_RELEASE_CAUSE, releaseCause),
            3, "Release Cause");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Controls the PS state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR psAction/Controls the PS state.
ViStatus _VI_FUNC rscmwins_ConfigurePSAction(ViSession instrSession,
                                             ViInt32   instance,
                                             ViInt32   psAction)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PS_ACTION, psAction),
            3, "PS Action");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the PS state.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR psState/Queries the PS state.
ViStatus _VI_FUNC rscmwins_QueryPSState(ViSession instrSession,
                                        ViInt32   instance,
                                        ViInt32*  psState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_PS_STATE, psState),
            3, "PS State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Lists current scenarios.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR catalog/Lists current scenarios.
ViStatus _VI_FUNC rscmwins_ScenarioCatalog(ViSession instrSession,
                                           ViInt32   instance,
                                           ViInt32   arraySize,
                                           ViChar    _VI_FAR catalog[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_SCENARIO_CATALOG, catalog),
            4, "Catalog");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the active scenario.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR active/Returns the active scenario.
ViStatus _VI_FUNC rscmwins_ScenarioActive(ViSession instrSession,
                                          ViInt32   instance,
                                          ViInt32*  active)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCENARIO_ACTIVE, active),
            3, "Active");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the latest entry of the event log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of generator firmware application.
/// HIPAR timeStamp/Returns the time stamp of the last entry in the format "hh:mm:ss".
/// HIPAR category/Returns the category of the entry, as indicated in the main view by an
/// HIPAR category/icon.
/// HIPAR event/Returns the text string describing the event, e.g. "Session Opened".
ViStatus _VI_FUNC rscmwins_QueryEventLogLastEntry(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   timestampBufferSize,
                                                  ViInt32   eventBufferSize,
                                                  ViChar    timeStamp[],
                                                  ViInt32*  category,
                                                  ViChar    event[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:ELOG:LAST?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    strncpy(timeStamp, p2buf, timestampBufferSize);

    p2buf = strtok(NULL, ",\n");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *category = RsCore_FindStringIndex(categoryArr, p2buf);
    if (*category < 0)
        return RSCMWINS_ERROR_INCORRECT_STATE;

    p2buf = strtok(NULL, ",\n");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    strncpy(event, p2buf, eventBufferSize);

    strtok(NULL, ",\n");

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN  Queries all entries of the event log.
/// HIFN     For each entry, three parameters are returned, from oldest to latest
/// HIFN  entry: {<Timestamp>, <Category>, <Event>}entry 1, {<Timestamp>,
/// HIFN  <Category>, <Event>}entry 2
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR timestamp/Valid Values:
/// HIPAR category/Valid Values:
/// HIPAR event/Valid Values:
ViStatus _VI_FUNC rscmwins_QueryEventLogAllEntries(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViInt32   bufferSize,
                                                   ViChar    _VI_FAR timeStamp[],
                                                   ViInt32   eventCategory[],
                                                   ViChar    _VI_FAR message[],
                                                   ViInt32*  numberOfEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  pbuffer = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  TSBufferSize = bufferSize - 1;
    ViInt32  MsgBufferSize = bufferSize - 1;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:ELOG:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &pbuffer));
    checkErr(rscmwins_CheckStatus(instrSession));

    timeStamp[0] = '\0';
    message[0] = '\0';
    p2buf = strtok(pbuffer, ",\r\n");
    while (p2buf && (i < arraySize))
    {
        if (TSBufferSize)
        {
            strncat(timeStamp, p2buf, TSBufferSize);
            TSBufferSize -= (ViInt32)strlen(p2buf);

            strncat(timeStamp, ",", TSBufferSize--);
        }

        p2buf = strtok(NULL, ",\r\n");
        eventCategory[i] = RsCore_FindStringIndex(logMessageCategoryArr, p2buf);

        p2buf = strtok(NULL, ",\r\n");
        if (MsgBufferSize)
        {
            strncat(message, p2buf, MsgBufferSize);
            MsgBufferSize -= (ViInt32)strlen(p2buf);

            strncat(message, ",", MsgBufferSize--);
        }

        i++;
        p2buf = strtok(NULL, ",\r\n");
    }
    if (strlen(message) > 0)
        message[strlen(message) - 1] = '\0';
    if (strlen(timeStamp) > 0)
        timeStamp[strlen(timeStamp) - 1] = '\0';
    *numberOfEntries = i;

Error:
    if (pbuffer != NULL)
        free(pbuffer);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Clears the event log.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_CleanEventLog(ViSession instrSession,
                                         ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CLEAN_EVENT_LOG, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the IMEI of the UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR imsi/Queries the IMSI of the UE.
/// HIPAR imei/Queries the IMEI of the UE.
ViStatus _VI_FUNC rscmwins_QueryUEInfo(ViSession instrSession,
                                       ViInt32   instance,
                                       ViInt32   arraySize,
                                       ViChar    _VI_FAR imsi[],
                                       ViChar    _VI_FAR imei[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_INFO_IMEI, imei),
            4, "IMEI");

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_INFO_IMSI, imsi),
            5, "IMSI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the IDs of all established default bearers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR IDs/Queries the IDs of all established default bearers.
ViStatus _VI_FUNC rscmwins_QueryUEInfoDefaultBearersCatalog(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   arraySize,
                                                            ViChar    _VI_FAR IDs[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_INFO_DEFAULT_BEARERS_CATALOG, IDs),
            4, "IDs");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the IPv4 addresses (<version> = 4) or the IPv6 prefixes
/// HIFN  (<version> = 6) assigned to the UE by the R&S CMW.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR version/Sets the version parameter.
/// HIPAR ipAddresses/Returns the IPv4 addresses (<version> = 4) or the IPv6 prefixes
/// HIPAR ipAddresses/(<version> = 6) assigned to the UE by the R&S CMW.
ViStatus _VI_FUNC rscmwins_QueryUEInfoAssignedIPAddresses(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   arraySize,
                                                          ViInt32   version,
                                                          ViChar    _VI_FAR ipAddresses[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap_ipv[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, version, 4, 6),
            4, "Version");

    snprintf(repCap_ipv, RS_REPCAP_BUF_SIZE, "%s,%ld", repCap, version);

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_INFO_ASSIGNED_IP_ADDRESSES, ipAddresses),
            5, "IP Addresses");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the UE RRC connection requests.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR multiTone/Returns whether the UE supports NPUSCH multi-tone transmissions, as
/// HIPAR multiTone/indicated by the UE in an RRC connection request.
/// HIPAR multiCarrier/Returns whether the UE supports multi-carrier operation, as indicated
/// HIPAR multiCarrier/by the UE in an RRC connection request.
/// HIPAR cIoTOptimization/Returns whether the UE supports user plane C-IoT EPS optimization, as
/// HIPAR cIoTOptimization/indicated by the UE in an RRC connection setup complete message.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesRRCConnectionEstablishment(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32*  multiTone,
                            ViInt32*  multiCarrier,
                            ViInt32*  cIoTOptimization)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTI_TONE, multiTone),
            3, "Multi Tone");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTI_CARRIER, multiCarrier),
            4, "Multi Carrier");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_USER_PLANE_C_IOT_OPTIMIZATION, cIoTOptimization),
            5, "c-IoT Optimization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the General UE Capabilities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR release/Returns the access stratum release supported by the UE. In Release 13,
/// HIPAR release/the allowed value is fixed.
/// HIPAR ueCategory/Returns the reported UE category.
/// HIPAR multipleDRB/Returns whether the UE supports multiple data radio bearers.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesGeneral(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32*  release,
                                                      ViInt32*  ueCategory,
                                                      ViInt32*  multipleDRB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_RELEASE, release),
            4, "Release");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_TRANSFER_CATEGORY, ueCategory),
            5, "UE Category");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTIPLE_DRB, multipleDRB),
            6, "Multiple DRB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns RF UE Capabilities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR multipleNSPmax/Returns whether the UE supports the handling of multiple NS-Pmax
/// HIPAR multipleNSPmax/values.
/// HIPAR powerClass14dBm/Indicates whether the UE supports a power class 6 with the maximum
/// HIPAR powerClass14dBm/output power of 14 dBm.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesRF(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32*  multipleNSPmax,
                                                 ViInt32*  powerClass14dBm)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTIPLE_NS_PMAX, multipleNSPmax),
            5, "Multiple NS Pmax");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_POWER_CLASS_14_DBM, powerClass14dBm),
            6, "Power Class 14 dBm");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns all operating bands supported by the UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arrySize/Sets the array size parameter.
/// HIPAR operatingBands/Returns all operating bands supported by the UE.
/// HIPAR returnedValues/Displays the returned values.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesRFOperatingBands(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   _VI_FAR operatingBands[],
                            ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = "";
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:UEC:TRAN:RF:BAND?", instance);
    checkErr(rscmwins_QueryViString(instrSession, cmd, RS_MAX_MESSAGE_BUF_SIZE, response));

    p2buf = strtok(response, ",");

    while (p2buf)
    {
        if (i < arraySize)
        {
            operatingBands[i] = RsCore_FindStringIndex(bandsArr, response);
            p2buf = strtok(NULL, ",\r\n");
        }

        i++;
    }

    if (returnedValues)
        *returnedValues = i;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns all operating bands supported by the UE and whether the UE
/// HIFN  supports a power class with 20 dBm in a band.
/// HIFN     For each supported band, a pair of values is returned: {<Bands>,
/// HIFN  <PowerClass>}1, {<Bands>, <PowerClass>}2
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR bands/The UE supports all listed operating bands.
/// HIPAR powerClass/Support of power class with 20 dBm in the band.
/// HIPAR returnedValues/Displays the returned values.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesRFBandsAll(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViInt32   arraySize,
                                                         ViInt32   bands[],
                                                         ViInt32   powerClass[],
                                                         ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf = "";
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:UEC:TRAN:RF:BAND:ALL?", instance);
    checkErr(rscmwins_QueryViString(instrSession, cmd, RS_MAX_MESSAGE_BUF_SIZE, response));

    p2buf = strtok(response, ",");

    while (p2buf != NULL)
    {
        if (i < arraySize)
        {
            bands[i] = RsCore_FindStringIndex(bandsArr, p2buf);

            p2buf = strtok(NULL, ",");
            if (p2buf == NULL)
                return RSCMWINS_ERROR_INCORRECT_STATE;
            powerClass[i] = RsCore_FindStringIndex(yesNoArr, p2buf);

            p2buf = strtok(NULL, ",");
        }
        i++;
    }

    if (returnedValues)
        *returnedValues = i;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns whether the UE supports a power class with 20 dBm.
/// HIFN     The information is returned per supported band. The values have the
/// HIFN  same order as the bands returned by the command
/// HIFN  SENSe:NIOT:SIGN:UECapability:TRANsfer:RF:BANDs?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR powerClasses/Returns whether the UE supports a power class with 20 dBm.
/// HIPAR returnedValues/Displays the returned values.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesRFPowerClasses(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   arraySize,
                                                             ViInt32   powerClasses[],
                                                             ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViInt32  i = 0;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:UEC:TRAN:RF:POWC?", instance);
    checkErr(rscmwins_QueryViString(instrSession, cmd, RS_MAX_MESSAGE_BUF_SIZE, response));

    p2buf = strtok(NULL, ",");

    while (p2buf != NULL)
    {
        if (i < arraySize)
        {
            powerClasses[i] = RsCore_FindStringIndex(yesNoArr, p2buf);
            p2buf = strtok(NULL, ",");
        }
        i++;
    }

    if (returnedValues)
        *returnedValues = i;

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the PDCP UE Capabilities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR supportedROHCProfiles/Returns which ROHC profiles the UE supports.
/// HIPAR maxROHCContextSessions/Returns the maximum number of header compression context sessions
/// HIPAR maxROHCContextSessions/supported by the UE.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesPDCP(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   arraySize,
                                                   ViChar    _VI_FAR supportedROHCProfiles[],
                                                   ViInt32*  maxROHCContextSessions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_ROHC_PROFILES, supportedROHCProfiles),
            4, "Supported ROHC Profiles");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MAX_ROHC_CONTEXT_SESSIONS, maxROHCContextSessions),
            5, "Max ROHC Context Sessions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a list of ROHC profile support information.
/// HIFN     For each profile, a pair of values is returned: {<Profiles>,
/// HIFN  <Enable>}1, {<Profiles>, <Enable>}2,
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the Profile.
/// HIPAR profiles/Name of the profile, as string, for example 'Profile0x0002'.
/// HIPAR enable/Displays the support of profile.
/// HIPAR numberOfEntries/Number of returned entries.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesAllROHCProfiles(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViInt32   bufferSize,
                            ViChar    _VI_FAR profile[],
                            ViInt32   enable[],
                            ViInt32*  numberOfEntries)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  response = NULL;
    ViChar*  p2buf;
    ViInt32  i = 0;
    ViInt32  profileBufferSize = bufferSize - 1;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "SENS:NIOT:SIGN%ld:UEC:TRAN:PDCP:ROHC:ALL?", instance);
    checkErr(RsCore_QueryViStringUnknownLength(instrSession, cmd, &response));

    checkErr(rscmwins_CheckStatus(instrSession));

    profile[0] = '\0';
    p2buf = strtok(response, ",");

    while (p2buf && (i < arraySize))
    {
        if (profileBufferSize)
        {
            strncat(profile, p2buf, profileBufferSize);
            profileBufferSize -= (ViInt32)strlen(p2buf);

            strncat(profile, ",", profileBufferSize--);
        }

        p2buf = strtok(NULL, ",\r\n");

        enable[i] = RsCore_FindStringIndex(yesNoArr, p2buf);

        i++;
        p2buf = strtok(NULL, ",\r\n");
    }

    if (strlen(profile) > 0)
        profile[strlen(profile) - 1] = '\0';

    if (numberOfEntries)
        *numberOfEntries = i;

Error:
    if (response != NULL)
        free(response);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns a list of ROHC profile names, as reference for the command
/// HIFN  SENSe:NIOT:SIGN:UECapability:TRANsfer:PDCP:PROFiles?.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR arraySize/Sets the array size parameter.
/// HIPAR profileNames/Returns a list of ROHC profile names, as reference for the command
/// HIPAR profileNames/SENSe:NIOT:SIGN:UECapability:TRANsfer:PDCP:PROFiles?.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesReferenceROHCProfileNames(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   arraySize,
                            ViChar    _VI_FAR profileNames[])
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, repCap, arraySize,
            RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_REFERENCE_ROHC_PROFILE_NAMES, profileNames),
            4, "Profile Names");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  QUeries MAC UE Capabilities.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR inactivityMonitor/Indicates whether the UE supports data inactivity monitoring.
/// HIPAR releaseAssistanceIndication/Indicates whether the UE supports release assistance indication.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesMAC(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32*  inactivityMonitor,
                                                  ViInt32*  releaseAssistanceIndication)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_INACTIVITY_MONITOR, inactivityMonitor),
            3, "Inactivity Monitor");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_RELEASE_ASSISTANCE_INDICATION, releaseAssistanceIndication),
            4, "Release Assistance Indication");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the UE Capabilities for physical layer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR multiTone/Returns whether the UE supports NPUSCH multi-tone transmissions, as
/// HIPAR multiTone/indicated by the UE in the information element UE-Capability-NB.
/// HIPAR multiCarrier/Returns whether the UE supports multi-carrier operation, as indicated
/// HIPAR multiCarrier/by the UE in the information element UE-Capability-NB.
/// HIPAR multiCarrierNPRACH/Indicates whether the UE supports NPRACH on non-anchor carrier, as
/// HIPAR multiCarrierNPRACH/specified in 3GPP TS 36.321 and 36.331.
/// HIPAR twoHARQProcesses/Indicates whether the UE supports two HARQ processes in DL and UL.
/// HIPAR interferenceRandomization/Indicates whether the UE supports interference randomization in
/// HIPAR interferenceRandomization/connected mode as specified in 3GPP TS 36.211.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesPhysicalLayer(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32*  multiTone,
                                                            ViInt32*  multiCarrier,
                                                            ViInt32*  multiCarrierNPRACH,
                                                            ViInt32*  twoHARQProcesses,
                                                            ViInt32*  interferenceRandomization)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_NPUSCH_MULTI_TONE, multiTone),
            3, "Multi Tone");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_UE_SUPPORTS_MULTI_CARRIER, multiCarrier),
            4, "Multi Carrier");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTI_CARRIER_NPRACH, multiCarrierNPRACH),
            5, "Multi Carrier NPRACH");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_TWO_HARQ_PROCESSES, twoHARQProcesses),
            6, "Two HARQ Processes");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_INTERFERENCE_RANDOMIZATION, interferenceRandomization),
            7, "Interference Randomization");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Indicates whether the UE supports paging on non-anchor carriers, as
/// HIFN  specified in 3GPP TS 36.331 and 36.304.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR multiCarrierPaging/Indicates whether the UE supports paging on non-anchor carriers, as
/// HIPAR multiCarrierPaging/specified in 3GPP TS 36.331 and 36.304.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesMultiCarrierPaging(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32*  multiCarrierPaging)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_MULTI_CARRIER_PAGING, multiCarrierPaging),
            3, "Multi Carrier Paging");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns whether the UE supports wake up signal.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR wakeUpSignal/Returns whether the UE supports wake up signal.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesPagingWakeUpSignal(
                            ViSession  instrSession,
                            ViInt32    instance,
                            ViBoolean* wakeUpSignal)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_PAGING_WAKE_UP_SIGNAL, wakeUpSignal),
            3, "Wake Up Signal");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Indicates the minimum gap the UE supports between WUS and associated
/// HIFN  paging occasion (PO) for eDRX as specified by 3GPP in TS 36.213 and TS
/// HIFN  36.304.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR minGap/Indicates the minimum gap the UE supports between WUS and associated
/// HIPAR minGap/paging occasion (PO) for eDRX as specified by 3GPP in TS 36.213 and TS
/// HIPAR minGap/36.304.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesPagingWUSMinGap(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64* minGap)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_PAGING_WUS_MIN_GAP, minGap),
            3, "Min Gap");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Indicates whether the UE supports early data transmission in user
/// HIFN  plane (UP) C-IoT EPS optimized mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR earlyDataup/Indicates whether the UE supports early data transmission in user
/// HIPAR earlyDataup/plane (UP) C-IoT EPS optimized mode.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesEarlyDataUp(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32*  earlyDataup)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_EARLY_DATA_UP, earlyDataup),
            3, "Early Data Up");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the SR HARQ.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_SR_WITH_HARQ
/// HIFN     RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_SR_WITHOUT_HARQ
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SENSe:NIOT:SIGN<i>:UECapability:TRANsfer:PHYSical:SR:HARQack:WITH?
/// HIFN     SENSe:NIOT:SIGN<i>:UECapability:TRANsfer:PHYSical:SR:HARQack:WOUT?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR withHARQ/Indicates whether the UE supports scheduling request with HARQ-ACK.
/// HIPAR withoutHARQ/Indicates whether the UE supports scheduling request without HARQ-ACK.
ViStatus _VI_FUNC rscmwins_QueryUECapabilitiesSR(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32*  withHARQ,
                                                 ViInt32*  withoutHARQ)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_SR_WITH_HARQ, withHARQ),
            3, "With HARQ");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_UE_CAPABILITIES_SR_WITHOUT_HARQ, withoutHARQ),
            4, "Without HARQ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the UE measurement report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables the UE measurement report.
ViStatus _VI_FUNC rscmwins_ConfigureUEMeasurementReportEnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_UE_MEASUREMENT_REPORT_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the RSRP reported by the UE as dimensionless index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR index/Returns the RSRP reported by the UE as dimensionless index.
ViStatus _VI_FUNC rscmwins_QueryUEMeasurementReportRSRPIndex(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32*  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_UE_MEASUREMENT_REPORT_RSRP_INDEX, index),
            3, "Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the RSRP value range, corresponding to the RSRP index reported
/// HIFN  by the UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR lower/Lower range.
/// HIPAR upper/Upper range.
ViStatus _VI_FUNC rscmwins_QueryUEMeasurementReportRSRPRange(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViReal64* lower,
                                                             ViReal64* upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, lower),
        2, "lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upper),
        3, "upper");

    checkErr(rscmwins_GetAttributeRawString(instrSession, repCap,
                RSCMWINS_ATTR_UE_MEASUREMENT_REPORT_RSRP_RANGE,
                RS_MAX_MESSAGE_BUF_SIZE, response));

    p2buf = strtok(response, ",");
    *lower = RsCore_Convert_String_To_ViReal64(p2buf);
    p2buf = strtok (NULL, ",");
    *upper = RsCore_Convert_String_To_ViReal64(p2buf);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the RSRQ reported by the UE as dimensionless index.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR index/Returns the RSRQ reported by the UE as dimensionless index.
ViStatus _VI_FUNC rscmwins_QueryUEMeasurementReportRSRQIndex(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32*  index)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_UE_MEASUREMENT_REPORT_RSRQ_INDEX, index),
            3, "Index");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the RSRQ value range corresponding to the dimensionless index
/// HIFN  values reported for the serving NB-IoT cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR lower/Lower range.
/// HIPAR upper/Upper range.
ViStatus _VI_FUNC rscmwins_QueryUEMeasurementReportRSRQRange(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViReal64* lower,
                                                             ViReal64* upper)
{
    ViStatus error = VI_SUCCESS;
    ViChar   response[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));


    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, lower),
        2, "lower");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, upper),
        3, "upper");

    checkErr(rscmwins_GetAttributeRawString(instrSession, repCap,
                RSCMWINS_ATTR_UE_MEASUREMENT_REPORT_RSRQ_RANGE,
                RS_MAX_MESSAGE_BUF_SIZE, response));

    p2buf = strtok(response, ",");
    *lower = RsCore_Convert_String_To_ViReal64(p2buf);
    p2buf = strtok (NULL, ",");
    *upper = RsCore_Convert_String_To_ViReal64(p2buf);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Activates the scenario Standard Cell and selects the signal paths.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR signalingUnit/Sets the signaling unit.
/// HIPAR rxConnector/Defines the RX Connector.
/// HIPAR rxConverter/Defines the RF input converter.
/// HIPAR txConnector/Selects the RF output connector.
/// HIPAR txConverter/Defines the RF output converter.
ViStatus _VI_FUNC rscmwins_ConfigureRFScenario(ViSession instrSession,
                                               ViInt32   instance,
                                               ViInt32   signalingUnit,
                                               ViInt32   rxConnector,
                                               ViInt32   rxConverter,
                                               ViInt32   txConnector,
                                               ViInt32   txConverter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, signalingUnit, RSCMWINS_VAL_SIGNALING_UNIT_SUA1, RSCMWINS_VAL_SIGNALING_UNIT_SUA6),
            3, "Signaling Unit");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxConnector, RSCMWINS_VAL_ANALYZER_CONNECTOR_RF1C, RSCMWINS_VAL_ANALYZER_CONNECTOR_R44C),
            4, "RX Connector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, rxConverter, RSCMWINS_VAL_CONV_RX1, RSCMWINS_VAL_CONV_RX4),
            5, "RX Converter");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, txConnector, RSCMWINS_VAL_TX_CONN_RF1C, RSCMWINS_VAL_TX_CONN_RB18),
            6, "TX Connector");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, txConverter, RSCMWINS_VAL_CONV_TX1, RSCMWINS_VAL_CONV_TX4),
            7, "TX Converter");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ROUT:NIOT:SIGN%ld:SCEN:SCEL %s,%s,%s,%s,%s",
            instance, signalingUnitArr[signalingUnit], rfInputConnectorArr[rxConnector], rfInputConverterArr[rxConverter], rfOutputConnectorArr[txConnector], rfOutputConverterArr[txConverter]);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the configured routing settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR scenario/Displays active scenario.
/// HIPAR rxConnector/Displays the RX Connector.
/// HIPAR rxConverter/Defines the RF input converter.
/// HIPAR txConnector/Displays the RF output connector.
/// HIPAR txConverter/Displays the RF output converter.
ViStatus _VI_FUNC rscmwins_QueryRFConfiguredRoutingSettings(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32*  scenario,
                                                            ViInt32*  rxConnector,
                                                            ViInt32*  rxConverter,
                                                            ViInt32*  txConnector,
                                                            ViInt32*  txConverter)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "ROUT:NIOT:SIGN%ld?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *scenario = RsCore_FindStringIndex(scenarioArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *rxConnector = RsCore_FindStringIndex(rfInputConnectorArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *rxConverter = RsCore_FindStringIndex(rfInputConverterArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *txConnector = RsCore_FindStringIndex(rfOutputConnectorArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *txConverter = RsCore_FindStringIndex(rfOutputConverterArr, p2buf);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF Output.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR attenuation/Defines an external attenuation (or gain, if the value is negative),
/// HIPAR attenuation/to be applied to the RF output connector.
/// HIPAR externalDelayCompensation/Defines the value of an external time delay in the output path and in
/// HIPAR externalDelayCompensation/the input path, so that it can be compensated.
ViStatus _VI_FUNC rscmwins_ConfigureRFOutput(ViSession instrSession,
                                             ViInt32   instance,
                                             ViReal64  attenuation,
                                             ViReal64  externalDelayCompensation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_OUTPUT_ATTENUATION, attenuation),
            3, "Attenuation");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_EXTERNAL_TIME_DELAY_OUTPUT, externalDelayCompensation),
            4, "External Delay Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF Input.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR attenuation/Defines an external attenuation (or gain, if the value is negative),
/// HIPAR attenuation/to be applied to the RF input connector.
/// HIPAR externalDelayCompensation/Defines the value of an external time delay in the output path and in
/// HIPAR externalDelayCompensation/the input path, so that it can be compensated.
ViStatus _VI_FUNC rscmwins_ConfigureRFInput(ViSession instrSession,
                                            ViInt32   instance,
                                            ViReal64  attenuation,
                                            ViReal64  externalDelayCompensation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_INPUT_ATTENUATION, attenuation),
            3, "Attenuation");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_EXTERNAL_TIME_DELAY_INPUT, externalDelayCompensation),
            4, "External Delay Compensation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the operating band (OB).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR operatingBand/Selects the operating band (OB).
ViStatus _VI_FUNC rscmwins_ConfigureRFFrequencyOperatingBand(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   operatingBand)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_SIGNAL_OPERATING_BAND, operatingBand),
            3, "Operating Band");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF Frequency DL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR channel/Select the DL and UL channel numbers. They must be valid for the
/// HIPAR channel/current operating band.
/// HIPAR frequencyOffset/Specifies a positive or negative frequency offset to be added to the
/// HIPAR frequencyOffset/center frequency of the configured DL or UL channel.
ViStatus _VI_FUNC rscmwins_ConfigureRFFrequencyDL(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   channel,
                                                  ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_CHANNEL_DL, channel),
            3, "Channel");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_FREQUENCY_OFFSET_DL, frequencyOffset),
            4, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF Frequency UL.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR channel/Select the DL and UL channel numbers. They must be valid for the
/// HIPAR channel/current operating band.
/// HIPAR frequencyOffset/Specifies a positive or negative frequency offset to be added to the
/// HIPAR frequencyOffset/center frequency of the configured DL or UL channel.
ViStatus _VI_FUNC rscmwins_ConfigureRFFrequencyUL(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   channel,
                                                  ViReal64  frequencyOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_CHANNEL_UL, channel),
            3, "Channel");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_FREQUENCY_OFFSET_UL, frequencyOffset),
            4, "Frequency Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF Expected Uplink Power.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enpMode/Selects the expected nominal power mode. The expected nominal power of
/// HIPAR enpMode/the UL signal can be defined manually or calculated automatically,
/// HIPAR enpMode/according to the UL power control settings.
/// HIPAR enpPower/Sets the expected nominal power of the UL signal in manual mode.
/// HIPAR margin/Sets the margin that the R&S&nbsp;CMW adds to the expected nominal
/// HIPAR margin/power to determine the reference level in manual mode. If the expected
/// HIPAR margin/nominal power is calculated automatically according to the UL power
/// HIPAR margin/control settings, a fix margin of 12 dB is used instead.
/// HIPAR mixerLevelOffset/Varies the input level of the mixer in the analyzer path.
ViStatus _VI_FUNC rscmwins_ConfigureRFExpectedUplinkPower(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   enpMode,
                                                          ViReal64  enpPower,
                                                          ViReal64  margin,
                                                          ViReal64  mixerLevelOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_ENP_MODE, enpMode),
            3, "ENP Mode");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_ENP_POWER, enpPower),
            4, "ENP Power");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_ENP_USER_MARGIN, margin),
            5, "Margin");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_RF_MIXER_LEVEL_OFFSET, mixerLevelOffset),
            6, "Mixer Level Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the operating band and the downlink channel number.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR band/Selects the operating band.
/// HIPAR channel/DL channel number.
ViStatus _VI_FUNC rscmwins_ConfigureRFAllChannels(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   band,
                                                  ViInt32   channel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, band, RSCMWINS_VAL_OB_UDEF, RSCMWINS_VAL_OB_85),
            3, "Band");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:RFS:ALL:BWCH %s,%ld", instance, rscmwins_rngOperatingBand.rangeValues[band].cmdString, channel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the RF DL Power Levels.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR nrsEPRE/Defines the energy per resource element (EPRE) of the narrowband
/// HIPAR nrsEPRE/reference signal (NRS). The power levels of resource elements used for
/// HIPAR nrsEPRE/other channels/signals are derived from this power level.
/// HIPAR ocng/Enables or disables the OFDMA channel noise generator (OCNG).
/// HIPAR awgn/Specifies the total level of the additional white Gaussian noise
/// HIPAR awgn/(AWGN) interferer. The unit dBm/15 kHz indicates the spectral density
/// HIPAR awgn/integrated across one subcarrier.
ViStatus _VI_FUNC rscmwins_ConfigureRFDLPowerLevels(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViReal64  nrsEPRE,
                                                    ViBoolean ocng,
                                                    ViReal64  awgn)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_DL_POWER_NRS_EPRE, nrsEPRE),
            3, "nrs EPRE");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_DL_POWER_OCNG, ocng),
            4, "OCNG");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_DL_POWER_AWGN, awgn),
            5, "AWGN");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the User Defined Channel values.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR ulMaximum/Queries the maximum channel number in uplink for user-defined
/// HIPAR ulMaximum/operating band.
/// HIPAR dlMaximumFreq/Queries the maximum downlink frequency for user-defined operating
/// HIPAR dlMaximumFreq/band.
/// HIPAR ulMaximumFreq/Queries the maximum uplink frequency for user-defined operating band.
/// HIPAR ulMinimumFreq/Queries the minimum uplink frequency for user-defined operating band.
ViStatus _VI_FUNC rscmwins_QueryRFUserDefinedChannel(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32*  ulMaximum,
                                                     ViReal64* dlMaximumFreq,
                                                     ViReal64* ulMaximumFreq,
                                                     ViReal64* ulMinimumFreq)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_RF_USER_DEFINED_CHANNEL_UL_MAXIMUM, ulMaximum),
            3, "UL Maximum");

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_RF_USER_DEFINED_FREQUENCY_DL_MAXIMUM, dlMaximumFreq),
            4, "DL Maximum Frequency");

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_RF_USER_DEFINED_FREQUENCY_UL_MAXIMUM, ulMaximumFreq),
            5, "UL Maximum Frequency");

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_RF_USER_DEFINED_FREQUENCY_UL_MINIMUM, ulMinimumFreq),
            6, "UL Minimum Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the user-defined operating band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enable/Enable or disables user-defined band.
/// HIPAR bandIndicator/Band Indicator.
/// HIPAR separationValue/UL / DL separation value.
/// HIPAR dlMinChannel/Minimum channel number in downlink.
/// HIPAR dlMaxChannel/Maximum channel number in downlink.
/// HIPAR dlMinFreq/Minimum frequency in downlink.
/// HIPAR ulMinChannel/Minimum channel number in uplink.
ViStatus _VI_FUNC rscmwins_ConfigureRFUserDefinedOperatingBand(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enable,
                            ViInt32   bandIndicator,
                            ViReal64  separationValue,
                            ViReal64  dlMinChannel,
                            ViReal64  dlMaxChannel,
                            ViReal64  dlMinFreq,
                            ViReal64  ulMinChannel)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bandIndicator, 1, 256),
            4, "Band Indicator");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dlMinChannel, 0, 262.143e+3), 6, "DL Minimum Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dlMaxChannel, 0, 262.143e+3), 7, "DL Maximum Channel");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, dlMinChannel, 0, 262.143e+3), 9, "UL Minimum Channel");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:RFS:UDEF:SET UDEF,%s,%ld,%.12lG,%.12lG,%.12lG,%.12lG,%.12lG", instance, enable ? "ON" : "OFF", bandIndicator, separationValue, dlMinChannel, dlMaxChannel, dlMinFreq, ulMinChannel);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the UL Power Control.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR nominalPower/Defines a cell-specific nominal power value for maximum subcarrier
/// HIPAR nominalPower/allocation in the UL. The maximum allowed allocation depends on the
/// HIPAR nominalPower/subcarrier spacing.
/// HIPAR maxAllowedPower/Specifies the maximum allowed UE power.
ViStatus _VI_FUNC rscmwins_ConfigureULPowerControl(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViReal64  nominalPower,
                                                   ViReal64  maxAllowedPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_CONTROL_NOMINAL_POWER, nominalPower),
            3, "Nominal Power");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_CONTROL_MAXIMUM_UE_POWER, maxAllowedPower),
            4, "Max Allowed Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the expected power of the first preamble, resulting from the
/// HIFN  UL power control settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR nprachPreamblePower/Queries the expected power of the first preamble, resulting from the
/// HIPAR nprachPreamblePower/UL power control settings.
ViStatus _VI_FUNC rscmwins_QueryULPowerControlNPRACHPreamblePower(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64* nprachPreamblePower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_UL_POWER_CONTROL_NPRACH_PREAMBLE_POWER, nprachPreamblePower),
            3, "NPRACH Preamble Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the expected NPUSCH power, for the NPUSCH formats 1 and 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR format/Sets the format.
/// HIPAR npuschFormatPower/Queries the expected NPUSCH power, for the NPUSCH formats 1 and 2.
ViStatus _VI_FUNC rscmwins_QueryULPowerControlNPUSCHFormatPower(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   format,
                            ViReal64* npuschFormatPower)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap_i[RS_REPCAP_BUF_SIZE] = "";
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, format, 1, 2),
            3, "Format");
    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap_i));

    snprintf(repCap, RS_REPCAP_BUF_SIZE, "%s,f%ld", repCap_i, format);

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_UL_POWER_CONTROL_NPUSCH_FORMAT_POWER, npuschFormatPower),
            3, "NPUSCH Format Power");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables advanced configuration of the NPRACH and open loop
/// HIFN  power settings via the other CONFigure:NIOT:SIGN:UL:ADPower:...
/// HIFN  commands.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables advanced configuration of the NPRACH and open loop
/// HIPAR enabled/power settings via the other CONFigure:NIOT:SIGN:UL:ADPower:...
/// HIPAR enabled/commands.
ViStatus _VI_FUNC rscmwins_ConfigureULPowerAdvancedEnabled(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_ADVANCED_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures advanced NPRACH and NPUSCH Power Settings
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR nrsPower/Specifies the 'nrs-Power' value, signaled to the UE if advanced UL
/// HIPAR nrsPower/power configuration applies.
/// HIPAR pirTargetPower/Specifies the 'preambleInitialReceivedTargetPower' value, signaled to
/// HIPAR pirTargetPower/the UE if advanced UL power configuration applies.
/// HIPAR p0NominalNPUSCH/Specifies the 'p0-NominalNPUSCH' value, signaled to the UE if advanced
/// HIPAR p0NominalNPUSCH/UL power configuration applies.
/// HIPAR pathlossCompensationAlpha/Specifies the value of parameter 'alpha', signaled to the UE if
/// HIPAR pathlossCompensationAlpha/advanced UL power configuration applies.
ViStatus _VI_FUNC rscmwins_ConfigureULPowerAdvanced(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViReal64  nrsPower,
                                                    ViReal64  pirTargetPower,
                                                    ViReal64  p0NominalNPUSCH,
                                                    ViInt32   pathlossCompensationAlpha)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_ADVANCED_NRS_POWER, nrsPower),
            3, "nrs Power");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_ADVANCED_PIR_TARGET_POWER, pirTargetPower),
            4, "PIR Target Power");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_ADVANCED_P0_NOMINAL_NPUSCH, p0NominalNPUSCH),
            5, "p0 Nominal NPUSCH");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_UL_POWER_ADVANCED_PATHLOSS_COMPENSATION_ALPHA, pathlossCompensationAlpha),
            6, "Pathloss Compensation Alpha");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the pathloss resulting from the advanced UL power settings.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR pathloss/Queries the pathloss resulting from the advanced UL power settings.
ViStatus _VI_FUNC rscmwins_QueryULPowerAdvancedPathloss(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViReal64* pathloss)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_UL_POWER_ADVANCED_PATHLOSS, pathloss),
            3, "Pathloss");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Physical Cell.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR duplexMode/Specifies the duplex mode. The current version only supports FDD.
/// HIPAR ulSubcarrierSpacing/Selects the uplink subcarrier spacing.
/// HIPAR physicalCellID/Defines the physical cell ID used for the generation of the
/// HIPAR physicalCellID/synchronization signals.
/// HIPAR operationMode/Selects the operation mode.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCell(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViInt32   duplexMode,
                                                 ViInt32   ulSubcarrierSpacing,
                                                 ViInt32   physicalCellID,
                                                 ViInt32   operationMode)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_DUPLEX_MODE, duplexMode),
            3, "Duplex Mode");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_SUBCARRIER_SPACING, ulSubcarrierSpacing),
            4, "UL Subcarrier Spacing");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_ID, physicalCellID),
            5, "Physical Cell ID");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_OPERATION_MODE, operationMode),
            6, "Operation Mode");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the dependency between UL and DL carrier
/// HIFN  frequencies for inband and guard-band operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR separation/Enables or disables the dependency between UL and DL carrier
/// HIPAR separation/frequencies for inband and guard-band operation.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellFlexibleULDLSeparation(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean separation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_ENABLE_FLEXIBLE_UL_DL_SEPARATION, separation),
            3, "Separation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Physical Cell In-Band/Guard-Band.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR lteBandwidth/Channel bandwidth for the generated LTE signal.
/// HIPAR lteControlRegionSize/Configures the control region size for the in-band operation mode.
/// HIPAR powerOffset/Defines the power offset of the NB-IoT NRS symbols relative to the LTE
/// HIPAR powerOffset/CRS symbols.
/// HIPAR inbandPCI/Selects the PCI mode for the in-band operation mode. It indicates
/// HIPAR inbandPCI/whether the NB-IoT cell uses the same physical cell ID as the LTE cell.
/// HIPAR inbandCarrierPosition/Specifies the number of the LTE RB at the NB-IoT channel position, for
/// HIPAR inbandCarrierPosition/the in-band operation mode.
/// HIPAR guardBandCarrierPosition/Selects the upper or lower guard band for guard-band operation mode.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellInBandGuardBand(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   lteBandwidth,
                            ViInt32   lteControlRegionSize,
                            ViInt32   powerOffset,
                            ViInt32   inbandPCI,
                            ViInt32   inbandCarrierPosition,
                            ViInt32   guardBandCarrierPosition)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_BANDWIDTH, lteBandwidth),
            3, "LTE Bandwidth");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CONTROL_REGION_SIZE, lteControlRegionSize),
            4, "LTE Control Region Size");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_POWER_OFFSET, powerOffset),
            5, "Power Offset");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_INBAND_PCI, inbandPCI),
            6, "Inband PCI");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_INBAND_CARRIER_POSITION, inbandCarrierPosition),
            7, "Inband Carrier Position");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_GUARD_BAND_POSITION, guardBandCarrierPosition),
            8, "Guard Band Carrier Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the number of the LTE RB at the NB-IoT UL channel position,
/// HIFN  for the in-band operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Disables | enables the shift of the UL channel.
/// HIPAR position/Specifies the number of the LTE RB at the NB-IoT UL channel position,
/// HIPAR position/for the in-band operation.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellInbandUplinkCarrierPosition(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    if (enabled)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_INBAND_UPLINK_CARRIER_POSITION, position),
                4, "Position");
    }
    else
    {
        viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_INBAND_UPLINK_CARRIER_POSITION_ENABLED, enabled),
                3, "Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the LTE subcarrier index at the NB-IoT UL channel position
/// HIFN  for the guard-band operation.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Disables/enables the shift of uplink channel
/// HIPAR position/Specifies the LTE subcarrier index at the NB-IoT UL channel position
/// HIPAR position/for the guard-band operation.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellGuardBandUplinkPosition(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_GUARD_BAND_UPLINK_POSITION_ENABLED, enabled),
            3, "Enabled");

    if (enabled)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_GUARD_BAND_UPLINK_POSITION, position),
                4, "Position");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries physical cell LTE frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR dl/Query the DL and UL LTE carrier center frequencies resulting from the
/// HIPAR dl/in-band mode or guard-band mode settings.
/// HIPAR ul/Query the DL and UL LTE carrier center frequencies resulting from the
/// HIPAR ul/in-band mode or guard-band mode settings.
ViStatus _VI_FUNC rscmwins_QueryPhysicalCellLTEFrequency(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViReal64* dl,
                                                         ViReal64* ul)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_PHYSICAL_CELL_LTE_FREQUENCY_DL, dl),
            3, "DL");

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_PHYSICAL_CELL_LTE_FREQUENCY_UL, ul),
            4, "UL");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries NB-IoT Raster Offsets.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR dl/Query the DL and UL channel raster offsets resulting from the in-band
/// HIPAR dl/mode or guard-band mode settings.
/// HIPAR ul/Query the DL and UL channel raster offsets resulting from the in-band
/// HIPAR ul/mode or guard-band mode settings.
ViStatus _VI_FUNC rscmwins_QueryPhysicalCellNBIoTRasterOffset(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViReal64* dl,
                            ViReal64* ul)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_PHYSICAL_CELL_CHANNEL_RASTER_OFFSET_DL, dl),
            3, "DL");

    viCheckParm(rscmwins_GetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_QUERY_PHYSICAL_CELL_CHANNEL_RASTER_OFFSET_UL, ul),
            4, "UL");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Physical Cell NPRACH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR noResponseToPreambles/Selects whether the signaling application ignores received preambles
/// HIPAR noResponseToPreambles/or not.
/// HIPAR powerRampingStep/Specifies the transmit power difference between two consecutive
/// HIPAR powerRampingStep/preambles.
/// HIPAR preambleFormat/Selects the preamble format.
/// HIPAR maxTransmissions/Configures the maximum number of preamble transmissions.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellNPRACH(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViBoolean noResponseToPreambles,
                                                       ViInt32   powerRampingStep,
                                                       ViInt32   preambleFormat,
                                                       ViInt32   maxTransmissions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_NO_RESPONSE_TO_PREAMBLES, noResponseToPreambles),
            3, "No Response To Preambles");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_POWER_RAMPING, powerRampingStep),
            4, "Power Ramping Step");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_PREAMBLE_FORMAT, preambleFormat),
            5, "Preamble Format");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_MAX_TRANSMISSIONS, maxTransmissions),
            6, "Max Transmissions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Specifies the maximum number of CE levels supported and signaled by
/// HIFN  the NB-IoT eNodeB.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberOfThreshold/Specifies the maximum number of CE levels supported and signaled by
/// HIPAR numberOfThreshold/the NB-IoT eNodeB.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_ConfigureConfigurePhysicalCellCENumberOfThreshold(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   numberOfThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_NUMBER_OF_THRESHOLD, numberOfThreshold),
            3, "Number Of Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the number of repetitions per preamble transmission attempt
/// HIFN  (repetitions with same preamble and power) for the selected CE level.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repetitionsPerPreamble/Specifies the number of repetitions per preamble transmission attempt
/// HIPAR repetitionsPerPreamble/(repetitions with same preamble and power) for the selected CE level
/// HIPAR level/Selects CE level. A higher CE level means a higher coverage
/// HIPAR level/enhancement.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_ConfigureConfigurePhysicalCellCERepetitionsPerPreamble(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   level,
                            ViInt32   repetitionsPerPreamble)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   repetAttrArr[] = {RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_REPETITIONS_PER_PREAMBLE_0,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_REPETITIONS_PER_PREAMBLE_1,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_REPETITIONS_PER_PREAMBLE_2};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, level, 0, 2),
            2, "Level");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, repetAttrArr[level], repetitionsPerPreamble),
            4, "Repetitions Per Preamble");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the maximum number of NPDCCH repetitions during random access
/// HIFN  (RA) procedure for the selected CE level
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR level/Selects CE level. A higher CE level means a higher coverage
/// HIPAR level/enhancement.
/// HIPAR maxRepetitions/Sets the maximum number of NPDCCH repetitions during random access
/// HIPAR maxRepetitions/(RA) procedure for the selected CE level
ViStatus _VI_FUNC rscmwins_ConfigureConfigurePhysicalCellCENPDCCHMaxRepetitions(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   level,
                            ViInt32   maxRepetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   maxRepAttrArr[] = {RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_NPDCCH_MAX_REPETITIONS_0,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_NPDCCH_MAX_REPETITIONS_1,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_NPDCCH_MAX_REPETITIONS_2};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, level, 0, 2),
            2, "Level");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, maxRepAttrArr[level], maxRepetitions),
            4, "Max Repetitions");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the RSRP threshold for NPRACH for the selected CE level. The
/// HIFN  value is defined per CE level. With increasing CE level, the RSRP
/// HIFN  threshold must decrease.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR level/Selects CE level. A higher CE level means a higher coverage
/// HIPAR level/enhancement.
/// HIPAR rsrpThreshold/Defines the RSRP threshold for NPRACH for the selected CE level. The
/// HIPAR rsrpThreshold/value is defined per CE level. With increasing CE level, the RSRP
/// HIPAR rsrpThreshold/threshold must decrease.
ViStatus _VI_FUNC rscmwins_ConfigureConfigurePhysicalCellCERSRPThreshold(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   level,
                            ViReal64  rsrpThreshold)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   thrAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_RSRP_THRESHOLD_1,
             RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_CE_RSRP_THRESHOLD_2};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, level, 1, 2),
            2, "Level");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, thrAttrArr[level], rsrpThreshold),
            4, "RSRP Threshold");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the CE level reported by the UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR report/Queries the CE level reported by the UE.
ViStatus _VI_FUNC rscmwins_QueryPhysicalCellCELevelReport(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32*  report)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_PHYSICAL_CELL_CE_LEVEL_REPORT, report),
            3, "Report");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures wake up signal (WUS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enableWUS/Enables or disables wake up signal (WUS).
/// HIPAR disablePaging/Disables or enables the paging associated with a WUS transmission.
/// HIPAR maxDurationFactor/Sets the maximal duration of WUS.
/// HIPAR numPOs/Sets the number of consecutive POs mapped to one WUS.
/// HIPAR drxCycles/Sets the maximum No. of consecutive relaxed DRX cycles during which
/// HIPAR drxCycles/the UE can use WUS for synchronization.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellWakeUpSignal(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViBoolean enableWUS,
                                                             ViBoolean disablePaging,
                                                             ViInt32   maxDurationFactor,
                                                             ViInt32   numPOs,
                                                             ViInt32   drxCycles)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_ENABLED, enableWUS),
            3, "Enable WUS");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_DISABLE_PAGING, disablePaging),
            4, "Disable Paging");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_MAX_DURATION_FACTOR, maxDurationFactor),
            5, "Max Duration Factor");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_NUM_POS, numPOs),
            6, "Num POs");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_NO_OF_DRX_CYCLES_RELAXED, drxCycles),
            7, "DRX Cycles");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Sets the first PO to send paging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR position/Sets the first PO to send paging.
ViStatus _VI_FUNC rscmwins_ConfigurePhysicalCellWakeUpSignalFirstPOPosition(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   position)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_FIRST_PO_POSITION, position),
            3, "Position");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the time offset.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR drx/Sets the offset from the end of the configured maximum WUS duration to
/// HIPAR drx/the associated PO for DRX with a non-zero gap.
/// HIPAR edrxShort/Sets the offset from the end of the configured maximum WUS duration to
/// HIPAR edrxShort/the associated PO for eDRX with a short non-zero gap.
/// HIPAR edrxLongEnabled/Disables | enables the long non-zero gap.
/// HIPAR edrxLong/Sets and enables the offset from the end of the configured maximum WUS
/// HIPAR edrxLong/duration to the associated PO for eDRX with a long non-zero gap.
ViStatus _VI_FUNC rscmwins_ConfigureConfigurePhysicalCellWakeUpSignalTimeOffset(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   drx,
                            ViInt32   edrxShort,
                            ViBoolean edrxLongEnabled,
                            ViInt32   edrxLong)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_TIME_OFFSET_DRX, drx),
            3, "drx");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_TIME_OFFSET_EDRX_SHORT, edrxShort),
            4, "EDRX Short");

    if(edrxLongEnabled)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_TIME_OFFSET_EDRX_LONG, edrxLong),
                6, "EDRX Long");
    }
    else
    {
        viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_PHYSICAL_CELL_WUS_TIME_OFFSET_EDRX_LONG_ENABLED, edrxLongEnabled),
                5, "EDRX Long Enabled");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the network identity.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mcc/Specifies the three-digit mobile country code (MCC). You can omit
/// HIPAR mcc/leading zeros.
/// HIPAR mnc/Specifies the mobile network code (MNC). You can omit leading zeros.
/// HIPAR mncDigits/Specifies the number of digits of the mobile network code (MNC).
/// HIPAR tac/Specifies the tracking area code.
/// HIPAR cidEUTRAN/Specifies the E-UTRAN cell identifier (28-digit binary number).
ViStatus _VI_FUNC rscmwins_ConfigureNetworkIdentity(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   mcc,
                                                    ViInt32   mnc,
                                                    ViInt32   mncDigits,
                                                    ViInt32   tac,
                                                    ViString  cidEUTRAN)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_MCC, mcc),
            3, "MCC");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_MNC, mnc),
            4, "MNC");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_MNC_DIGITS, mncDigits),
            5, "MNC Digits");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_TAC, tac),
            6, "TAC");

    viCheckParm(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_CID_EUTRAN, cidEUTRAN),
            7, "CID Eutran");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the network security.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR authentication/Enables or disables authentication, to be performed during the attach
/// HIPAR authentication/procedure.
/// HIPAR enabled/Enables or disables security (integrity protection).
/// HIPAR integrityAlgorithm/Selects an algorithm for integrity protection.
/// HIPAR secretKey/Defines the secret key K as 32-digit hexadecimal number. You can omit
/// HIPAR secretKey/leading zeros.
/// HIPAR milenage/Enables or disables using the MILENAGE algorithm set instead of the
/// HIPAR milenage/standard algorithms.
/// HIPAR opc/Specifies the key OPc as 32-digit hexadecimal number.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkSecurity(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViBoolean authentication,
                                                    ViBoolean enabled,
                                                    ViInt32   integrityAlgorithm,
                                                    ViString  secretKey,
                                                    ViBoolean milenage,
                                                    ViString  opc)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_AUTHENTICATION, authentication),
            3, "Authentication");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_ENABLED, enabled),
            4, "Enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_ALGORITHM, integrityAlgorithm),
            5, "Integrity Algorithm");

    viCheckParm(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_SECRET_KEY, secretKey),
            6, "Secret Key");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_MILENAGE, milenage),
            7, "Milenage");

    viCheckParm(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SECURITY_OPC, opc),
            8, "OPC");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the default IMSI.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR defaultIMSI/Specifies the default IMSI.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkUEIdentity(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViString  defaultIMSI)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_IDENTITY_UE_IMSI, defaultIMSI),
            3, "Default IMSI");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the network timer.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR outofSync/Specifies the time after which the instrument, having waited for a
/// HIPAR outofSync/signal from the connected UE, releases the connection.
/// HIPAR t3324Enabled/Configures timer T3324, that means the idle time before PSM, see also
/// HIPAR t3324Enabled/CONFigure:NIOT:SIGN:CONNection:PSMallowed.
/// HIPAR t3324Value/Configures timer T3324, that means the idle time before PSM, see also
/// HIPAR t3324Value/CONFigure:NIOT:SIGN:CONNection:PSMallowed.
/// HIPAR t3412Enabled/Configures timer T3412 for periodic tracking area updates.
/// HIPAR t3412Value/Configures timer T3412 for periodic tracking area updates.
/// HIPAR t3412Extended/Configures an extended value for timer T3412.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkTimer(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViReal64  outofSync,
                                                 ViBoolean t3324Enabled,
                                                 ViReal64  t3324Value,
                                                 ViBoolean t3412Enabled,
                                                 ViReal64  t3412Value,
                                                 ViReal64  t3412Extended)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_OUT_OF_SYNC, outofSync),
            3, "Out Of Sync");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_T3324_ENABLED, t3324Enabled),
            4, "T3324 Enabled");
    if (t3324Enabled == VI_TRUE)
    {
        viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_T3324_VALUE, t3324Value),
                5, "T3324 Value");
    }

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_T3412_ENABLED, t3412Enabled),
            6, "T3412 Enabled");
    if (t3412Enabled == VI_TRUE)
    {
        viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_T3412_VALUE, t3412Value),
                7, "T3412 Value");
    }

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIMER_ADDITIONAL_T3412, t3412Extended),
            8, "T3412 Extended");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the network synchronization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR syncZone/Selects the synchronization zone for the signaling application.
/// HIPAR syncOffset/Configures the timing offset relative to the time zone.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkSynchronization(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   syncZone,
                                                           ViReal64  syncOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SYNC_ZONE, syncZone),
            3, "Sync Zone");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_SYNC_OFFSET, syncOffset),
            4, "Sync Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the network time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR timeSource/Selects the date and time source.
/// HIPAR daylightSavingTime/Specifies a daylight saving time (DST) offset for the time source DATE
/// HIPAR daylightSavingTime/(see CONFigure:NIOT:SIGN:CELL:TIME:TTSRc)
/// HIPAR daylightSavingTimeValue/Specifies a daylight saving time (DST) offset for the time source DATE
/// HIPAR daylightSavingTimeValue/(see CONFigure:NIOT:SIGN:CELL:TIME:TTSRc)
/// HIPAR localTimeZoneOffset/Specifies the time zone offset for the time source DATE (see
/// HIPAR localTimeZoneOffset/CONFigure:NIOT:SIGN:CELL:TIME:TTSRc).
ViStatus _VI_FUNC rscmwins_ConfigureNetworkTime(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   timeSource,
                                                ViBoolean daylightSavingTime,
                                                ViInt32   daylightSavingTimeValue,
                                                ViReal64  localTimeZoneOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_SOURCE, timeSource),
            3, "Time Source");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_DAYLIGHT_SAVING_TIME_ENABLED, daylightSavingTime),
            4, "Daylight Saving Time");
    if (daylightSavingTime == VI_TRUE)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_DAYLIGHT_SAVING_TIME_VALUE, daylightSavingTimeValue),
                5, "Daylight Saving Time Value");
    }

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_LOCAL_TIME_ZONE_OFFSET, localTimeZoneOffset),
            6, "Local Time Zone Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Triggers the transfer of the date and time information to the UE.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkTimeSendNow(ViSession instrSession,
                                                       ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_SEND_NOW, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies whether the date and time information is sent to the UE
/// HIFN  during the attach procedure or not.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR sendTime/Specifies whether the date and time information is sent to the UE
/// HIPAR sendTime/during the attach procedure or not.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkTimeSendTime(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViBoolean sendTime)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_NETWORK_TIME_SEND_TIME, sendTime),
            3, "Send Time");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the network UTC date and time.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR month/Sets the month.
/// HIPAR day/Sets the day.
/// HIPAR year/Sets the year.
/// HIPAR hour/Sets the hour.
/// HIPAR minute/Sets the minute.
/// HIPAR second/Sets the seconds.
ViStatus _VI_FUNC rscmwins_ConfigureNetworkUTCDateTime(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   month,
                                                       ViInt32   day,
                                                       ViInt32   year,
                                                       ViInt32   hour,
                                                       ViInt32   minute,
                                                       ViReal64  second)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmdDate[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   cmdTime[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, month, 1, 12),
            3, "Month");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, day, 1, 31),
            4, "Day");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, hour, 0, 23),
            6, "Hour");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, minute, 0, 59),
            7, "Minute");
    viCheckParm(RsCore_InvalidViReal64Range(instrSession, second, 0.000, 59.999), 8, "Second");

    snprintf(cmdDate, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CELL:TIME:TDAT %ld,%ld,%ld", instance, day, month, year);
    snprintf(cmdTime, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CELL:TIME:TTME %ld,%ld,%.12lG", instance, hour, minute, second);

    checkErr(RsCore_Write(instrSession, cmdDate));
    checkErr(rscmwins_CheckStatus(instrSession));

    checkErr(RsCore_Write(instrSession, cmdTime));
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the connection type to be applied.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR connectionType/Selects the connection type to be applied.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionType(ViSession instrSession,
                                                   ViInt32   instance,
                                                   ViInt32   connectionType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_CONNECTION_TYPE, connectionType),
            3, "Connection Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the test mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Specifies whether the UE is forced into a test mode. If enabled, the
/// HIPAR enabled/message 'ACTIVATE TEST MODE' is sent to the UE.
/// HIPAR ipVersion/Specifies which IP versions are allowed in test mode.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionTestMode(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViBoolean enabled,
                                                       ViInt32   ipVersion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_TEST_MODE_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_TEST_MODE_IP_VERSION, ipVersion),
            4, "IP Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the C-IoT optimization type to be used for the attach in data
/// HIFN  application mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR cIoTOptimization/Selects the C-IoT optimization type to be used for the attach in data
/// HIPAR cIoTOptimization/application mode.
/// HIPAR ipVersion/Specifies which IP versions are allowed in data application mode.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionDataApplication(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   cIoTOptimization,
                            ViInt32   ipVersion)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_C_IOT, cIoTOptimization),
            3, "CIoT Optimization");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_DATA_APPLICATION_MODE_IP_VERSION, ipVersion),
            4, "IP Version");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the destination server IP address and port.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR ipAddress/Specifies the destination server IP address (IPv4) for non-IP data
/// HIPAR ipAddress/delivery.
/// HIPAR port/Specifies the destination UDP port for non-IP data delivery.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionDestinationServer(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViString  ipAddress,
                            ViInt32   port)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_DESTINATION_SERVER_IP_ADDRESS, ipAddress),
            3, "IP Address");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_DESTINATION_SERVER_PORT, port),
            4, "Port");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the RRC connection.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR rrcKeep/Selects whether the RRC connection is kept or released after attach.
/// HIPAR timer/Configures the inactivity timeout for disabled 'Keep RRC Connection'
/// HIPAR timer/(CONFigure:NIOT:SIGN:CONNection:RRC:KEEP OFF).
ViStatus _VI_FUNC rscmwins_ConfigureConnectionRRC(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViBoolean rrcKeep,
                                                  ViInt32   timer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_RRC_KEEP, rrcKeep),
            3, "RRC Keep");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_RRC_TIMER, timer),
            4, "Timer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Selects the scheduling type.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR schedulingType/Selects the scheduling type.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionSchedulingType(ViSession instrSession,
                                                             ViInt32   instance,
                                                             ViInt32   schedulingType)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_SCHEDULING_TYPE, schedulingType),
            3, "Scheduling Type");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the paging cycle.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR generalDefaultPagingCycle/Selects the cell-specific default paging cycle.
/// HIPAR idleMode/Enables or disables eDRX.
/// HIPAR timeWindow/Configures the duration of one paging time window in idle mode.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionPaging(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   generalDefaultPagingCycle,
                                                     ViBoolean idleMode,
                                                     ViInt32   timeWindow)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_DEFAULT_PAGING_CYCLE, generalDefaultPagingCycle),
            3, "General Default Paging Cycle");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_PAGING_IDLE_MODE, idleMode),
            4, "Idle Mode");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_PAGING_TIME_WINDOW, timeWindow),
            5, "Time Window");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the field 'nB' in the 'PCCH-Config-NB' in system
/// HIFN  information block 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR pcchConfignB/Configures the field 'nB' in the 'PCCH-Config-NB' in system
/// HIPAR pcchConfignB/information block 2.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionPCCHConfignB(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   pcchConfignB)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_PCCH_CONFIG_NB, pcchConfignB),
            3, "PCCH Config nB");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the field 'modificationPeriodCoeff' in the 'BCCH-Config-NB'
/// HIFN  in system information block 2.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR periodCoefficient/Configures the field 'modificationPeriodCoeff' in the 'BCCH-Config-NB'
/// HIPAR periodCoefficient/in system information block 2.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionModificationPeriodCoefficient(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   periodCoefficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_MODIFICATION_PERIOD_COEFFICIENT, periodCoefficient),
            3, "Period Coefficient");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies whether a UE request for power-saving mode (PSM) is accepted
/// HIFN  or rejected.
/// HIFN     If you want to accept PSM requests, configure also timer T3324, see
/// HIFN  CONFigure:NIOT:SIGN:CELL:TOUT:T3324.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR psmAllowed/Specifies whether a UE request for power-saving mode (PSM) is accepted
/// HIPAR psmAllowed/or rejected.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionPSMAllowed(ViSession instrSession,
                                                         ViInt32   instance,
                                                         ViBoolean psmAllowed)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_PSM_ALLOWED, psmAllowed),
            3, "PSM Allowed");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the duration of one eDRX cycle in idle mode.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR generalCycleLength/Configures the duration of one eDRX cycle in idle mode.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionCycleLength(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViInt32   generalCycleLength)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_CYCLE_LENGTH, generalCycleLength),
            3, "General Cycle Length");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures a downlink reference measurement channel (DL RMC). Only
/// HIFN  certain value combinations are accepted, see Scheduling Type DL RMC.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR subcarriers/
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Start subcarrier.
/// HIPAR mcsindex/MCS Index value.
/// HIPAR subframes/Number of subframes.
/// HIPAR repetitions/Number of NPDSCH repetitions.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionDLRMC(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   subcarriers,
                                                    ViInt32   startSubcarrier,
                                                    ViInt32   MCSindex,
                                                    ViInt32   subframes,
                                                    ViInt32   repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSCMWINS_VAL_SUBC_SC12, RSCMWINS_VAL_SUBC_SC12),
            3, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startSubcarrier, 0, 0),
            4, "Start Subcarrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, MCSindex, RSCMWINS_VAL_MCS_INDEX_MI6, RSCMWINS_VAL_MCS_INDEX_MI6),
            5, "MCS Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframes, RSCMWINS_VAL_SUBFRAMES_NSF01, RSCMWINS_VAL_SUBFRAMES_NSF01),
            6, "Subframes");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetitions, RSCMWINS_VAL_REPETITIONS_NR1, RSCMWINS_VAL_REPETITIONS_NR128),
            7, "Repetitions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:RMC:DL %s,%ld,%s,%s,%s", instance, rscmwins_rngDNDSubcarriers.rangeValues[subcarriers].cmdString, startSubcarrier,
             rscmwins_rngDNDMCSIndex.rangeValues[MCSindex].cmdString, rscmwins_rngDNDSubframes.rangeValues[subframes].cmdString, rscmwins_rngDNDRepetitions.rangeValues[repetitions].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries the configured MCS index and the resulting modulation scheme
/// HIFN  and transport block size for the scheduling type DL RMC
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mcsIndex/MCS Index Value.
/// HIPAR modulation/Displays RMC Modulation.
/// HIPAR transportBlockSize/Displays transport block size.
ViStatus _VI_FUNC rscmwins_QueryConnectionDLRMC(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32*  MCSIndex,
                                                ViInt32*  modulation,
                                                ViInt32*  transportBlockSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:RMC:DL:MCS?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *MCSIndex = RsCore_FindStringIndex(MCSIndexArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *modulation = RsCore_FindStringIndex(modulationArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    sscanf(p2buf, "%ld", transportBlockSize);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables or disables RRC paging after SI transmission.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables RRC paging after SI transmission.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionRRCAllowPagingAfterSITransmission(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_RRC_ALLOW_PAGING_AFTER_SI_TRANSMISSION, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables / disables interference randomization.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables / disables interference randomization.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionInterferenceRandomization(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_INTERFERENCE_RANDOMIZATION, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables the paging collision report.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables the paging collision report.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionPagingCollisionReport(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_GENERAL_PAGING_COLLISION_REPORT, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the EDT.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables the paging collision report.
/// HIPAR resourceNumbers/Sets the No. of resource units for by early data transmission.
/// HIPAR userPlanEnabled/Enables or disables EDT in user plan (UP) CIoT EPS optimized mode.
/// HIPAR tbsEnabled/Enables or disables small TBS for EDT.
/// HIPAR tbsSubsetEnabled/Enables or disables the small TBS subset for EDT.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionEDT(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViBoolean enabled,
                                                  ViInt32   resourceNumbers,
                                                  ViBoolean userPlanEnabled,
                                                  ViBoolean tbsEnabled,
                                                  ViBoolean tbsSubsetEnabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_EDT_ENABLED, enabled),
            3, "Enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_EDT_RESOURCE_NUMBERS, resourceNumbers),
            4, "Resource Numbers");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_EDT_USER_PLAN_ENABLED, userPlanEnabled),
            5, "User Plan Enabled");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_EDT_SMALL_TBS_ENABLED, tbsEnabled),
            6, "TBS Enabled");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_EDT_SMALL_TBS_SUBSET_ENABLED, tbsSubsetEnabled),
            7, "TBS Subset Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures an uplink reference measurement channel (UL RMC) for NPUSCH
/// HIFN  format 1.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Start subcarrier.
/// HIPAR mcsindex/MCS Index value.
/// HIPAR resourceUnits/Number of resource units.
/// HIPAR repetitions/Number of NPUSCH repetitions.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionULRMC(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   subcarriers,
                                                    ViInt32   startSubcarrier,
                                                    ViInt32   mcsindex,
                                                    ViInt32   resourceUnits,
                                                    ViInt32   repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSCMWINS_VAL_SUBC_SC1, RSCMWINS_VAL_SUBC_SC12),
            3, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startSubcarrier, 0, 47),
            4, "Start Subcarrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, mcsindex, RSCMWINS_VAL_MCS_INDEX_MI0, RSCMWINS_VAL_MCS_INDEX_MI5),
            5, "MCS Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resourceUnits, RSCMWINS_VAL_RESOURCE_UNITS_NRU01, RSCMWINS_VAL_RESOURCE_UNITS_NRU02),
            6, "Resource Units");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetitions, RSCMWINS_VAL_REPETITIONS_NR1, RSCMWINS_VAL_REPETITIONS_NR128),
            7, "Repetitions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:RMC:UL %s,%ld,%s,%s,%s", instance, rscmwins_rngDNDSubcarriers.rangeValues[subcarriers].cmdString, startSubcarrier,
             rscmwins_rngDNDMCSIndex.rangeValues[mcsindex].cmdString, rscmwins_rngDNDResourceUnits.rangeValues[resourceUnits].cmdString, rscmwins_rngDNDRepetitions.rangeValues[repetitions].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the configured MCS index and the resulting modulation scheme
/// HIFN  and transport block size for the scheduling type UL RMC for NPUSCH
/// HIFN  format 1
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mcsIndex/MCS Index Value.
/// HIPAR modulation/Displays RMC Modulation.
/// HIPAR transportBlockSize/Displays transport block size.
ViStatus _VI_FUNC rscmwins_QueryConnectionULRMC(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32*  MCSIndex,
                                                ViInt32*  modulation,
                                                ViInt32*  transportBlockSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:RMC:UL:MCS?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *MCSIndex = RsCore_FindStringIndex(MCSIndexArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *modulation = RsCore_FindStringIndex(modulationArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    sscanf(p2buf, "%ld", transportBlockSize);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries scheduling parameters for a UL RMC for NPUSCH format 2. These
/// HIFN  settings are not configurable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Displays the start subcarrier.
/// HIPAR mcsindex/Displays the MCS Index.
/// HIPAR resourceunits/Displays the resource units.
/// HIPAR repetitions/Displays the repetitions.
ViStatus _VI_FUNC rscmwins_QueryConnectionULRMCFormat(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViInt32*  subcarriers,
                                                      ViInt32*  startSubcarrier,
                                                      ViInt32*  modulation,
                                                      ViInt32*  resourceUnits,
                                                      ViInt32*  repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:RMC:FORM?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *subcarriers = RsCore_FindStringIndex(subcarriersArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    sscanf(p2buf, "%ld", startSubcarrier);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *modulation = RsCore_FindStringIndex(modulationArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *resourceUnits = RsCore_FindStringIndex(resourceUnitsArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *repetitions = RsCore_FindStringIndex(repetitionsArr, p2buf);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Selects the pattern of grants sent to the UE.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_USER_PATTERN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:UDPattern
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR pattern/Selects the pattern of grants sent to the UE.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionUserDefinedPattern(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   pattern)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_USER_PATTERN, pattern),
            3, "Pattern");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the downlink user-defined scheduling.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:DL[:SET<setNr>]
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR setNumber/For future use.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Start subcarrier.
/// HIPAR mcsindex/MCS Index value.
/// HIPAR subframes/Number of subframes.
/// HIPAR repetitions/Number of NPDSCH repetitions.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionUserDL(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   setNumber,
                                                     ViInt32   subcarriers,
                                                     ViInt32   startSubcarrier,
                                                     ViInt32   MCSindex,
                                                     ViInt32   subframes,
                                                     ViInt32   repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             setNumber;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSCMWINS_VAL_SUBC_SC12, RSCMWINS_VAL_SUBC_SC12),
            4, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startSubcarrier, 0, 0),
            5, "Start Subcarrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, MCSindex, RSCMWINS_VAL_MCS_INDEX_MI0, RSCMWINS_VAL_MCS_INDEX_MI12),
            6, "MCS Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subframes, RSCMWINS_VAL_SUBFRAMES_NSF01, RSCMWINS_VAL_SUBFRAMES_NSF10),
            7, "Subframes");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetitions, RSCMWINS_VAL_REPETITIONS_NR1, RSCMWINS_VAL_REPETITIONS_NR8),
            8, "Repetitions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:UDEF:DL %s,%ld,%s,%s,%s", instance, rscmwins_rngDNDSubcarriers.rangeValues[subcarriers].cmdString, startSubcarrier,
             rscmwins_rngDNDMCSIndex.rangeValues[MCSindex].cmdString, rscmwins_rngDNDSubframes.rangeValues[subframes].cmdString, rscmwins_rngDNDRepetitions.rangeValues[repetitions].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the downlink user-defined scheduling.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:DL[:SET<setNr>]
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR setNumber/For future use.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Start subcarrier.
/// HIPAR mcsindex/MCS Index value.
/// HIPAR resourceUnits/Number of resource units.
/// HIPAR repetitions/Number of NPDSCH repetitions.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionUserUL(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32   setNumber,
                                                     ViInt32   subcarriers,
                                                     ViInt32   startSubcarrier,
                                                     ViInt32   MCSindex,
                                                     ViInt32   resourceUnits,
                                                     ViInt32   repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             setNumber;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, subcarriers, RSCMWINS_VAL_SUBC_SC1, RSCMWINS_VAL_SUBC_SC12),
            4, "Subcarriers");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, startSubcarrier, 0, 47),
            5, "Start Subcarrier");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, MCSindex, RSCMWINS_VAL_MCS_INDEX_MI0, RSCMWINS_VAL_MCS_INDEX_MI13),
            6, "MCS Index");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resourceUnits, RSCMWINS_VAL_RESOURCE_UNITS_NRU01, RSCMWINS_VAL_RESOURCE_UNITS_NRU10),
            7, "Resource Units");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, repetitions, RSCMWINS_VAL_REPETITIONS_NR1, RSCMWINS_VAL_REPETITIONS_NR128),
            8, "Repetitions");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:UDEF:UL %s,%ld,%s,%s,%s", instance, rscmwins_rngDNDSubcarriers.rangeValues[subcarriers].cmdString, startSubcarrier,
             rscmwins_rngDNDMCSIndex.rangeValues[MCSindex].cmdString, rscmwins_rngDNDResourceUnits.rangeValues[resourceUnits].cmdString, rscmwins_rngDNDRepetitions.rangeValues[repetitions].cmdString);
    checkErr(RsCore_Write(instrSession, cmd));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the configured MCS index and the resulting modulation scheme
/// HIFN  and transport block size for DL or UL user-defined scheduling.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:DL:MCSindex?
/// HIFN     CONFigure:NIOT:SIGN:CONNection:UDEFined:UL:MCSindex?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR direction/Sets the direction.
/// HIPAR mcsIndex/Displays MCS Index value.
/// HIPAR modulation/Displays modulation scheme.
/// HIPAR transportBlockSize/Displays Transport Block Size.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionUserMCSIndex(ViSession instrSession,
                                                           ViInt32   instance,
                                                           ViInt32   direction,
                                                           ViInt32*  mcsIndex,
                                                           ViInt32*  modulation,
                                                           ViInt32*  transportBlockSize)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, direction, RSCMWINS_VAL_DIRECTION_DL, RSCMWINS_VAL_DIRECTION_UL),
            3, "Direction");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:UDEF:%s:MCS?", instance, directionArr[direction]);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *mcsIndex = RsCore_FindStringIndex(MCSIndexArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *modulation = RsCore_FindStringIndex(modulationArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *transportBlockSize = atol(p2buf);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Queries scheduling parameters for a UL RMC for NPUSCH format 2. These
/// HIFN  settings are not configurable.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR subcarriers/Number of subcarriers.
/// HIPAR startSubcarrier/Displays the start subcarrier.
/// HIPAR mcsindex/Displays the MCS Index.
/// HIPAR resourceunits/Displays the resource units.
/// HIPAR repetitions/Displays the repetitions.
ViStatus _VI_FUNC rscmwins_QueryConnectionUserFormat(ViSession instrSession,
                                                     ViInt32   instance,
                                                     ViInt32*  subcarriers,
                                                     ViInt32*  startSubcarrier,
                                                     ViInt32*  modulation,
                                                     ViInt32*  resourceUnits,
                                                     ViInt32*  repetitions)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar   response [RS_MAX_MESSAGE_BUF_SIZE] = "";
    ViChar*  p2buf;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "CONF:NIOT:SIGN%ld:CONN:UDEF:FORM?", instance);
    checkErr(RsCore_QueryViString(instrSession, cmd, response));

    checkErr(rscmwins_CheckStatus(instrSession));

    p2buf = strtok(response, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *subcarriers = RsCore_FindStringIndex(subcarriersArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    sscanf(p2buf, "%ld", startSubcarrier);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *modulation = RsCore_FindStringIndex(modulationArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *resourceUnits = RsCore_FindStringIndex(resourceUnitsArr, p2buf);

    p2buf = strtok(NULL, ",");
    if (p2buf == NULL)
        return RSCMWINS_ERROR_INCORRECT_STATE;
    *repetitions = RsCore_FindStringIndex(repetitionsArr, p2buf);

Error:
    //if(p2buf != NULL) free(p2buf);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function configures the HARQ procedure for downlink and uplink
/// HIFN  transmissions.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_ENABLED
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_MAX_NUMBER_OF_TRANSMISSIONS
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_PROCESSES
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:HARQen
/// HIFN     CONFigure:NIOT:SIGN:CONNection:MAXNumtr
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NUMHqpc
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR harqEnable/Enables or disables HARQ for downlink transmissions.
/// HIPAR maxNoOfTransmissions/Specifies the maximum number of transmissions, including initial
/// HIPAR max.No.maxNoOfTransmissions/transmissions and retransmissions.
/// HIPAR noOfHARQProcesses/Specifies the number of HARQ processes.
ViStatus _VI_FUNC rscmwins_ConfigureHARQSettings(ViSession instrSession,
                                                 ViInt32   instance,
                                                 ViBoolean harqEnable,
                                                 ViInt32   maxNoOfTransmissions,
                                                 ViInt32   noOfHARQProcesses)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_ENABLED, harqEnable),
            3, "HARQ Enable");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_MAX_NUMBER_OF_TRANSMISSIONS, maxNoOfTransmissions),
            4, "Max. No. of Transmissions");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_HARQ_PROCESSES, noOfHARQProcesses),
            5, "No. of HARQ Processes");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables non-anchor carrier for the transmission of user data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables non-anchor carrier for the transmission of user data.
ViStatus _VI_FUNC rscmwins_eConfigureConnectionNAC13Enabled(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the UL and DL non-anchor channel for data transmission as UL
/// HIFN  and DL frequency.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR dlFrequency/Specifies the DL non-anchor channel for data transmission as DL
/// HIPAR dlFrequency/frequency.
/// HIPAR ulFrequency/Specifies the UL non-anchor channel for data transmission as UL
/// HIPAR ulFrequency/frequency.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC13ChannelFrequency(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   dlFrequency,
                            ViInt32   ulFrequency)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_DL_CHANNEL_FREQUENCY, dlFrequency),
            3, "DL Frequency");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_UL_CHANNEL_FREQUENCY, ulFrequency),
            3, "UL Frequency");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the bitmap to be used for the scheduling of DL channel
/// HIFN  carrying user data.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR bitmap/Defines the bitmap to be used for the scheduling of DL channel
/// HIPAR bitmap/carrying user data.
/// HIPAR bitmapExplicit/Specifies the allowed DL subframes for anchor or non-anchor data
/// HIPAR bitmapExplicit/transmissions.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC13DLBitmap(ViSession instrSession,
                                                            ViInt32   instance,
                                                            ViInt32   bitmap,
                                                            ViInt32   bitmapExplicit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_DL_BITMAP, bitmap),
            3, "Bitmap");

    if (bitmap == RSCMWINS_VAL_NAC_BITMAP_EXB)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_DL_BITMAP_EXPLICIT, bitmapExplicit),
                4, "Bitmap Explicit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the gap.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR gap/Defines the transmission gap configuration to be applied for DL data
/// HIPAR gap/transmission scheduling for non-anchor carrier.
/// HIPAR threshold/Specifies the maximum number of NPDCCH repetitions (dl-GapThreshold)
/// HIPAR threshold/configured for data transmission via non-anchor carrier before the DL
/// HIPAR threshold/transmission gap is applied.
/// HIPAR periodicity/Selects the transmission gap cycle (64, 128, 256, or 512 subframes)
/// HIPAR periodicity/for data transmission via non-anchor carrier.
/// HIPAR durationCoefficient/Sets the gap duration coefficient (dl-GapDurationCoeff) for data
/// HIPAR durationCoefficient/transmission via non-anchor carrier.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC13Gap(ViSession instrSession,
                                                       ViInt32   instance,
                                                       ViInt32   gap,
                                                       ViInt32   threshold,
                                                       ViInt32   periodicity,
                                                       ViInt32   durationCoefficient)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_DL_GAP, gap),
            3, "Gap");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_GAP_THRESHOLD, threshold),
            4, "threshold");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_GAP_PERIODICITY, periodicity),
            5, "Periodicity");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_GAP_DURATION_COEFFICIENT, durationCoefficient),
            6, "Duration Coefficient");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the power of a non-anchor carrier relative to the level of
/// HIFN  narrowband reference signal (NRS).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR offset/Specifies the power of a non-anchor carrier relative to the level of
/// HIPAR offset/narrowband reference signal (NRS).
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC13NRSPowerOffset(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_13_NRS_POWER_OFFSET, offset),
            3, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the paging.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables/disables the paging via non-anchor carriers.
/// HIPAR instance/No. of firmware application.
/// HIPAR weightAnchor/Enables/disables and sets the relative weight of the anchor paging
/// HIPAR weightAnchor/carrier for uneven paging load distribution across the anchor and
/// HIPAR weightAnchor/non-anchor carriers.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14Paging(ViSession instrSession,
                                                          ViInt32   instance,
                                                          ViBoolean enabled,
                                                          ViInt32   weightAnchor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_ENABLED, enabled),
            3, "enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_WEIGHT_ANCHOR, weightAnchor),
            4, "Weight Anchor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables/disables the uneven paging load distribution across the anchor
/// HIFN  and non-anchor carriers. For the setting of paging weight.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables/disables the uneven paging load distribution across the anchor
/// HIPAR enabled/and non-anchor carriers. For the setting of paging weight.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingWeightAnchorEnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_WEIGHT_ANCHOR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables/disables paging for the corresponding non-anchor carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR enabled/Enables/disables paging for the corresponding non-anchor carrier.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingCarrierState(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   enabledAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_ENABLED,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_ENABLED};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            2, "Carrier");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, enabledAttrArr[carrier], enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the DL carrier.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_CHANNEL_FREQUENCY
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_CHANNEL_OFFSET
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_GAP
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_THRESHOLD
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_PERIODICITY
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_DURATION_COEFFICIENT
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_NRS_POWER_OFFSET
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_CHANNEL_FREQUENCY
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_CHANNEL_OFFSET
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_GAP
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_THRESHOLD
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_PERIODICITY
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_DURATION_COEFFICIENT
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_NRS_POWER_OFFSET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:CHANnel:DL
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:OFFSet:DL
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:DGAP
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:GAPTh
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:GAPPeriod
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:GAPDc
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:NRSPo
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:CHANnel:DL
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:OFFSet:DL
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:DGAP
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:GAPTh
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:GAPPeriod
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:GAPDc
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:NRSPo
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR channelFrequency/Specifies the DL non-anchor paging carrier as DL frequency or channel
/// HIPAR channelFrequency/No.
/// HIPAR carrier/Selects the carrier.
/// HIPAR instance/No. of firmware application.
/// HIPAR channelOffset/Specifies the power offset for non-anchor paging carrier relative to
/// HIPAR channelOffset/NRS level.
/// HIPAR gap/Defines the transmission gap configuration to be applied for
/// HIPAR gap/non-anchor paging scheduling.
/// HIPAR gapThreshold/Specifies the maximum number of NPDCCH repetitions configured for
/// HIPAR gapThreshold/non-anchor paging carrier before the DL transmission gap is applied.
/// HIPAR gapPeriodicity/Selects the transmission gap cycle (64, 128, 256, or 512 subframes)
/// HIPAR gapPeriodicity/for data transmission via non-anchor carrier.
/// HIPAR gapDurationCoefficient/Sets the gap duration coefficient (dl-GapDurationCoeff) for paging
/// HIPAR gapDurationCoefficient/non-anchor carrier.
/// HIPAR nrsPowerOffset/Specifies the power offset for non-anchor paging carrier relative to
/// HIPAR nrsPowerOffset/NRS level.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingDLCarrier(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViInt32   channelFrequency,
                            ViInt32   channelOffset,
                            ViInt32   gap,
                            ViInt32   gapThreshold,
                            ViInt32   gapPeriodicity,
                            ViInt32   gapDurationCoefficient,
                            ViInt32   NRSPowerOffset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   channelFrequencyAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_CHANNEL_FREQUENCY,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_CHANNEL_FREQUENCY};
    ViAttr   channelOffsetAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_CHANNEL_OFFSET,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_CHANNEL_OFFSET};
    ViAttr   gapAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_GAP,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_GAP};
    ViAttr   gapThresholdAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_THRESHOLD,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_THRESHOLD};
    ViAttr   gapPeriodicityAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_PERIODICITY,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_PERIODICITY};
    ViAttr   gapDurationCoefficientAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_GAP_DURATION_COEFFICIENT,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_GAP_DURATION_COEFFICIENT};
    ViAttr   NRSPowerOffsetAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_NRS_POWER_OFFSET,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_NRS_POWER_OFFSET};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, channelFrequencyAttrArr[carrier], channelFrequency),
            4, "Channel Frequency");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, channelOffsetAttrArr[carrier], channelOffset),
            5, "Channel Offset");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, gapAttrArr[carrier], gap),
            6, "gap");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, gapThresholdAttrArr[carrier], gapThreshold),
            7, "Gap Threshold");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, gapPeriodicityAttrArr[carrier], gapPeriodicity),
            8, "Gap Periodicity");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, gapDurationCoefficientAttrArr[carrier], gapDurationCoefficient),
            9, "Gap Duration Coefficient");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, NRSPowerOffsetAttrArr[carrier], NRSPowerOffset),
            10, "NRS Power Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines the bitmap to schedule paging for the corresponding carrier.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_BITMAP
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_BITMAP_EXPLICIT
/// HIFN     RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_BITMAP
/// HIFN
/// HIFN  RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_BITMAP_EXPLICIT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:DBMap
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:ONE:EXBitmap
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:DBMap
/// HIFN     CONFigure:NIOT:SIGN:CONNection:NAC:PAGing:CARRier:TWO:EXBitmap
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR bitmap/Defines the bitmap to schedule paging for the corresponding carrier.
/// HIPAR bitmapExplicit/Specifies the allowed DL subframes for anchor or non-anchor paging
/// HIPAR bitmapExplicit/scheduling.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingDLCarrierBitmap(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViInt32   bitmap,
                            ViInt32   bitmapExplicit)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   bitmapAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_BITMAP,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_BITMAP};
    ViAttr   bitmapExplicitAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_DL_BITMAP_EXPLICIT,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_DL_BITMAP_EXPLICIT};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, bitmapAttrArr[carrier], bitmap),
            4, "Bitmap");

    if (bitmap == RSCMWINS_VAL_NAC_BITMAP_EXB)
    {
        viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, bitmapExplicitAttrArr[carrier], bitmapExplicit),
                5, "Bitmap Explicit");
    }

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the paging carrier PCCH.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR enabled/Enables/disables paging via the corresponding non-archor carrier.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR numOfRepetitions/Enables/disables and sets the maximum number of repetitions for paging
/// HIPAR numOfRepetitions/transmissions over the corresponding non-anchor carrier.
/// HIPAR pagingWeight/Enables/disables and sets the relative weight of the non-anchor paging
/// HIPAR pagingWeight/carrier for uneven paging load distribution across the carriers.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingCarrierPCCH(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViBoolean enabled,
                            ViInt32   numOfRepetitions,
                            ViInt32   pagingWeight)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   enabledAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_PCCH_ENABLED,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_PCCH_ENABLED};
    ViAttr   numOfRepetitionsAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_PCCH_NUM_OF_REPETITIONS,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_PCCH_NUM_OF_REPETITIONS};
    ViAttr   pagingWeightAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_PCCH_PAGING_WEIGHT,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_PCCH_PAGING_WEIGHT};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, enabledAttrArr[carrier], enabled),
            4, "enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, numOfRepetitionsAttrArr[carrier], numOfRepetitions),
            5, "Num Of Repetitions");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, pagingWeightAttrArr[carrier], pagingWeight),
            6, "pagingWeight");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Acivates/deactivates the uneven paging load distribution across the
/// HIFN  non-anchor carriers. For the setting of paging weight.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR enabled/Acivates/deactivates the uneven paging load distribution across the
/// HIPAR enabled/non-anchor carriers. For the setting of paging weight.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingCarrierPCCHWeightEnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   enabledAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_PCCH_PAGING_WEIGHT_ENABLED,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_PCCH_PAGING_WEIGHT_ENABLED};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, enabledAttrArr[enabled], enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the WUS.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR enabled/Enables or disables wake-up signal (WUS) for non-anchor paging
/// HIPAR enabled/carriers.
/// HIPAR maxDurationFactor/Sets the maximal duration of WUS for non-anchor paging carriers
/// HIPAR maxDurationFactor/expressed in factor of subframes 1/x.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14PagingCarrierWUS(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViBoolean enabled,
                            ViInt32   maxDurationFactor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   enabledAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_WUS_ENABLED,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_WUS_ENABLED};
    ViAttr   maxDurationFactorAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_1_MAX_DURATION_FACTOR,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_PAGING_CARRIER_2_MAX_DURATION_FACTOR};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, enabledAttrArr[carrier], enabled),
            4, "enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, maxDurationFactorAttrArr[carrier], maxDurationFactor),
            5, "Max Duration Factor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Enables the random access (RA) procedure via non-anchor carriers.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables the random access (RA) procedure via non-anchor carriers.
/// HIPAR anchorEnabled/Enables/disables anchor carrier NPRACH resources.
/// HIPAR anchor/Sets the selection probability for the anchor carrier NPRACH resource.
/// HIPAR anchor/All non-anchor carriers NPRACH resources have equal probability between
/// HIPAR anchor/them.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14RACH(ViSession instrSession,
                                                        ViInt32   instance,
                                                        ViBoolean enabled,
                                                        ViBoolean anchorEnabled,
                                                        ViInt32   anchor)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_ENABLED, enabled),
            3, "enabled");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_NPRACH_PROBABILITY_ANCHOR_ENABLED, anchorEnabled),
            4, "Anchor Enabled");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_NPRACH_PROBABILITY_ANCHOR, anchor),
            5, "Anchor");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables the random access via the corresponding non-anchor carrier.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR enabled/Enables the random access via the corresponding non-anchor carrier.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14RACHCarrierEnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
    ViAttr   enabledAttrArr[] = {0,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_CARRIER_1_RACH_ENABLED,
             RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_CARRIER_2_RACH_ENABLED};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            2, "Carrier");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, enabledAttrArr[carrier], enabled),
            4, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the UL channel.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR carrier/Selects the carrier.
/// HIPAR channelFrequency/Specifies the UL non-anchor RA carrier as UL frequency or channel No.
/// HIPAR channelOffset/Sets the UL carrier channel offset of the NB-IoT channel number to
/// HIPAR channelOffset/EARFCN.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNAC14RACHULCarrier(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   carrier,
                            ViInt32   channelFrequency,
                            ViInt32   channelOffset)
{
     ViStatus error = VI_SUCCESS;
     ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";
     ViAttr   channelFrequencyAttrArr[] = {0,
              RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_CARRIER_1_UL_CHANNEL_FREQUENCY,
              RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_CARRIER_2_UL_CHANNEL_FREQUENCY};
     ViAttr   channelOffsetAttrArr[] = {0,
              RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_CARRIER_1_UL_CHANNEL_OFFSET,
              RSCMWINS_ATTR_CONFIGURE_CONNECTION_NAC_14_RACH_CARRIER_2_UL_CHANNEL_OFFSET};

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, carrier, 1, 2),
            3, "Carrier");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, channelFrequencyAttrArr[carrier], channelFrequency),
            4, "Channel Frequency");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, channelOffsetAttrArr[carrier], channelOffset),
            5, "Channel Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures NPDCCH search space.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR maxNoRepetitionsRmax/Specifies the maximum number of repetitions for NPDCCH transmissions
/// HIPAR maxNoRepetitionsRmax/for the UE.
/// HIPAR instance/No. of firmware application.
/// HIPAR noRepetitions/Specifies the number of repetitions for NPDCCH transmissions related
/// HIPAR noRepetitions/to R max.
/// HIPAR startSF/Specifies NPDCCH starting subframe.
/// HIPAR offset/Specifies the NPDCCH offset related to R max.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNPDCCHUESpecificSearchSpace(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   maxNoRepetitionsRmax,
                            ViInt32   noRepetitions,
                            ViInt32   startSF,
                            ViInt32   offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NPDCCH_MAX_NO_REPETITIONS_RMAX, maxNoRepetitionsRmax),
            3, "Max No Repetitions Rmax");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NPDCCH_NO_REPETITIONS, noRepetitions),
            4, "No Repetitions");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NPDCCH_START_SF, startSF),
            5, "Start SF");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NPDCCH_OFFSET, offset),
            6, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the NPDCCH search space parameters.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR noRepetitions/Queries the number of repetitions R for actual NPDCCH transmissions.
/// HIPAR period/Queries the periodicity T of NPDCCH transmissions.
/// HIPAR offset/Queries the NPDCCH offset in subframes.
ViStatus _VI_FUNC rscmwins_QueryConnectionNPDCCHUESpecificSearchSpace(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32*  noRepetitions,
                            ViInt32*  period,
                            ViInt32*  offset)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_CONNECTION_NPDCCH_NO_REPETITIONS, noRepetitions),
            3, "No Repetitions");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_CONNECTION_NPDCCH_PERIOD, period),
            4, "Period");

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_CONNECTION_NPDCCH_OFFSET, offset),
            5, "Offset");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Specifies the maximum number of repetitions for NPDCCH transmissions
/// HIFN  with paging (R max).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR maxNoRepetitionsRmax/Specifies the maximum number of repetitions for NPDCCH transmissions
/// HIPAR maxNoRepetitionsRmax/with paging (R max).
ViStatus _VI_FUNC rscmwins_ConfigureConnectionNPDCCHCSSMaxNoRepetitionsRmax(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   maxNoRepetitionsRmax)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_NPDCCH_CSS_MAX_NO_REPETITIONS_RMAX, maxNoRepetitionsRmax),
            3, "Max No Repetitions Rmax");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Enables or disables DRX in connected status.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/Enables or disables DRX in connected status.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionConnectedDRXState(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the connected DRX.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR onDurationTimer/Configures the onDurationTimer of a DRX cycle.
/// HIPAR inactivityTimer/No. of NPDCCH subframes after the subframe in which the DL or UL HARQ
/// HIPAR inactivityTimer/RTT timer expires
/// HIPAR retransmissionTimer/Configures the DRX RetransmissionTimer.
/// HIPAR cycle/Configures the duration of one DRX cycle.
/// HIPAR startOffset/Configures the DRX Start Offset, shifting all DRX cycles.
/// HIPAR ulRetransTimer/Configures the DRX UL retransmission timer.
ViStatus _VI_FUNC rscmwins_ConfigureConnectionConnectedDRXOnDurationTimer(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   onDurationTimer,
                            ViInt32   inactivityTimer,
                            ViInt32   retransmissionTimer,
                            ViInt32   cycle,
                            ViInt32   startOffset,
                            ViInt32   ulRetransTimer)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_ON_DURATION_TIMER, onDurationTimer),
            3, "On Duration Timer");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_INACTIVITY_TIMER, inactivityTimer),
            4, "Inactivity Timer");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_RETRANSMISSION_TIMER, retransmissionTimer),
            5, "Retransmission Timer");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_CYCLE, cycle),
            6, "Cycle");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_START_OFFSET, startOffset),
            7, "Start Offset");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_CONNECTION_CONNECTED_DRX_UL_RETRANS_TIMER, ulRetransTimer),
            8, "Ul Retrans Timer");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Configures the Scheduling Request.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_SCHEDULING_REQUEST_WITH_HARQ
/// HIFN     RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ
/// HIFN
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN<i>:CONNection:SREQuest:HARQack:WITH
/// HIFN     CONFigure:NIOT:SIGN<i>:CONNection:SREQuest:HARQack:WOUT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR withHARQ/Enables or disables scheduling requests with HARQ-ack for NB-IoT
/// HIPAR withHARQ/transmission.
/// HIPAR withoutHARQ/Enables or disables scheduling requests without HARQ-ack for NB-IoT
/// HIPAR withoutHARQ/transmission.
ViStatus _VI_FUNC rscmwins_ConfigureSchedulingRequest(ViSession instrSession,
                                                      ViInt32   instance,
                                                      ViBoolean withHARQ,
                                                      ViBoolean withoutHARQ)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITH_HARQ, withHARQ),
            3, "With HARQ");

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ, withoutHARQ),
            4, "Without HARQ");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Configures the Scheduling Request Without HARQ.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR alpha/Sets the fractional power control parameter alpha-r15 for SR without
/// HIPAR alpha/HARQ-ACK.
/// HIPAR carrierIndex/Sets the NPRACH carrier index for SR without HARQ-ack.
/// HIPAR resourceIndex/Selects the index of the NPRACH resource in the NPRACH-ParametersList
/// HIPAR resourceIndex/for the UL carrier indicated by nprach-CarrierIndex.
/// HIPAR sr/Sets the p0-SR-r15 to specify target power for SR without HARQ-ACK.
/// HIPAR indexFormat/Selects the NPRACH resource format.
/// HIPAR formatValue/Sets the index of the subcarrier in NPRACH-ParametersList for the
/// HIPAR formatValue/indicated UL carrier.
ViStatus _VI_FUNC rscmwins_ConfigureSchedulingRequestWithoutHARQ(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViInt32   alpha,
                            ViInt32   carrierIndex,
                            ViInt32   resourceIndex,
                            ViReal64  sr,
                            ViInt32   indexFormat,
                            ViInt32   formatValue)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_ALPHA, alpha),
            3, "Alpha");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_CARRIER_INDEX, carrierIndex),
            4, "Carrier Index");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_RESOURCE_INDEX, resourceIndex),
            5, "Resource Index");

    viCheckParm(rscmwins_SetAttributeViReal64(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_SR, sr),
            6, "SR");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_SUBCARRIER_INDEX_FORMAT, indexFormat),
            7, "Index Format");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_WITHOUT_HARQ_SUBCARRIER_INDEX_FORMAT_VALUE, formatValue),
            8, "Format Value");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If enabled, the R&S CMW ignores scheduling requests from the UE and
/// HIFN  sends no response.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR enabled/If enabled, the R&S CMW ignores scheduling requests from the UE and
/// HIPAR enabled/sends no response.
ViStatus _VI_FUNC rscmwins_ConfigureSchedulingRequestNoResponseToSREnabled(
                            ViSession instrSession,
                            ViInt32   instance,
                            ViBoolean enabled)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViBoolean(instrSession, repCap, RSCMWINS_ATTR_SCHEDULING_REQUEST_NO_RESPONSE_TO_SR_ENABLED, enabled),
            3, "Enabled");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Starts, stops, or aborts the measurement:INITiate... starts or
/// HIFN  restarts the measurement. The measurement enters the 'RUN'
/// HIFN  state.STOP... halts the measurement immediately. The measurement enters
/// HIFN  the 'RDY' state. Measurement results are kept. The resources remain
/// HIFN  allocated to the measurement.ABORt... halts the measurement
/// HIFN  immediately. The measurement enters the 'OFF' state. All measurement
/// HIFN  values are set to NAV. Allocated resources are released.Use
/// HIFN  FETCh...STATe? to query the current measurement state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_INITIATE_BLER_MEASUREMENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     INITiate:NIOT:SIGN:BLER
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_InitiateBLERMeasurement(ViSession instrSession,
                                                   ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_INITIATE_BLER_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts, stops, or aborts the measurement:INITiate... starts or
/// HIFN  restarts the measurement. The measurement enters the 'RUN'
/// HIFN  state.STOP... halts the measurement immediately. The measurement enters
/// HIFN  the 'RDY' state. Measurement results are kept. The resources remain
/// HIFN  allocated to the measurement.ABORt... halts the measurement
/// HIFN  immediately. The measurement enters the 'OFF' state. All measurement
/// HIFN  values are set to NAV. Allocated resources are released.Use
/// HIFN  FETCh...STATe? to query the current measurement state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_STOP_BLER_MEASUREMENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     STOP:NIOT:SIGN:BLER
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_StopBLERMeasurement(ViSession instrSession,
                                               ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_STOP_BLER_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Starts, stops, or aborts the measurement:INITiate... starts or
/// HIFN  restarts the measurement. The measurement enters the 'RUN'
/// HIFN  state.STOP... halts the measurement immediately. The measurement enters
/// HIFN  the 'RDY' state. Measurement results are kept. The resources remain
/// HIFN  allocated to the measurement.ABORt... halts the measurement
/// HIFN  immediately. The measurement enters the 'OFF' state. All measurement
/// HIFN  values are set to NAV. Allocated resources are released.Use
/// HIFN  FETCh...STATe? to query the current measurement state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_ABORT_BLER_MEASUREMENT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     ABORt:NIOT:SIGN:BLER
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
ViStatus _VI_FUNC rscmwins_AbortBLERMeasurement(ViSession instrSession,
                                                ViInt32   instance)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    checkErr(rscmwins_SetAttributeViString(instrSession, repCap, RSCMWINS_ATTR_ABORT_BLER_MEASUREMENT, NULL));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the main measurement state. Use FETCh:...:STATe:ALL? to query
/// HIFN  the measurement state including the substates. Use&nbsp;INITiate...,
/// HIFN  STOP..., ABORt...to change the measurement state.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_QUERY_BLER_STATE
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:NIOT:SIGN:BLER:STATe?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR blerState/Queries the main measurement state. Use FETCh:...:STATe:ALL? to query
/// HIPAR blerState/the measurement state including the substates. Use&nbsp;INITiate...,
/// HIPAR blerState/STOP..., ABORt...to change the measurement state.
ViStatus _VI_FUNC rscmwins_QueryBLERState(ViSession instrSession,
                                          ViInt32   instance,
                                          ViInt32*  blerState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_QUERY_BLER_STATE, blerState),
            3, "BLER State");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Queries the main measurement state and the measurement substates. Both
/// HIFN  measurement substates are relevant for running measurements only.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:NIOT:SIGN:BLER:STATe:ALL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR mainState/Displays the main state.
/// HIPAR synchronizationState/Displays the synchronization state.
/// HIPAR resourceState/
ViStatus _VI_FUNC rscmwins_QueryBLERStateAll(ViSession instrSession,
                                             ViInt32   instance,
                                             ViInt32*  mainState,
                                             ViInt32*  synchronizationState,
                                             ViInt32*  resourceState)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:SIGN%ld:BLER:STAT:ALL?", instance);
    checkErr(rscmwins_QueryMeasStatus(instrSession, cmd, mainState, synchronizationState, resourceState));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Defines a timeout for the measurement. The timer is started when the
/// HIFN  measurement is
/// HIFN     initiated via a READ or INIT command. It is not started if the
/// HIFN  measurement is initiated
/// HIFN     manually ([ON | OFF] key or [RESTART | STOP] key).
/// HIFN     When the measurement has completed the first measurement cycle (first
/// HIFN  single shot),
/// HIFN     the statistical depth is reached and the timer is reset.
/// HIFN     If the first measurement cycle has not been completed when the timer
/// HIFN  expires, the
/// HIFN     measurement is stopped. The measurement state changes to RDY. The
/// HIFN  reliability indicator
/// HIFN     is set to 1, indicating that a measurement timeout occurred. Still
/// HIFN  running READ,
/// HIFN     FETCh or CALCulate commands are completed, returning the available
/// HIFN  results. At
/// HIFN     least for some results, there are no values at all or the statistical
/// HIFN  depth has not been
/// HIFN     reached.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_CONFIGURE_BLER_TIMEOUT
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:BLER:TOUT
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR blerTimeout/Defines a timeout for the measurement.
ViStatus _VI_FUNC rscmwins_ConfigureBLERTimeout(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   blerTimeout)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_BLER_TIMEOUT, blerTimeout),
            2, "BLER Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  The settings configure the scope of the measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_CONFIGURE_BLER_REPETITION
/// HIFN     RSCMWINS_ATTR_CONFIGURE_BLER_SAMPLES
/// HIFN     RSCMWINS_ATTR_CONFIGURE_BLER_ALGORITHM
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     CONFigure:NIOT:SIGN:BLER:REPetition
/// HIFN     CONFigure:NIOT:SIGN:BLER:SAMPles
/// HIFN     CONFigure:NIOT:SIGN:BLER:ERCalc
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR repetition/Specifies whether the measurement is stopped after a single shot or
/// HIPAR repetition/repeated continuously.
/// HIPAR numberofSamples/Defines the number of DL transmissions to be processed per measurement
/// HIPAR numberofSamples/cycle.
/// HIPAR errorRatioCalculation/Selects the formula to be used for calculation of the BLER from the
/// HIPAR errorRatioCalculation/number of ACK, NACK and DTX.
ViStatus _VI_FUNC rscmwins_ConfigureBLERMeasurement(ViSession instrSession,
                                                    ViInt32   instance,
                                                    ViInt32   repetition,
                                                    ViInt32   numberofSamples,
                                                    ViInt32   errorRatioCalculation)
{
    ViStatus error = VI_SUCCESS;
    ViChar   repCap[RS_REPCAP_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_instanceRepCap(instrSession, instance, repCap));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_BLER_REPETITION, repetition),
            3, "Repetition");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_BLER_SAMPLES, numberofSamples),
            4, "Number of Samples");

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, repCap, RSCMWINS_ATTR_CONFIGURE_BLER_ALGORITHM, errorRatioCalculation),
            5, "Error Ratio Calculation");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  Returns the absolute results of the BLER measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:NIOT:SIGN:BLER:ABSolute?
/// HIFN     FETCh:NIOT:SIGN:BLER:RELative?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR resultType/Selects the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwins_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwins_FetchMeasurementBLER(ViSession instrSession,
                                                ViInt32   instance,
                                                ViInt32   resultType,
                                                ViInt32   arraySize,
                                                ViInt32*  reliability,
                                                ViReal64  _VI_FAR results[],
                                                ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");
    viCheckParm(RsCore_InvalidViInt32Range(instrSession, resultType, RSCMWINS_VAL_RESULT_TYPE_ABS, RSCMWINS_VAL_RESULT_TYPE_REL),
            3, "Result Type");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:SIGN%ld:BLER:%s?", instance, resultTypeArr[resultType]);

    checkErr(rscmwins_GetMeas(instrSession, cmd, 1000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Returns the absolute results of the BLER measurement.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FETCh:NIOT:SIGN:BLER:UL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instance/No. of firmware application.
/// HIPAR resultType/Selects the result type.
/// HIPAR arraySize/Sets the number of values allocated for result array.
/// HIPAR reliability/Integer value of the reliability indicator for the entire measurement
/// HIPAR reliability/result. To translate this code into human-readable string, use the
/// HIPAR reliability/rscmwins_ReliabilityIndicator.
/// HIPAR results/Returns the results of the measurement.
/// HIPAR returnedValues/Returns number of results returned from the instrument.
ViStatus _VI_FUNC rscmwins_FetchMeasurementBLERUL(ViSession instrSession,
                                                  ViInt32   instance,
                                                  ViInt32   resultType,
                                                  ViInt32   arraySize,
                                                  ViInt32*  reliability,
                                                  ViReal64  _VI_FAR results[],
                                                  ViInt32*  returnedValues)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE] = "";
             resultType;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, instance, 1, 16),
            2, "Instance");

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "FETC:NIOT:SIGN%ld:BLER:UL?", instance);

    checkErr(rscmwins_GetMeas(instrSession, cmd, 1000, arraySize, reliability, results, returnedValues));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViInt32 attribute.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViUInt32  attributeID,
                                               ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the value of a ViReal64 attribute.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the value of a Vi String attribute. It is also used
/// HIFN  to set an event-type attribute (attribute without a parameter). In this
/// HIFN  case the string value is ignored.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes. If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible. They handle order dependencies and
/// HIFN  multithread locking for you. In addition, they perform status checking
/// HIFN  only after setting all of the attributes. In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change. Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeId,
                                                ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViString(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a RawString attribute. It is also used
/// HIFN  to set an event-type attribute (attribute without a parameter). In this
/// HIFN  case the string value is ignored.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes. If the
/// HIFN  attribute represents an instrument state, this function performs
/// HIFN  instrument I/O in the following cases:
/// HIFN
/// HIFN     - State caching is disabled for the entire session or for the
/// HIFN  particular attribute.
/// HIFN
/// HIFN     - State caching is enabled and the currently cached value is invalid
/// HIFN  or is different than the value you specify.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible. They handle order dependencies and
/// HIFN  multithread locking for you. In addition, they perform status checking
/// HIFN  only after setting all of the attributes. In contrast, when you set
/// HIFN  multiple attributes using the SetAttribute functions, the functions
/// HIFN  check the instrument status after each call.
/// HIFN
/// HIFN     Also, when state caching is enabled, the high-level functions that
/// HIFN  configure multiple attributes perform instrument I/O only for the
/// HIFN  attributes whose value you change. Thus, you can safely call the
/// HIFN  high-level functions without the penalty of redundant instrument I/O.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeRawString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViRawString(instrSession, repeatedCapabilityName,
                                           attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function sets the value of a ViBoolean attribute.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeViBoolean(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeId,
                                                 ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function sets the value of a ViSession attribute.
/// HIFN
/// HIFN     This is a low-level function that you can use to set the values of
/// HIFN  instrument-specific attributes and inherent attributes.
/// HIFN
/// HIFN     This instrument driver contains high-level functions that set most of
/// HIFN  the instrument attributes. It is best to use the high-level driver
/// HIFN  functions as much as possible.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value to which you want to set the attribute.
ViStatus _VI_FUNC rscmwins_SetAttributeViSession(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeId,
                                                 ViSession attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_SetAttributeViSession(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Function used by Hi-level functions to obtain a repCap name from:
/// HIFN - attributeID
/// HIFN - repCapNameID
/// HIFN - 0-based index in the repCapNames
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwgprf_init or
/// HIPAR instrSession/rscmwgprf_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeId/Pass the ID of an attribute.
/// HIPAR repCapNameId/RepCap Name ID.
/// HIPAR index/Zero based index in the repCapNames.
/// HIPAR bufferSize/Pass the number of bytes in the string you allocate for the repCapName.
/// HIPAR repCapName/String returning RepCap name
ViStatus _VI_FUNC rscmwins_GetAttributeRepCapName(ViSession instrSession,
                                                  ViUInt32  attributeId,
                                                  ViString  repCapNameId,
                                                  ViInt32   index,
                                                  ViInt32   bufferSize,
                                                  ViChar    repCapName[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetAttributeRepCapName(instrSession, attributeId, repCapNameId, index, bufferSize, repCapName));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the repeated capability id(s) that belongs to
/// HIFN  the attribute you specify. More then one id is listed in comma
/// HIFN  separated string.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id(s) parameter.
/// HIPAR repeatedCapabilityId_s/The buffer in which the function returns the repeated capability id(s)
/// HIPAR repeatedCapabilityId_s/belonging to the attribute. The buffer must be of type ViChar and have
/// HIPAR repeatedCapabilityId_s/at least as many bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rscmwins_GetAttributeRepeatedCapabilityIds(ViSession instrSession,
                                                             ViUInt32  attributeID,
                                                             ViInt32   bufferSize,
                                                             ViChar    _VI_FAR repeatedCapabilityIds[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            3, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNameIds(instrSession, attributeID, bufferSize, repeatedCapabilityIds));

Error:
    return error;
}


/// HIFN  This function returns the repeated capability name(s) that belongs to
/// HIFN  the attribute and single repeated capability id you may specify. More
/// HIFN  then one name is listed in comma separated string.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR repeatedCapabilityId/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityId/this parameter defines the Id of the repeated capability on which to
/// HIPAR repeatedCapabilityId/query repeated capability id name(s).
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Repeated Capability Id Name(s) parameter.
/// HIPAR repeatedCapabilityIdName_s/The buffer in which the function returns the repeated capability id
/// HIPAR repeatedCapabilityIdName_s/name(s) belonging to the attribute. The buffer must be of type ViChar
/// HIPAR repeatedCapabilityIdName_s/and have at least as many bytes as indicated in the Buffer Size
/// HIPAR repeatedCapabilityIdName_s/parameter.
ViStatus _VI_FUNC rscmwins_GetAttributeRepeatedCapabilityIdNames(
                            ViSession instrSession,
                            ViUInt32  attributeID,
                            ViString  repeatedCapabilityId,
                            ViInt32   bufferSize,
                            ViChar    repeatedCapabilityIdNames[])
{
    ViStatus error = VI_SUCCESS;

    viCheckParm(RsCore_InvalidViInt32Range(instrSession, bufferSize, 1, RS_MAX_SHORT_MESSAGE_BUF_SIZE),
            4, "Buffer Size");

    checkErr(RsCore_GetAttributeRepCapNamesAll(instrSession,
        attributeID,
        repeatedCapabilityId,
        bufferSize,
        repeatedCapabilityIdNames));

Error:
    return error;
}


/// HIFN  This function queries the value of a ViInt32 attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViInt32 variable.
ViStatus _VI_FUNC rscmwins_GetAttributeViInt32(ViSession instrSession,
                                               ViString  repeatedCapabilityName,
                                               ViUInt32  attributeId,
                                               ViInt32*  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);
    error = RsCore_GetAttributeViInt32(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a ViReal64 attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViReal64 variable.
ViStatus _VI_FUNC rscmwins_GetAttributeViReal64(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeId,
                                                ViReal64* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViReal64(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a ViString attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIFN
/// HIFN     You must provide a ViChar array to serve as a buffer for the value.
/// HIFN  You pass the number of bytes in the buffer as the Buffer Size
/// HIFN  parameter. If the current value of the attribute, including the
/// HIFN  terminating NUL byte, is larger than the size you indicate in the
/// HIFN  Buffer Size parameter, the function copies Buffer Size - 1 bytes into
/// HIFN  the buffer, places an ASCII NUL byte at the end of the buffer, and
/// HIFN  returns the buffer size you must pass to get the entire value.  For
/// HIFN  example, if the value is "123456" and the Buffer Size is 4, the
/// HIFN  function places "123" into the buffer and returns 7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute. The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rscmwins_GetAttributeViString(ViSession instrSession,
                                                ViString  repeatedCapabilityName,
                                                ViUInt32  attributeID,
                                                ViInt32   bufferSize,
                                                ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a RawString attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIFN
/// HIFN     You must provide a ViChar array to serve as a buffer for the value.
/// HIFN  You pass the number of bytes in the buffer as the Buffer Size
/// HIFN  parameter. If the current value of the attribute, including the
/// HIFN  terminating NUL byte, is larger than the size you indicate in the
/// HIFN  Buffer Size parameter, the function copies Buffer Size - 1 bytes into
/// HIFN  the buffer, places an ASCII NUL byte at the end of the buffer, and
/// HIFN  returns the buffer size you must pass to get the entire value.  For
/// HIFN  example, if the value is "123456" and the Buffer Size is 4, the
/// HIFN  function places "123" into the buffer and returns 7.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Attribute Value parameter.
/// HIPAR attributeValue/The buffer in which the function returns the current value of the
/// HIPAR attributeValue/attribute. The buffer must be of type ViChar and have at least as many
/// HIPAR attributeValue/bytes as indicated in the Buffer Size parameter.
ViStatus _VI_FUNC rscmwins_GetAttributeRawString(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViInt32   bufferSize,
                                                 ViChar    attributeValue[])
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViRawString(instrSession, repeatedCapabilityName,
                                           attributeID, RS_VAL_DIRECT_USER_CALL, bufferSize, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN  This function queries the value of a ViBoolean attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViBoolean variable.
ViStatus _VI_FUNC rscmwins_GetAttributeViBoolean(ViSession  instrSession,
                                                 ViString   repeatedCapabilityName,
                                                 ViUInt32   attributeId,
                                                 ViBoolean* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViBoolean(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the value of a ViSession attribute.
/// HIFN
/// HIFN     You can use this function to get the values of instrument- specific
/// HIFN  attributes and inherent attributes.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Returns the current value of the attribute.  Pass the address of a
/// HIPAR attributeValue/ViSession variable.
ViStatus _VI_FUNC rscmwins_GetAttributeViSession(ViSession  instrSession,
                                                 ViString   repeatedCapabilityName,
                                                 ViUInt32   attributeId,
                                                 ViSession* attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_GetAttributeViSession(instrSession, repeatedCapabilityName, attributeId, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks the validity of a value you specify for a ViInt32
/// HIFN  attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value which you want to verify as a valid value for the
/// HIPAR attributeValue/attribute.
ViStatus _VI_FUNC rscmwins_CheckAttributeViInt32(ViSession instrSession,
                                                 ViString  repeatedCapabilityName,
                                                 ViUInt32  attributeID,
                                                 ViInt32   attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViInt32(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks the validity of a value you specify for a
/// HIFN  ViReal64 attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value which you want to verify as a valid value for the
/// HIPAR attributeValue/attribute.
ViStatus _VI_FUNC rscmwins_CheckAttributeViReal64(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViUInt32  attributeID,
                                                  ViReal64  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViReal64(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks the validity of a value you specify for a
/// HIFN  ViString attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value which you want to verify as a valid value for the
/// HIPAR attributeValue/attribute.
ViStatus _VI_FUNC rscmwins_CheckAttributeViString(ViSession instrSession,
                                                  ViString  repeatedCapabilityName,
                                                  ViUInt32  attributeID,
                                                  ViString  attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViString(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks the validity of a value you specify for a
/// HIFN  ViBoolean attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value which you want to verify as a valid value for the
/// HIPAR attributeValue/attribute.
ViStatus _VI_FUNC rscmwins_CheckAttributeViBoolean(ViSession instrSession,
                                                   ViString  repeatedCapabilityName,
                                                   ViUInt32  attributeID,
                                                   ViBoolean attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViBoolean(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function checks the validity of a value you specify for a
/// HIFN  ViSession attribute.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR repeatedCapabilityName/If the attribute applies to a repeated capability such as a trace,
/// HIPAR repeatedCapabilityName/this parameter specifies the name of the repeated capability on which
/// HIPAR repeatedCapabilityName/to set the value of the attribute. If the attribute does not apply to a
/// HIPAR repeatedCapabilityName/repeated capability, then pass NULL or an empty string.
/// HIPAR attributeID/Pass the ID of an attribute.
/// HIPAR attributeValue/Pass the value which you want to verify as a valid value for the
/// HIPAR attributeValue/attribute.
ViStatus _VI_FUNC rscmwins_CheckAttributeViSession(ViSession instrSession,
                                                   ViString  repeatedCapabilityName,
                                                   ViUInt32  attributeID,
                                                   ViSession attributeValue)
{
    ViStatus error = VI_SUCCESS;

    (void)RsCore_LockSession(instrSession);

    error = RsCore_CheckAttributeViSession(instrSession, repeatedCapabilityName, attributeID, RS_VAL_DIRECT_USER_CALL, attributeValue);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function resets the instrument to a known state and sends
/// HIFN  initialization commands to the instrument. The initialization commands
/// HIFN  set instrument settings such as Headers Off, Short Command form, and
/// HIFN  Data Transfer Binary to the state necessary for the operation of the
/// HIFN  instrument driver.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *RST
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_reset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    if (RsCore_Simulating(instrSession) == VI_FALSE)
    {
        checkErr(rscmwins_ClearStatus(instrSession));
        checkErr(RsCore_Write(instrSession, "*RST;*CLS")); // IGNORE: Check double write
        checkErr(RsCore_QueryViStringShort(instrSession, "*OPC?", NULL));
    }

    checkErr(rscmwins_DefaultInstrSetup(instrSession));
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);

    return error;
}


/// HIFN  This function runs the instrument's self test routine and returns the
/// HIFN  test result(s).
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *TST?
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR selfTestResult/This control contains the value returned from the instrument self
/// HIPAR selfTestResult/test.  Zero means success.  For any other code, see the device's
/// HIPAR selfTestResult/operator's manual.
/// HIPAR selfTestMessage/This control contains the string returned from the self test. See the
/// HIPAR selfTestMessage/device's operation manual for an explanation of the string's contents.
ViStatus _VI_FUNC rscmwins_self_test(ViSession instrSession,
                                     ViInt16*  selfTestResult,
                                     ViChar    testMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, selfTestResult), 2, "Null address for Test Result");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, testMessage), 3, "Null address for Test Message");

    checkErr(RsCore_SelfTest(instrSession, selfTestResult, 48, testMessage));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the revision numbers of the instrument driver
/// HIFN  and instrument firmware.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR instrumentDriverRevision/Returns the instrument driver software revision numbers in the form of
/// HIPAR instrumentDriverRevision/a string.
/// HIPAR firmwareRevision/Returns the instrument firmware revision numbers in the form of a
/// HIPAR firmwareRevision/string.
ViStatus _VI_FUNC rscmwins_revision_query(ViSession instrSession,
                                          ViChar    driverRev[],
                                          ViChar    instrRev[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, driverRev), 2, "Null address for Instrument Driver Revision");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, instrRev), 3, "Null address for Instrument Revision");

    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_SPECIFIC_DRIVER_REVISION, 0, 256, driverRev));
    checkErr(RsCore_GetAttributeViString(instrSession, NULL, RS_ATTR_INSTRUMENT_FIRMWARE_REVISION, 0, 256, instrRev));
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the number of the SCPI version, which is
/// HIFN  relevant for the instrument.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_SYSTEM_VERSION
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:VERSion?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the agesa_init or
/// HIPAR instrSession/agesa_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR length/Sets the allocated memory length of System Version string.
/// HIPAR systemVersion/Returns the number of the SCPI version, which is relevant for the
/// HIPAR systemVersion/instrument.
ViStatus _VI_FUNC rscmwins_SystemVersion(ViSession instrSession,
                                         ViInt32   length,
                                         ViChar    systemVersion[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwins_GetAttributeViString(instrSession, "", RSCMWINS_ATTR_SYSTEM_VERSION,
            length, systemVersion),
            2, "System Version");
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads an error code and a message from the instrument's
/// HIFN  error queue.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_SYSTEM_ERROR
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:ERRor?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Returns the error code read from the instrument's error queue.
/// HIPAR errorMessage/Returns the error message string read from the instrument's error
/// HIPAR errorMessage/message queue.
ViStatus _VI_FUNC rscmwins_error_query(ViSession instrSession,
                                       ViInt32*  errorCode,
                                       ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error Code");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_ErrorQueryAll(instrSession, errorCode, 1024, errorMessage));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads all system messages and returns a list of comma
/// HIFN  separated strings. Each string corresponds to an entry in the table
/// HIFN  SYSTEM MESSAGES.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_SYSTEM_ALL_ERRORS
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:ERRor:ALL?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR bufferSize/Sets the size of Errors string.
/// HIPAR errors/Returns the error message string read from the instrument's error
/// HIPAR errors/message queue.
ViStatus _VI_FUNC rscmwins_ErrorList(ViSession instrSession,
                                     ViInt32   bufferSize,
                                     ViChar    errors[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidViInt32RangeMin(instrSession, bufferSize, 0), 2, "Buffer Size");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errors), 3, "Null address for Errors");

    checkErr(rscmwins_GetAttributeViString( instrSession, "", RSCMWINS_ATTR_SYSTEM_ALL_ERRORS,
        bufferSize, errors));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function converts a status code returned by an instrument driver
/// HIFN  function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR errorCode/Pass the Status parameter that is returned from any of the instrument
/// HIPAR errorCode/driver functions.
/// HIPAR errorMessage/Returns the user-readable message string that corresponds to the
/// HIPAR errorMessage/status code you specify.
ViStatus _VI_FUNC rscmwins_error_message(ViSession instrSession,
                                         ViStatus  errorCode,
                                         ViChar    errorMessage[])
{
    ViStatus error = VI_SUCCESS;
    static   RsCoreStringValueTable errorTable =
             {
             RSCMWINS_ERROR_CODES_AND_MSGS,
    {0,      NULL}
             };

    (void)(RsCore_LockSession(instrSession));

    // all VISA and RS error codes are handled as well as codes in the table
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorMessage), 3, "Null address for Error Message");

    checkErr(RsCore_GetSpecificDriverStatusDesc(instrSession, errorCode, errorMessage, 256, errorTable));

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function converts a reliability code returned by an instrument
/// HIFN  driver function into a user-readable string.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR reliabilityCode/Pass the Reliability value that is returned from any of the instrument
/// HIPAR reliabilityCode/driver functions that return Reliability.
/// HIPAR reliabilityMessage/Returns the user-readable message string that corresponds to the
/// HIPAR reliabilityMessage/status code you specify.
ViStatus _VI_FUNC rscmwins_ReliabilityIndicator(ViSession instrSession,
                                                ViInt32   reliabilityCode,
                                                ViChar    reliabilityMessage[])
{
    ViStatus  error = VI_SUCCESS;
    ViInt16   cnt = 0;
    ViBoolean found = VI_FALSE;

    static RsCoreStringValueTable reliabilityTable =
    {
        {0, "OK"},
        {1, "Measurement Timeout"},
        {2, "Capture Buffer Overflow"},
        {3, "Overdriven"},
        {4, "Underdriven"},
        {6, "Trigger Timeout"},
        {7, "Acquisition Error"},
        {8, "Sync Error"},
        {9, "Uncal"},
        {15, "Reference Frequency Error"},
        {16, "RF Not Available"},
        {17, "RF Level not Settled"},
        {18, "RF Frequency not Settled"},
        {19, "Call not Established"},
        {20, "Call Type not Usable"},
        {21, "Call Lost"},
        {23, "Missing Option"},
        {24, "Invalid RF Setting"},
        {26, "Resource Conflict"},
        {27, "No Sensor Connected"},
        {28, "Unexpected Parameter Change"},
        {30, "File not Found"},
        {31, "No DTM reply"},
        {32, "ACL Disconnected"},
        {40, "ARB File CRC Error"},
        {42, "ARB Header Tag Invalid"},
        {43, "ARB Segment Overflow"},
        {44, "ARB File not Found"},
        {45, "ARB Memory Overflow"},
        {46, "ARB Sample Rate out of Range"},
        {47, "ARB Cycles out of Range"},
        {50, "Startup Error"},
        {51, "No Reply"},
        {52, "Connection Error"},
        {53, "Configuration Error"},
        {54, "Filesystem Error"},
        {93, "OCXO Oven Temperature too low"},
        {101, "Firmware Error"},
        {102, "Unidentified Error"},
        {103, "Parameter Error"},
        {104, "Not Functional"},
        {0, NULL}
    };

    (void)RsCore_LockSession(instrSession);

    for (cnt = 0; reliabilityTable[cnt].string; cnt++)
    {
        if (reliabilityTable[cnt].value == reliabilityCode)
        {
            sprintf(reliabilityMessage, "%s", reliabilityTable[cnt].string);
            if (reliabilityCode)
                error = RSCMWINS_ERROR_RELIABILITY_INDICATOR;
            found = VI_TRUE;
        }
    }

    if (!found)
    {
        sprintf(reliabilityMessage, "Unrecognized reliability indicator! Report problem to R&S support!");
        error = RS_WARN_UNKNOWN_INSTRUMENT_STATUS;
    }

    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function initiates an instrument reset.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     RSCMWINS_ATTR_SYSTEM_PRESET
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     SYSTem:PRESet
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_Preset(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_SetAttributeViString(instrSession, "", RSCMWINS_ATTR_SYSTEM_PRESET, NULL));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function clears status.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *CLS
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_ClearStatus(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ClearStatus(instrSession));
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the ID Query response string.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *IDN?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwlm_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR idQueryResponse/Returns the ID Query response string. The array should consist of at
/// HIPAR idQueryResponse/least 256 elements.
ViStatus _VI_FUNC rscmwins_IDQueryResponse(ViSession instrSession,
                                           ViChar    idQueryResponse[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_QueryViString(instrSession, "*IDN?", 256, idQueryResponse));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  Stops further command processing until all commands sent before *WAI
/// HIFN  have been executed.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *WAI
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwlm_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_ProcessAllPreviousCommands(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_WriteInstrData(instrSession, "*WAI"));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the OPC.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     *OPC?
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwlm_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR opc/Queries the OPC.
ViStatus _VI_FUNC rscmwins_QueryOPC(ViSession instrSession,
                                    ViInt32*  opc)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(rscmwins_QueryViInt32(instrSession, "*OPC?", opc));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function specifies the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwlm_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwins_SetVISATimeout(ViSession instrSession,
                                          ViUInt32  VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_SetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the minimum timeout value to use (in
/// HIFN  milliseconds) when accessing the device associated with the given
/// HIFN  session.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwwlm_init or
/// HIPAR instrSession/rscmwwlm_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR visaTimeout/This control gets the timeout value.
ViStatus _VI_FUNC rscmwins_GetVISATimeout(ViSession instrSession,
                                          ViUInt32* VISATimeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_GetVisaTimeout(instrSession, VISATimeout));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function switches ON/OFF various error checkings performed inside
/// HIFN  of the driver.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR optionChecking/This control switches driver option checking On or Off. If the option
/// HIPAR optionChecking/checking is ON, the driver checks the presence of the required
/// HIPAR optionChecking/instrument option before it sends the command to the instrument. Switch
/// HIPAR optionChecking/it OFF in special cases when you are sure that your instrument accepts
/// HIPAR optionChecking/the command. In any case the option checking on the instrument site is
/// HIPAR optionChecking/always active, and will cause an error if the command is not supported.
/// HIPAR rangeChecking/This control switches driver parameter range checking On or Off. If
/// HIPAR rangeChecking/the range checking is ON, the driver checks the range of the input
/// HIPAR rangeChecking/parameter before it sends the command to the instrument. Switch it OFF
/// HIPAR rangeChecking/in special cases when you are sure that your instrument accepts the
/// HIPAR rangeChecking/values outside the driver-restricted range. In any case, the parameter
/// HIPAR rangeChecking/range checking on the instrument site is always active, and will cause
/// HIPAR rangeChecking/an error if value is outside the instrument-accepted range.
/// HIPAR statusChecking/This control enables or disables instrument state checking (reading of
/// HIPAR statusChecking/the status byte and checking it for error). All driver functions are
/// HIPAR statusChecking/using state checking. Switch this function to ON when debugging your
/// HIPAR statusChecking/application. For better bus throughput and instrument performance
/// HIPAR statusChecking/switch it OFF.
ViStatus _VI_FUNC rscmwins_ConfigureErrorChecking(ViSession instrSession,
                                                  ViBoolean optionChecking,
                                                  ViBoolean rangeChecking,
                                                  ViBoolean statusChecking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->optionChecking = optionChecking;
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_QUERY_INSTRUMENT_STATUS, 0, statusChecking));
    checkErr(RsCore_SetAttributeViBoolean(instrSession, "", RS_ATTR_RANGE_CHECK, 0, rangeChecking));

Error:
    return error;
}

/// HIFN  This function sets the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwins_SetTimeout(ViSession instrSession,
                                      ViInt32   timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwins_SetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function returns the OPC timeout.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.  The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR timeout/This control returns the timeout value.
ViStatus _VI_FUNC rscmwins_GetTimeout(ViSession instrSession,
                                      ViInt32*  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(rscmwins_GetAttributeViInt32(instrSession, "", RS_ATTR_OPC_TIMEOUT, timeout),
            3, "Timeout");

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIFN  automatically in CheckStatus() when Instrument Status Error is
/// HIFN  detected.
/// HIFN     Use the function rspwrmeter_GetError afterwards to get the detailed
/// HIFN  information about the instrument error(s).
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR autoSystErrQuery/If TRUE (default after init is FALSE), the driver calls SYST:ERR?
/// HIPAR autoSystErrQuery/automatically in CheckStatus() when Instrument Status Error is
/// HIPAR autoSystErrQuery/detected.
ViStatus _VI_FUNC rscmwins_ConfigureAutoSystemErrQuery(ViSession instrSession,
                                                       ViBoolean autoSystErrQuery)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->autoSystErrQuery = autoSystErrQuery;

Error:
    return error;
}


/// HIFN  Session-related mutex switch.
/// HIFN     If TRUE (default after init is FALSE), the driver sets a
/// HIFN  mutual-exclusion mechanism for every driver's hi-level function and
/// HIFN  attribute access.
/// HIFN     Set this value to TRUE if you plan to use one session in more than one
/// HIFN  thread.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function.The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR multiThreadLocking/Session-related mutex switch.
ViStatus _VI_FUNC rscmwins_ConfigureMultiThreadLocking(ViSession instrSession,
                                                       ViBoolean multiThreadLocking)
{
    ViStatus         error = VI_SUCCESS;
    RsCoreSessionPtr rsSession = NULL;

    checkErr(RsCore_GetRsSession(instrSession, &rsSession));
    rsSession->multiThreadLocking = multiThreadLocking;

Error:
    return error;
}


/// HIFN  This function retrieves and then clears the error information for the
/// HIFN  session or the current execution thread. One exception exists: If the
/// HIFN  BufferSize parameter is 0, the function does not clear the error
/// HIFN  information. By passing 0 for the buffer size, the caller can ascertain
/// HIFN  the buffer size required to get the entire error description string and
/// HIFN  then call the function again with a sufficiently large buffer.
/// HIFN
/// HIFN     If the user specifies a valid session for the InstrumentHandle
/// HIFN  parameter, Get Error retrieves and then clears the error information
/// HIFN  for the session. If the user passes NULL for the InstrumentHandle
/// HIFN  parameter, this function retrieves and then clears the error
/// HIFN  information for the current execution thread.  If the InstrumentHandle
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error. Normally, the error information describes the first error
/// HIFN  that occurred since the user last called rscmwins_GetError or
/// HIFN  rscmwins_ClearError.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR code/Returns the error code for the session or execution thread.
/// HIPAR buffersize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR buffersize/Description parameter.
/// HIPAR description/Returns the error description for the session or execution thread. If
/// HIPAR description/there is no description, the function returns an empty string.
ViStatus _VI_FUNC rscmwins_GetError(ViSession instrSession,
                                    ViStatus* errorCode,
                                    ViInt32   bufferSize,
                                    ViChar    description[])
{
    ViStatus error = VI_SUCCESS;

    // Lock Session - do not jump to the end, even with invalid instrSession the function continues further
    (void)(RsCore_LockSession(instrSession));

    // Test for nulls and acquire error data
    viCheckParm(RsCore_InvalidNullPointer(instrSession, errorCode), 2, "Null address for Error");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, description), 4, "Null address for Description");

    checkErr(RsCore_GetErrorCompleteDescription(instrSession, &rscmwins_error_message, errorCode, bufferSize, description)
    );

Error:  
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function clears the error code and error description for the
/// HIFN  session. If the user specifies a valid session for the InstrumentHandle
/// HIFN  parameter, this function clears the error information for the session.
/// HIFN  If the user passes NULL for the Vi parameter, this function clears
/// HIFN  the error information for the current execution thread. If the Vi
/// HIFN  parameter is an invalid session, the function does nothing and returns
/// HIFN  an error.
/// HIFN     The function clears the error code by setting it to VI_SUCCESS.  If
/// HIFN  the error description string is non-NULL, the function de-allocates the
/// HIFN  error description string and sets the address to NULL.
/// HIFN
/// HIFN     Maintaining the error information separately for each thread is useful
/// HIFN  if the user does not have a session handle to pass to the
/// HIFN  rscmwins_GetError function, which occurs when a call to rscmwins_init or
/// HIFN  rscmwins_InitWithOptions fails.
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_ClearError(ViSession instrSession)
{
    return RsCore_ClearErrorInfo(instrSession);
}


/// HIFN  This function is used to read data from the instrument and write it to
/// HIFN  a user specified file on the host computer.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FORMat:BASE[:DATA]
/// HIFN     MMEMory:DATA
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from instrument to
/// HIPAR source/control computer.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/instrument to control computer applies.
ViStatus _VI_FUNC rscmwins_ReadToFileFromInstrument(ViSession instrSession,
                                                    ViString  source,
                                                    ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA? '%s'", source);
    checkErr(RsCore_QueryBinaryDataBlockToFile(instrSession, cmd, destination, RS_VAL_TRUNCATE));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function is used to read data from the host computer and write it
/// HIFN  to a user specified file in the instrument.
/// HIFN
/// HIFN     Attribute(s):
/// HIFN     -
/// HIFN
/// HIFN     Remote-control command(s):
/// HIFN     FORMat:BASE[:DATA]
/// HIFN     MMEMory:DATA
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR source/This control selects file for the data transfer from control computer
/// HIPAR source/to instrument.
/// HIPAR destination/This control defines destination file to which the data transfer from
/// HIPAR destination/control computer to instrument applies.
ViStatus _VI_FUNC rscmwins_WriteFromFileToInstrument(ViSession instrSession,
                                                     ViString  source,
                                                     ViString  destination)
{
    ViStatus error = VI_SUCCESS;
    ViChar   cmd[RS_MAX_MESSAGE_BUF_SIZE];

    checkErr(RsCore_LockSession(instrSession));

    snprintf(cmd, RS_MAX_MESSAGE_BUF_SIZE, "MMEM:DATA '%s',", destination);
    checkErr(RsCore_WriteBinaryDataFromFile(instrSession, cmd, source));
    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function writes a user-specified string to the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
ViStatus _VI_FUNC rscmwins_WriteInstrData(ViSession instrSession,
                                          ViString  writeBuffer)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    error = RsCore_Write(instrSession, writeBuffer);

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function reads data from the instrument.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR numberofBytesToRead/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data that
/// HIPAR readBuffer/was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument and
/// HIPAR numBytesRead/stored in the Read Buffer.
ViStatus _VI_FUNC rscmwins_ReadInstrData(ViSession instrSession,
                                         ViInt32   numBytes,
                                         ViChar    rdBuf[],
                                         ViUInt32* bytesRead)
{
    ViStatus error = VI_SUCCESS;
    ViChar*  responseString = NULL;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ReadViStringUnknownLength(instrSession, &responseString));
    checkErr(RsCore_CopyToUserBufferAsciiData(rdBuf, numBytes, responseString));
    *bytesRead = (ViUInt32)strlen(responseString);

Error:
    if (responseString)
        free(responseString);

    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a command to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for an instrument operation that can
/// HIFN take a long time to complete, e.g. self-alignment.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
ViStatus _VI_FUNC rscmwins_WriteCommandWithOPCSync(ViSession instrSession,
                                                   ViString  writeBuffer,
                                                   ViUInt32  timeout)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_WriteWithOpc(instrSession, writeBuffer, timeout));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}

/// HIFN Sends a query to the instrument synchronised with OPC-polling.
/// HIFN Use this function to wait for a response that can take a long
/// HIFN time to complete e.g. self-alignment, calibration or self-test.
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/This control accepts the Instrument Handle returned by the Initialize
/// HIPAR instrSession/function to select the desired instrument driver session.
/// HIPAR writeBuffer/Pass the string to be written to the instrument.
/// HIPAR timeout/This control sets the timeout value.
/// HIPAR bufferSize/Pass the maximum number of bytes to read from the instruments.
/// HIPAR readBuffer/After this function executes, this parameter contains the data
/// HIPAR readBuffer/that was read from the instrument.
/// HIPAR numBytesRead/Returns the number of bytes actually read from the instrument
/// HIPAR numBytesRead/and stored in the Read Buffer.
ViStatus _VI_FUNC rscmwins_QueryWithOPCSync(ViSession instrSession,
                                            ViString  writeBuffer,
                                            ViUInt32  timeout,
                                            ViInt32   bufferSize,
                                            ViChar    readBuffer[],
                                            ViUInt32* numBytesRead)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, writeBuffer, 1, -1),
        2, "Write Buffer (string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, readBuffer),
        5, "Read Buffer");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBufferWithOpc(instrSession, writeBuffer, timeout, bufferSize, readBuffer, (ViInt32*)numBytesRead));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ViBoolean value.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rscmwins_QueryViBoolean(ViSession  instrSession,
                                          ViString   cmd,
                                          ViBoolean* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViBoolean(instrSession, cmd, value));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ViInt32 value.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViInt32 value
ViStatus _VI_FUNC rscmwins_QueryViInt32(ViSession instrSession,
                                        ViString  cmd,
                                        ViInt32*  value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViInt32(instrSession, cmd, value));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ViReal64 value.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR value/Returns the ViReal64 value
ViStatus _VI_FUNC rscmwins_QueryViReal64(ViSession instrSession,
                                         ViString  cmd,
                                         ViReal64* value)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViReal64(instrSession, cmd, value));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function queries the ViString value.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the rscmwins_init or
/// HIPAR instrSession/rscmwins_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
/// HIPAR command/The query to be sent to the instrument.
/// HIPAR bufferSize/Pass the number of bytes in the ViChar array you specify for the
/// HIPAR bufferSize/Value parameter.
/// HIPAR value/Returns the ViBoolean value
ViStatus _VI_FUNC rscmwins_QueryViString(ViSession instrSession,
                                         ViString  cmd,
                                         ViInt32   bufferSize,
                                         ViChar    _VI_FAR value[])
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    viCheckParm(RsCore_InvalidStringLength(instrSession, cmd, 1, -1),
        2, "Command (null string length)");
    viCheckParm(RsCore_InvalidNullPointer(instrSession, value), 3, "Value");

    checkErr(RsCore_QueryViStringUnknownLengthToUserBuffer(instrSession, cmd, bufferSize, value, NULL));

    checkErr(rscmwins_CheckStatus(instrSession));

Error:
    (void)RsCore_UnlockSession(instrSession);
    return error;
}


/// HIFN  This function performs the following operations:
/// HIFN
/// HIFN     - Closes the instrument I/O session.
/// HIFN
/// HIFN     - Destroys the instrument driver session and all of its attributes.
/// HIFN
/// HIFN     - Deallocates any memory resources the driver uses.
/// HIFN
/// HIFN     Notes:
/// HIFN
/// HIFN     (1) You must unlock the session before calling rscmwins_close.
/// HIFN
/// HIFN     (2) After calling rscmwins_close, you cannot use the instrument driver
/// HIFN  again until you call rscmwins_init or rscmwins_InitWithOptions.
/// HIFN
/// HIFN
/// HIRET Returns the status code of this operation.
/// HIPAR instrSession/The ViSession handle that you obtain from the RSCMWINS_init or
/// HIPAR instrSession/RSCMWINS_InitWithOptions function. The handle identifies a particular
/// HIPAR instrSession/instrument session.
ViStatus _VI_FUNC rscmwins_close(ViSession instrSession)
{
    ViStatus error = VI_SUCCESS;

    checkErr(RsCore_LockSession(instrSession));

    checkErr(RsCore_ViClose(instrSession));

Error:
    (void)RsCore_Dispose(instrSession);
    (void)RsCore_UnlockSession(instrSession);
    return error;
}
